@@ -51,7 +51,5 @@ fi
 # Jaxe doit etre lance dans son repertoire
 cd $JAXE_HOME
 
-classpath="lib/Jaxe.jar:lib/jazzy.jar:lib/log4j-1.2.14.jar:lib/fop_and_libs.jar:lib/jing.jar:config"
-
-execution="exec \"$JAVACMD\" -Xmx256m -classpath $classpath jaxe/Jaxe $parametres"
+execution="exec \"$JAVACMD\" -Xmx256m -jar lib/Jaxe.jar $parametres"
 eval $execution
