@@ -38,23 +38,11 @@ public class ActionInsertionBalise extends TextAction {
      */
     private static final Logger LOG = Logger.getLogger(ActionInsertionBalise.class);
     
-    static String newline = Jaxe.newline;
     JaxeDocument doc;
     Element refElement;
     String nomNoeud;
     String typeNoeud;
     
-    @Deprecated
-    public ActionInsertionBalise(final JaxeDocument doc, final Element balise) {
-        super(doc.cfg.titreBalise(balise));
-        this.doc = doc;
-        this.nomNoeud = doc.cfg.nomBalise(balise);
-        this.refElement = doc.cfg.referenceElement(nomNoeud);
-        this.typeNoeud = doc.cfg.noeudtypeBalise(balise);
-        if ("".equals(typeNoeud))
-            typeNoeud = "element";
-    }
-    
     public ActionInsertionBalise(final JaxeDocument doc, final String titre, final Element refElement,
             final String nomNoeud, final String typeNoeud) {
         super(titre);
@@ -90,8 +78,7 @@ public class ActionInsertionBalise extends TextAction {
                 }
                 if (end - start > 0 && doc.rootJE != null) {
                     JaxeElement parent2 = doc.rootJE.elementA(end);
-                    if (parent2 != null && parent2.debut.getOffset() == end &&
-                            !(parent2 instanceof JESwing))
+                    if (parent2 != null && parent2.debut.getOffset() == end && !(parent2 instanceof JESwing))
                         parent2 = parent2.getParent() ;
                     if (parent2 != null && (parent2.noeud.getNodeType() == Node.TEXT_NODE || (parent2 instanceof JEStyle && !typeAffichage.equals("style")))) {
                         if (end > parent2.debut.getOffset() && end <= parent2.fin.getOffset()) {
@@ -100,7 +87,7 @@ public class ActionInsertionBalise extends TextAction {
                         }
                     }
                 }
-                if (parent == null && doc.rootJE != null) {
+                if (parent == null && doc.rootJE != null && !"commentaire".equals(typeNoeud)) {
                     doc.getGestionErreurs().pasSousLaRacine(refElement);
                     return;
                 }
@@ -132,27 +119,9 @@ public class ActionInsertionBalise extends TextAction {
                 }
                 if (typeAffichage.equals("style")) {
                     if (end - start > 0) {
-                        final JEStyle newje = JEStyle.nouveau(doc, start, end, refElement);
-                        if (newje != null) { // pas null si ajout de balise sur la slection
-                            final String texte = doc.textPane.getText(start, end-start);
-                            doc.textPane.debutEditionSpeciale(JaxeResourceBundle.getRB().getString("style.Style"), false);
-                            JaxeUndoableEdit jedit = new JaxeUndoableEdit(JaxeUndoableEdit.SUPPRIMER,
-                                doc, texte, start);
-                            jedit.doit();
-                            jedit = new JaxeUndoableEdit(JaxeUndoableEdit.AJOUTER, newje);
-                            jedit.doit();
-                            doc.textPane.finEditionSpeciale();
-                        }
-                        if (start != end) {
-                            JaxeElement jeBeg = doc.elementA(start);
-                            while (!((jeBeg instanceof JEStyle) ||(jeBeg instanceof JETexte))) {
-                                start++;
-                                end++;
-                                jeBeg = doc.elementA(start);
-                            }
-                            doc.textPane.setSelectionStart(start);
-                            doc.textPane.setSelectionEnd(end);
-                        }
+                        JEStyle.appliquer(doc, start, end, refElement);
+                        doc.textPane.setSelectionStart(start);
+                        doc.textPane.setSelectionEnd(end);
                     }
                 } else {
                     boolean editionSpeciale = false;
@@ -262,9 +231,9 @@ public class ActionInsertionBalise extends TextAction {
                                 }
                             }
                         }
-                        if (doc.rootJE == null) {
+                        if (doc.rootJE == null && "element".equals(typeNoeud)) {
                             doc.cfg.ajouterAttributsEspaces((Element)newel);
-                            doc.DOMdoc.appendChild(newel);
+                            JaxeElement.insererDOM(doc, pos, newel);
                             doc.textPane.debutIgnorerEdition();
                             newje.creer(pos, newel);
                             doc.textPane.finIgnorerEdition();
@@ -327,11 +296,6 @@ public class ActionInsertionBalise extends TextAction {
         }
     }
     
-    @Deprecated
-    public Element getDefbalise() {
-        return(doc.cfg.getBaliseDef(nomNoeud));
-    }
-    
     public Element getRefElement() {
         return(refElement);
     }
