@@ -49,13 +49,15 @@ public class Balise extends JComponent implements MouseListener, MouseMotionList
     static protected ImageIcon iconeInvalide = new ImageIcon(ImageKeeper.loadImage("images/invalide.gif", true));
     
     static Font police = (Font) UIManager.getDefaults().get("Button.font");
-    final static Color jauneLeger = new Color(250, 250, 180);
-    final static Color rougeFonce = new Color(150, 0, 0);
-    final static Color orange = new Color(255, 210, 170);
-    final static Color bleuClair = new Color(220, 240, 250);
+    final static Color jauneLeger = new Color(250, 250, 190);
+    final static Color rougeFonce = new Color(160, 20, 20);
+    final static Color orange = new Color(255, 220, 170);
+    final static Color bleuClair = new Color(225, 245, 255);
     final static Color violet = new Color(220, 210, 255);
     final static Color vertClair = new Color(210, 250, 220);
     final static Color vertJaune = new Color(230, 250, 180);
+    final static Color vertFonce = new Color(20, 80, 20);
+    final static Color bleuFonce = new Color(20, 20, 160);
     
     static Color[][] couleurs = { { jauneLeger, rougeFonce, orange },
             { bleuClair, rougeFonce, violet }, { vertClair, rougeFonce, vertJaune } };
@@ -385,7 +387,7 @@ public class Balise extends JComponent implements MouseListener, MouseMotionList
         d.width += fm.stringWidth(texte);
         if (attributsVisibles && typeBalise != FIN && je.refElement != null) {
             final NamedNodeMap listeAttributs = je.noeud.getAttributes();
-            final Config cfg = je.doc.cfg;
+            final Config cfg = je.doc.cfg.getRefConf(je.refElement);
             final ArrayList<Element> latt = cfg.listeAttributs(je.refElement);
             for (Element refatt : latt) {
                 final Node natt = listeAttributs.getNamedItemNS(cfg.espaceAttribut(refatt),
@@ -406,8 +408,13 @@ public class Balise extends JComponent implements MouseListener, MouseMotionList
 
     @Override
     public Dimension getMaximumSize() {
-        if (division)
-            return (super.getMaximumSize());
+        if (division) {
+            if (System.getProperty("os.name").startsWith("Mac OS")) { // curseur pas visible  droite sur la JVM d'Apple avec getMaximumSize()
+                final Dimension tps = je.doc.textPane.getSize();
+                return (new Dimension(tps.width - 20*je.indentations() - 2, 21));
+            } else
+                return (super.getMaximumSize());
+        }
         return (getPreferredSize());
     }
 
@@ -450,7 +457,7 @@ public class Balise extends JComponent implements MouseListener, MouseMotionList
         final Graphics2D g2d = (Graphics2D)g;
         final Object antialiasing = g2d.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
         g2d.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
-//            g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
+        g2d.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
         final Insets bords = getInsets();
         final Dimension dim = getSize();
         final Polygon poly = new Polygon();
@@ -555,17 +562,15 @@ public class Balise extends JComponent implements MouseListener, MouseMotionList
             if (typeBalise == DEBUT)
                 pxmax -= fleche;
             final int py = dim.height - fm.getDescent() - 2 - bords.bottom;
-            final Color vertFonce = new Color(0, 70, 0);
-            final Color bleuFonce = new Color(0, 0, 150);
             final NamedNodeMap listeAttributs = je.noeud.getAttributes();
-            final ArrayList<Element> latt = je.doc.cfg.listeAttributs(je.refElement);
+            final Config cfg = je.doc.cfg.getRefConf(je.refElement);
+            final ArrayList<Element> latt = cfg.listeAttributs(je.refElement);
             for (Element refatt : latt) {
-                final Node natt = listeAttributs.getNamedItemNS(je.doc.cfg.espaceAttribut(refatt),
-                    je.doc.cfg.nomAttribut(refatt));
+                final Node natt = listeAttributs.getNamedItemNS(cfg.espaceAttribut(refatt), cfg.nomAttribut(refatt));
                 if (natt != null) {
                     px += fm.stringWidth(" ");
-                    String titreAtt = je.doc.cfg.titreAttribut(je.refElement, refatt);
-                    String titreVal = je.doc.cfg.titreValeurAttribut(je.refElement, refatt, natt.getNodeValue());
+                    String titreAtt = cfg.titreAttribut(je.refElement, refatt);
+                    String titreVal = cfg.titreValeurAttribut(je.refElement, refatt, natt.getNodeValue());
                     final String texteAtt = titreAtt + "=" + titreVal;
                     if (px + fm.stringWidth(texteAtt) > pxmax) {
                         g.drawString("...", px, py);
