@@ -43,6 +43,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
     String[] noms;
     String[] espaces;
     String[] defauts;
+    boolean[] obligatoire;
     boolean avecdoc = false;
     JLabel[] tbval;
     boolean valide = false;
@@ -67,6 +68,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
         espaces = new String[natt];
         champs = new JComponent[natt];
         defauts = new String[natt];
+        obligatoire = new boolean[natt];
         final String[] documentation = new String[natt];
         tbval = new JLabel[natt];
         for (int i=0; i<natt; i++) {
@@ -74,6 +76,11 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
             noms[i] = conf.nomAttribut(att);
             titres[i] = conf.titreAttribut(refElement, att);
             espaces[i] = conf.espaceAttribut(att);
+            if (espaces[i] != null) {
+                final String prefixe = conf.prefixeAttribut(el, att);
+                if (prefixe != null)
+                    noms[i] = prefixe + ":" + noms[i];
+            }
             documentation[i] = conf.documentationAttribut(refElement, att);
             if (documentation[i] != null) {
                 avecdoc = true;
@@ -81,6 +88,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
             }
             String elval = el.getAttribute(noms[i]);
             defauts[i] = conf.valeurParDefaut(att);
+            obligatoire[i] = conf.attributObligatoire(refElement, att);
             if ("".equals(elval) && defauts[i] != null && el.getAttributeNode(noms[i]) == null)
                 elval = defauts[i];
             final ArrayList<String> lval = conf.listeValeursAttribut(att);
@@ -137,7 +145,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
                 }
             }
             final boolean attvalide;
-            if ("".equals(elval) && !conf.estObligatoire(att))
+            if ("".equals(elval) && !obligatoire[i] && defauts[i] == null)
                 attvalide = true;
             else
                 attvalide = conf.attributValide(att, elval);
@@ -181,8 +189,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
         final JPanel qpane = new JPanel(new GridLayout(titres.length, 1));
         for (int i=0; i<titres.length; i++) {
             final JLabel label = new JLabel(titres[i]);
-            final Element att = latt.get(i);
-            if (conf.estObligatoire(att))
+            if (obligatoire[i])
                 label.setForeground(new Color(150, 0, 0)); // rouge fonc
             else
                 label.setForeground(new Color(0, 100, 0)); // vert fonc
@@ -346,13 +353,12 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
         for (int i=0; i<champs.length; i++) {
             if (champs[i] == src) {
                 final int index = ((JComboBox)src).getSelectedIndex();
-                final Element att = latt.get(i);
                 final int indexval;
                 if (defauts[i] == null)
                     indexval = index - 1;
                 else
                     indexval = index;
-                final boolean attvalide = !(indexval < 0 && conf.estObligatoire(att));
+                final boolean attvalide = !(indexval < 0 && obligatoire[i]);
                 final ImageIcon icone;
                 if (!attvalide)
                     icone = iconeInvalide;
@@ -370,7 +376,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
                 final String valeur = ((JTextComponent)champs[i]).getText();
                 final boolean attvalide;
                 final Element att = latt.get(i);
-                if ("".equals(valeur) && !conf.estObligatoire(att))
+                if ("".equals(valeur) && !obligatoire[i] && defauts[i] == null)
                     attvalide = true;
                 else
                     attvalide = conf.attributValide(att, valeur);
@@ -426,10 +432,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
         try {
             for (int i=0; i<rep.length; i++)
                 if (rep[i] != null) {
-                    if ("".equals(rep[i]) && !"".equals(el.getAttribute(noms[i])) &&
-                            !el.getAttribute(noms[i]).equals(defauts[i]))
-                        el.removeAttribute(noms[i]);
-                    else if (rep[i].equals(defauts[i]))
+                    if (!obligatoire[i] && (("".equals(rep[i]) && defauts[i] == null) || rep[i].equals(defauts[i])))
                         el.removeAttribute(noms[i]);
                     else if (!"".equals(rep[i]) || defauts[i] != null)
                         el.setAttributeNS(espaces[i], noms[i], rep[i]);
@@ -444,9 +447,7 @@ public class DialogueAttributs extends JDialog implements ActionListener, KeyLis
     protected boolean checkAtt() {
         final String[] rep = lireReponses();
         for (int i=0; i<latt.size(); i++) {
-            final Element att = latt.get(i);
-            if (conf.estObligatoire(att) && (rep[i] == null ||
-                "".equals(rep[i]))) {
+            if (obligatoire[i] && (rep[i] == null || "".equals(rep[i]))) {
                 getToolkit().beep();
                 if (champs[i] instanceof JTextComponent)
                     ((JTextComponent)champs[i]).selectAll();
