@@ -67,30 +67,31 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
 
     private static final ResourceBundle rb = JaxeResourceBundle.getRB();
 
+    static String texteRecherche = null;
+    static String texteXpathRecherche = null;
+    
     private final JTextPane textPane;
     private final StyledDocument doc;
     private final JPanel bpane, cpane;
-    JTextField tfRechercher;
+    private JTextField tfRechercher;
     private final JTextField tfRemplacer;
     private final JLabel textRech = new JLabel(rb.getString("rechercher.Rechercher"));
-    private final JLabel textRemp = new JLabel(rb.getString("rechercher.RemplacerPar")); 
+    private final JLabel textRemp = new JLabel(rb.getString("rechercher.RemplacerPar"));
     private final JButton bXpath, btout, bremplacer, bremplrech;
-    JRadioButton bfichier, bsel;
-    private final JCheckBox typeXpath, chkmaj; 
+    private JRadioButton bfichier, bsel;
+    private final JCheckBox typeXpath, chkmaj;
     private boolean dansSelection = false;
     private boolean ignorerCasse = false;
-    public boolean RechXpath = false;
-    static String texteRecherche = null;
-    static String texteXpathRecherche = null;   
-    public JaxeDocument Jdoc;
+    private boolean rechXpath = false;
+    private JaxeDocument jdoc;
 
-    public DialogueRechercher(final JaxeDocument Jdoc, final JTextPane textPane) {
+    public DialogueRechercher(final JaxeDocument jdoc, final JTextPane textPane) {
         super((JFrame)textPane.getTopLevelAncestor(), rb.getString("rechercher.Rechercher"), false);
         this.textPane = textPane;
-        this.Jdoc = Jdoc;
+        this.jdoc = jdoc;
         doc = textPane.getStyledDocument();
         if (textPane.getSelectionEnd() != textPane.getSelectionStart())
-            dansSelection = true; 
+            dansSelection = true;
         
         cpane = new JPanel();
         cpane.setLayout(new BoxLayout(cpane, BoxLayout.Y_AXIS));
@@ -111,7 +112,7 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         tfRemplacer = new JTextField("", 40);
         tfpane.add(tfRemplacer);
         
-        bpane = new JPanel(new GridLayout(2,1)); 
+        bpane = new JPanel(new GridLayout(2,1));
         bpane.setPreferredSize(new Dimension(85,50));
         bXpath = new JButton(rb.getString("rechercher.Xpath"));
         bXpath.setActionCommand("xpath");
@@ -194,13 +195,13 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
     public void actionPerformed(final ActionEvent e) {
         final String cmd = e.getActionCommand();
         if ("suivant".equals(cmd)){
-            if (!RechXpath)
+            if (!rechXpath)
                 suivant();
             else 
                 suivantXpath();
         }
         else if ("precedent".equals(cmd)){
-            if (!RechXpath)
+            if (!rechXpath)
                 precedent();
             else 
                 precedentXpath();
@@ -215,13 +216,14 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
             dansSelection = false;
         else if ("selection".equals(cmd))
             dansSelection = true;
-        else if ("xpath".equals(cmd)){
-            this.setVisible(false);
-            if (Jdoc.cfg!=null) {
-                final DialogueXpath Xpa = new DialogueXpath(this,Jdoc);
+        else if ("xpath".equals(cmd)) {
+            setVisible(false);
+            if (jdoc.cfg != null) {
+                new DialogueXpath(this, jdoc);
             } else {
-                JOptionPane.showMessageDialog(this,rb.getString("rechercher.ErrConfig"),rb.getString("rechercher.ExprXpath"), JOptionPane.INFORMATION_MESSAGE);
-                this.setVisible(true);
+                JOptionPane.showMessageDialog(this, rb.getString("rechercher.ErrConfig"),
+                    rb.getString("rechercher.ExprXpath"), JOptionPane.INFORMATION_MESSAGE);
+                setVisible(true);
             }
         }
     }
@@ -231,8 +233,8 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
             ignorerCasse = (e.getStateChange() == ItemEvent.SELECTED);
         }
         if (e.getSource() == typeXpath) {
-            RechXpath = (e.getStateChange() == ItemEvent.SELECTED);
-            if (RechXpath) {
+            rechXpath = (e.getStateChange() == ItemEvent.SELECTED);
+            if (rechXpath) {
                 enableButton(false);
                 bpane.add(bXpath);
             } else {
@@ -241,7 +243,7 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
                 bpane.remove(bXpath);
             }
             cpane.updateUI();
-            pack(); 
+            pack();
         }
     }
     
@@ -253,11 +255,10 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         bfichier.setEnabled(b);
         bsel.setEnabled(b);
         chkmaj.setEnabled(b);
-        if (b) {
+        if (b)
             tfRechercher.setText(texteRecherche);
-        } else {
+        else
             tfRechercher.setText(texteXpathRecherche);
-        }
     }
     
     public void rechercher() {
@@ -291,8 +292,13 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         suivantXpath(textPane.getCaretPosition());
     }
     
+    private void changerTitre(final int numero, final int total) {
+        String titre = rb.getString("rechercher.nbXpath").replace("NUMRESULT", Integer.toString(numero));
+        titre = titre.replace("NBTOTAL", Integer.toString(total));
+        setTitle(rb.getString("rechercher.Rechercher") + " " + titre);
+    }
+    
     public void suivantXpath(final int rech_pos) {
-        String result = "";
         int goTo = rech_pos;
         texteXpathRecherche = tfRechercher.getText();
         if (texteXpathRecherche == null || texteXpathRecherche.length() == 0)
@@ -300,23 +306,19 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         final NodeList nodeOkXpath = getXpathNodeList(tfRechercher.getText());
         if (nodeOkXpath != null) {
             for (int i=0; i<nodeOkXpath.getLength(); i++) {
-                final JaxeElement je = Jdoc.getElementForNode(nodeOkXpath.item(i));
+                final JaxeElement je = jdoc.getElementForNode(nodeOkXpath.item(i));
                 if (je != null) {
                     final int debNode = je.debut.getOffset();
                     if (debNode > rech_pos) {
                         goTo = debNode;
-                        result = rb.getString("rechercher.nbXpath").replace("NUMRESULT", Integer.toString(i+1));
-                        result = result.replace("NBTOTAL", Integer.toString(nodeOkXpath.getLength()));
-                        setTitle(rb.getString("rechercher.Rechercher") + " " + result );
+                        changerTitre(i+1, nodeOkXpath.getLength());
                         break;
                     }
                 }
             }
             if ((nodeOkXpath.getLength() > 0) && (goTo == rech_pos)) {
-                result = rb.getString("rechercher.nbXpath").replace("NUMRESULT", "1");
-                result = result.replace("NBTOTAL", Integer.toString(nodeOkXpath.getLength()));
-                setTitle(rb.getString("rechercher.Rechercher") + " " + result );
-                final JaxeElement je = Jdoc.getElementForNode(nodeOkXpath.item(0));
+                changerTitre(1, nodeOkXpath.getLength());
+                final JaxeElement je = jdoc.getElementForNode(nodeOkXpath.item(0));
                 if (je != null)
                     goTo = je.debut.getOffset();
             }
@@ -331,7 +333,7 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
                 textPane.setCaretPosition(goTo);
             } else {
                 getToolkit().beep();
-                setTitle(rb.getString("rechercher.Rechercher"));                
+                setTitle(rb.getString("rechercher.Rechercher"));
             }
         }
         toFront();
@@ -342,7 +344,6 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
     }
     
     public void precedentXpath(final int rech_pos) {
-        String result = "";
         int goTo = rech_pos;
         texteXpathRecherche = tfRechercher.getText();
         if (texteXpathRecherche == null || texteXpathRecherche.length() == 0)
@@ -350,25 +351,21 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         final NodeList nodeOkXpath = getXpathNodeList(tfRechercher.getText());
         if (nodeOkXpath != null) {
             for (int i=(nodeOkXpath.getLength()-1); i>=0; i--) {
-                final JaxeElement je = Jdoc.getElementForNode(nodeOkXpath.item(i));
+                final JaxeElement je = jdoc.getElementForNode(nodeOkXpath.item(i));
                 if (je != null) {
                     final int debNode = je.debut.getOffset();
                     if (debNode < rech_pos) {
                         goTo = debNode;
-                        result = rb.getString("rechercher.nbXpath").replace("NUMRESULT", Integer.toString(i+1));
-                        result = result.replace("NBTOTAL", Integer.toString(nodeOkXpath.getLength()));
-                        setTitle(rb.getString("rechercher.Rechercher") + " " + result );
+                        changerTitre(i+1, nodeOkXpath.getLength());
                         break;
                     }
                 }
             }
             if ((nodeOkXpath.getLength() > 0) && (goTo == rech_pos)) {
-                result = rb.getString("rechercher.nbXpath").replace("NUMRESULT", Integer.toString(nodeOkXpath.getLength()));
-                result = result.replace("NBTOTAL", Integer.toString(nodeOkXpath.getLength()));
-                setTitle(rb.getString("rechercher.Rechercher") + " " + result ); 
-                final JaxeElement je = Jdoc.getElementForNode(nodeOkXpath.item(nodeOkXpath.getLength()-1));
+                changerTitre(1, nodeOkXpath.getLength());
+                final JaxeElement je = jdoc.getElementForNode(nodeOkXpath.item(nodeOkXpath.getLength()-1));
                 if (je != null)
-                    goTo = je.debut.getOffset();           
+                    goTo = je.debut.getOffset();
             }
             if (nodeOkXpath.getLength() != 0) {
                 try {
@@ -391,25 +388,25 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
             final XPath xpath = XPathFactory.newInstance().newXPath();
             xpath.setNamespaceContext(new NamespaceContext() {
                 public String getNamespaceURI(String prefix) {
-                    final ArrayList<String> espaces = Jdoc.cfg.listeEspaces();
+                    final ArrayList<String> espaces = jdoc.cfg.listeEspaces();
                     for (final String espace : espaces) {
                         if (!"".equals(espace)) {
-                            if (prefix != null && prefix.equals(Jdoc.cfg.prefixeEspace(espace)))
+                            if (prefix != null && prefix.equals(jdoc.cfg.prefixeEspace(espace)))
                                 return(espace);
                         }
                     }
-                    return(Jdoc.DOMdoc.lookupNamespaceURI(prefix));
+                    return(jdoc.DOMdoc.lookupNamespaceURI(prefix));
                 }
                 public String getPrefix(String namespaceURI) {
-                    return(Jdoc.cfg.prefixeEspace(namespaceURI));
+                    return(jdoc.cfg.prefixeEspace(namespaceURI));
                 }
                 public Iterator getPrefixes(String namespaceURI) {
                     final ArrayList<String> al = new ArrayList<String>();
-                    al.add(Jdoc.cfg.prefixeEspace(namespaceURI));
+                    al.add(jdoc.cfg.prefixeEspace(namespaceURI));
                     return(al.iterator());
                 }
             });
-            return( (NodeList)xpath.evaluate(nodePath, Jdoc.DOMdoc, XPathConstants.NODESET) );
+            return( (NodeList)xpath.evaluate(nodePath, jdoc.DOMdoc, XPathConstants.NODESET) );
         } catch (final XPathExpressionException e) {
             String message = e.getMessage();
             if (message == null && e.getCause() != null)
@@ -420,11 +417,15 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
     }
     
     public void suivant() {
-        suivant (textPane.getSelectionStart());
+        suivant(textPane.getSelectionStart());
     }
     
     // recherche suivant a partir de rech_pos
     public void suivant(int rech_pos) {
+        if (rechXpath) {
+            suivantXpath(rech_pos);
+            return;
+        }
         texteRecherche = tfRechercher.getText();
         if (texteRecherche == null || texteRecherche.length() == 0)
             return;
@@ -436,37 +437,36 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         if ((rech_pos + len) > doc.getLength()) 
             rech_pos = 0;
         int ind = -1;
-        String text;
         try {
-            if ((textPane.getSelectionEnd()-textPane.getSelectionStart()) == len)
-                if ((!ignorerCasse && doc.getText(rech_pos,len).equals(texteRecherche)) ||
-                        (ignorerCasse && doc.getText(rech_pos,len).equalsIgnoreCase(texteRecherche))) 
-                rech_pos++;
+            if ((textPane.getSelectionEnd() - textPane.getSelectionStart()) == len) {
+                final String texte = doc.getText(rech_pos, len);
+                if ((!ignorerCasse && texte.equals(texteRecherche)) ||
+                        (ignorerCasse && texte.equalsIgnoreCase(texteRecherche))) 
+                    rech_pos++;
+            }
             for (int i=rech_pos; i<doc.getLength()-len; i++) {
-                if (includeComponent(i,len)==(-1)) {
-                    text = doc.getText(i, len);
-                    if ((!ignorerCasse && text.equals(texteRecherche)) ||
-                            (ignorerCasse && text.equalsIgnoreCase(texteRecherche))) {
+                if (includeComponent(i, len) == -1) {
+                    final String texte = doc.getText(i, len);
+                    if ((!ignorerCasse && texte.equals(texteRecherche)) ||
+                            (ignorerCasse && texte.equalsIgnoreCase(texteRecherche))) {
                         ind = i;
                         break;
                     }
-                }
-                else i=includeComponent(i,len);      
+                } else
+                    i = includeComponent(i, len);
             }
         } catch (final BadLocationException ex) {
             LOG.error("suivant(int) - BadLocationException", ex);
             return;
         }
         if (ind != -1) {
+            textPane.getCaret().setSelectionVisible(true); // for Windows (see bug 4273908)
             textPane.setCaretPosition(ind);
-            textPane.moveCaretPosition(ind+len);
-        }
-        else if (rech_pos != 0) {
-                suivant(0);
-             }
-             else {
-                getToolkit().beep();
-             }
+            textPane.moveCaretPosition(ind + len);
+        } else if (rech_pos != 0)
+            suivant(0);
+        else
+            getToolkit().beep();
         toFront();
     }
 
@@ -489,71 +489,67 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
             rech_pos = doc.getLength() - len;
         if (rech_pos < 0)
             rech_pos = 0;
-        String text;
         try {
-            if ((textPane.getSelectionEnd()-textPane.getSelectionStart()) == len)
-                if ((!ignorerCasse && doc.getText(rech_pos,len).equals(texteRecherche)) ||
-                        (ignorerCasse && doc.getText(rech_pos,len).equalsIgnoreCase(texteRecherche))) 
-                rech_pos--;
+            if ((textPane.getSelectionEnd() - textPane.getSelectionStart()) == len) {
+                final String texte = doc.getText(rech_pos, len);
+                if ((!ignorerCasse && texte.equals(texteRecherche)) ||
+                        (ignorerCasse && texte.equalsIgnoreCase(texteRecherche))) 
+                    rech_pos--;
+            }
             for (int i=rech_pos; i>=0; i--) {
-                if (includeComponent(i,len)==(-1)){ 
-                    text = doc.getText(i, len);
-                    if ((!ignorerCasse && text.equals(texteRecherche)) ||
-                            (ignorerCasse && text.equalsIgnoreCase(texteRecherche))) {
+                if (includeComponent(i, len) == -1) {
+                    final String texte = doc.getText(i, len);
+                    if ((!ignorerCasse && texte.equals(texteRecherche)) ||
+                            (ignorerCasse && texte.equalsIgnoreCase(texteRecherche))) {
                         ind = i;
                         break;
                     }
-                }
-                else i=includeComponent(i,len)-len+1;    
+                } else
+                    i = includeComponent(i, len) - len + 1;
             }
         } catch (final BadLocationException ex) {
             LOG.error("precedent(int) - BadLocationException", ex);
             return;
         }
         if (ind != -1) {
+            textPane.getCaret().setSelectionVisible(true);
             textPane.setCaretPosition(ind);
             textPane.moveCaretPosition(ind+len);
-        }
-        else if (rech_pos != (doc.getLength()-len)) {
-                precedent(doc.getLength()-len); 
-             }
-             else {
-                getToolkit().beep(); 
-             }
-        toFront();     
+        } else if (rech_pos != (doc.getLength() - len))
+            precedent(doc.getLength() - len);
+        else
+            getToolkit().beep();
+        toFront();
     }
 
     public void remplacer() {
         final String texteRecherche = tfRechercher.getText();
         final String texteRemplacer = tfRemplacer.getText();
-        if (textPane.getSelectionStart()==textPane.getSelectionEnd()) return;
+        if (textPane.getSelectionStart() == textPane.getSelectionEnd())
+            return;
         final int start = textPane.getSelectionStart();
         final int end = textPane.getSelectionEnd();
         try {
-            if (!texteRecherche.equals("")){ 
-                textPane.setSelectionStart(0);
-                textPane.setSelectionEnd(0);
+            if (!texteRecherche.equals("")) {
                 final int lenRech = texteRecherche.length();
                 final int lenRemp = texteRemplacer.length();
                 int fin = end-lenRech+1;
-                final String text;
                 for (int i=start; i<fin; i++) {
-                    if ((!ignorerCasse && texteRecherche.equals(doc.getText(i,lenRech))) ||
-                           (ignorerCasse &&texteRecherche.equalsIgnoreCase(doc.getText(i,lenRech)))) 
-                        {
-                        if (includeComponent(i,lenRech)==(-1)){
+                    final String texte = doc.getText(i, lenRech);
+                    if ((!ignorerCasse && texteRecherche.equals(texte)) ||
+                            (ignorerCasse &&texteRecherche.equalsIgnoreCase(texte))) {
+                        if (includeComponent(i, lenRech) == -1) {
                             doc.remove(i, lenRech);
                             doc.insertString(i, texteRemplacer, null);
                             fin = fin - lenRech + lenRemp;
-                            i+=lenRemp-1;
-                        }
-                        else i=includeComponent(i,lenRech);
+                            i += lenRemp - 1;
+                        } else
+                            i = includeComponent(i, lenRech);
                     }
                 }
                 textPane.setSelectionStart(start);
-                textPane.setSelectionEnd(fin+lenRech-1);
-            }
-            else {
+                textPane.setSelectionEnd(fin + lenRech - 1);
+            } else {
                 if (start != end)
                     doc.remove(start, end - start);
                 doc.insertString(start, texteRemplacer, null);
@@ -565,7 +561,7 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
     }
 
     public void toutRemplacer() {
-        if (dansSelection){
+        if (dansSelection) {
              remplacer();
              return;
         }
@@ -581,20 +577,19 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         int ind = -1;
         final int i0 = 0;
         int ifin = doc.getLength()-len;
-        String text;
         try {
             for (int i=i0; i<ifin; i++) {
-                if (includeComponent(i,len)==(-1)){
-                    text = doc.getText(i, len);
-                    if ((!ignorerCasse && text.equals(texteRecherche)) ||
-                            (ignorerCasse && text.equalsIgnoreCase(texteRecherche))) {
+                if (includeComponent(i, len) == -1) {
+                    final String texte = doc.getText(i, len);
+                    if ((!ignorerCasse && texte.equals(texteRecherche)) ||
+                            (ignorerCasse && texte.equalsIgnoreCase(texteRecherche))) {
                         ind = i;
                         doc.remove(i, len);
                         doc.insertString(i, texteRemplacer, null);
-                        ifin = doc.getLength()-len;
+                        ifin = doc.getLength() - len;
                     }
-                }
-                else i=includeComponent(i,len);    
+                } else
+                    i = includeComponent(i, len);
             }
         } catch (final BadLocationException ex) {
             LOG.error("toutRemplacer() - BadLocationException", ex);
@@ -602,9 +597,8 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
                 ((JaxeTextPane)textPane).finEditionSpeciale();
             return;
         }
-        if (ind == -1) {
+        if (ind == -1)
             getToolkit().beep();
-        }
         if (textPane instanceof JaxeTextPane)
             ((JaxeTextPane)textPane).finEditionSpeciale();
     }
@@ -618,18 +612,18 @@ public class DialogueRechercher extends JDialog implements ActionListener, ItemL
         return(tfRechercher.getText());
     }
     
-    private int includeComponent(final int position, final int lenght) {
+    public void setTexteRecherche(final String texte) {
+        tfRechercher.setText(texte);
+    }
+    
+    private int includeComponent(final int position, final int longueur) {
         int posResult = -1;
-        for (int i=position; i<lenght+position; i++) {
-            Element element = null;
-            Component component = null;
-            Icon icon = null;
-            element = doc.getCharacterElement(i);
-            component = StyleConstants.getComponent(element.getAttributes());
-            icon = StyleConstants.getIcon(element.getAttributes());
-            if ((component != null) || (icon != null)){
+        for (int i=position; i<longueur+position; i++) {
+            final Element element = doc.getCharacterElement(i);
+            final Component component = StyleConstants.getComponent(element.getAttributes());
+            final Icon icon = StyleConstants.getIcon(element.getAttributes());
+            if ((component != null) || (icon != null))
                 posResult = i;
-            }
         }
         return posResult;
     }
