@@ -27,6 +27,8 @@ import java.awt.event.ItemListener;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.util.ArrayList;
+import java.util.Collections;
+import java.util.Comparator;
 import java.util.ResourceBundle;
 
 import javax.swing.*;
@@ -35,6 +37,7 @@ import javax.swing.event.CaretListener;
 
 import org.w3c.dom.Element;
 
+
 public class DialogueXpath extends JFrame implements ActionListener, ItemListener, CaretListener {
     /**
      * Logger for this class
@@ -42,32 +45,25 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
     private static final Logger LOG = Logger.getLogger(DialogueXpath.class);
     
     private static final ResourceBundle rb = JaxeResourceBundle.getRB();
-    JTextField textF = new JTextField("");
-    JCheckBox box1, box2;
-    ButtonGroup groupe;
-    private JButton butOk, butAnnuler;
-    private ArrayList<Element> listeElements = new ArrayList<Element>();
-    DialogueRechercher DialRech;
-    JaxeDocument doc;
-    private final JComboBox list;
+    private JTextField textF;
+    private ArrayList<Element> listeTousElements;
+    private ArrayList<Element> listeElements;
+    private DialogueRechercher dialRech;
+    private JaxeDocument doc;
+    private final JComboBox choix;
     private Element refElementCourant;
     private String nomElementCourant = "";
     private JTextField textInterne;
     private ZoneAttribut[] attributs;
-    String typeRechTxt;
+    private String typeRechTxt;
     private final JPanel pane;
-
-    private JPanel exprPane;
-
     private final JPanel listPane;
-
-    private JPanel textPane, attrPane, buttonPane;
-    private int natt;
     private boolean existNS = false;
-
-    public DialogueXpath(final DialogueRechercher DR, final JaxeDocument doc) {
+    
+    
+    public DialogueXpath(final DialogueRechercher dr, final JaxeDocument doc) {
         super(rb.getString("xpath.ExprXpath"));
-        DialRech = DR;
+        dialRech = dr;
         this.doc = doc;
         final org.w3c.dom.Node DomRoot = doc.DOMdoc.getDocumentElement();
         if (DomRoot != null) {
@@ -80,7 +76,8 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
 
         listPane = new JPanel(new FlowLayout());
         final JLabel textElt = new JLabel(rb.getString("xpath.TitreElt"));
-        listeElements = triListingElt(doc.cfg.listeTousElements());
+        listeTousElements = doc.cfg.listeTousElements();
+        listeElements = triListingElt(listeTousElements);
         final ArrayList<String> listeTitre = new ArrayList<String>();
         for (final Element ref : listeElements) {
             if (existNS) {
@@ -92,31 +89,29 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
             } else
                 listeTitre.add(doc.cfg.titreElement(ref));
         }
-        list = new JComboBox(listeTitre.toArray());
+        choix = new JComboBox(listeTitre.toArray());
         if ("".equals(nomElementCourant)) {
             refElementCourant = listeElements.get(0);
             nomElementCourant = doc.cfg.nomElement(refElementCourant);
-            final ArrayList<Element> latt = doc.cfg.listeAttributs(refElementCourant);
-            natt = latt.size();
         }
-        list.addItemListener(this);
+        choix.addItemListener(this);
         listPane.add(textElt);
-        listPane.add(list);
-                
+        listPane.add(choix);
+        
         majAttPanel();
         majExprXpath();
         setContentPane(pane);
         
         pack();
         final Dimension screen = Toolkit.getDefaultToolkit().getScreenSize();
-        setLocation((screen.width - getSize().width)/2,(screen.height - getSize().height)/2);
+        setLocation((screen.width - getSize().width)/2, (screen.height - getSize().height)/2);
         setVisible(true);
         
         addWindowListener(new WindowAdapter() {
             @Override
             public void windowClosing(final WindowEvent e) {
                 dispose();
-                DialRech.setVisible(true);
+                dialRech.setVisible(true);
             }
         });
     }
@@ -128,25 +123,26 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
             JOptionPane.showMessageDialog(this, rb.getString("xpath.ErrBalise"), rb.getString("xpath.Err"), JOptionPane.INFORMATION_MESSAGE);
             return;
         }
-        textPane = new JPanel();
+        final Config conf = doc.cfg.getRefConf(refElementCourant);
+        final JPanel textPane = new JPanel();
         textPane.setLayout(new BoxLayout(textPane, BoxLayout.X_AXIS));
         final JLabel includingText = new JLabel(rb.getString("xpath.IncludingText"));
         textInterne = new JTextField("");
         textInterne.addCaretListener(this);
         typeRechTxt = "contient";
-        groupe = new ButtonGroup();
-        box1 = new JCheckBox(rb.getString("xpath.TexteExact"),false);
-        groupe.add(box1);
-        box1.addItemListener(new ItemListener() {
-            public void itemStateChanged(final ItemEvent e) {   
+        final ButtonGroup groupe = new ButtonGroup();
+        final JCheckBox boxExact = new JCheckBox(rb.getString("xpath.TexteExact"), false);
+        groupe.add(boxExact);
+        boxExact.addItemListener(new ItemListener() {
+            public void itemStateChanged(final ItemEvent e) {
                 typeRechTxt = "texte exact";
                 majExprXpath();
             }
         } );
-        box2 = new JCheckBox(rb.getString("xpath.Contient"),true);
-        groupe.add(box2);
-        box2.addItemListener(new ItemListener() {
-            public void itemStateChanged(final ItemEvent e) {   
+        final JCheckBox boxContient = new JCheckBox(rb.getString("xpath.Contient"), true);
+        groupe.add(boxContient);
+        boxContient.addItemListener(new ItemListener() {
+            public void itemStateChanged(final ItemEvent e) {
                 typeRechTxt = "contient";
                 majExprXpath();
             }
@@ -155,15 +151,15 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
         textPane.add(includingText);
         textPane.add(Box.createHorizontalStrut(2));
         textPane.add(textInterne);
-        textPane.add(box1);
-        textPane.add(box2);
+        textPane.add(boxExact);
+        textPane.add(boxContient);
         textPane.add(Box.createHorizontalStrut(5));
         
-        attrPane = new JPanel();
+        final JPanel attrPane = new JPanel();
         final JScrollPane scp = new JScrollPane(attrPane,
                 ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS, ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER) {
             public Dimension getPreferredSize() {
-                Dimension dim = super.getPreferredSize();
+                final Dimension dim = super.getPreferredSize();
                 if (dim.height > 500)
                     dim.height = 500;
                 return(dim);
@@ -173,9 +169,30 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
         scp.setOpaque(false);
         scp.setBorder(BorderFactory.createTitledBorder(
             BorderFactory.createEtchedBorder(1, Color.white,new Color(70,70,70)),
-            rb.getString("xpath.TitleAtt") + " \" " + doc.cfg.nomElement(refElementCourant) + " \"", 1, 2));
-        final ArrayList<Element> latt = doc.cfg.listeAttributs(refElementCourant);
-        natt = latt.size();
+            rb.getString("xpath.TitleAtt") + " \" " + conf.nomElement(refElementCourant) + " \"", 1, 2));
+        
+        //final ArrayList<Element> latt = conf.listeAttributs(refElementCourant);
+        // il peut y avoir plusieurs lments avec le mme nom mais dfinis avec des attributs diffrents: on veut obtenir tous les attributs possibles
+        final ArrayList<Element> latt = new ArrayList<Element>();
+        for (final Element ref : listeTousElements) {
+            if (nomElementCourant != null && nomElementCourant.equals(conf.nomElement(ref))) {
+                for (final Element attref : conf.listeAttributs(ref)) {
+                    final String nomAttref = conf.nomAttribut(attref);
+                    if (nomAttref != null) {
+                        boolean trouve = false;
+                        for (final Element attref2 : latt)
+                            if (nomAttref.equals(conf.nomAttribut(attref2))) {
+                                trouve = true;
+                                break;
+                            }
+                        if (!trouve)
+                            latt.add(attref);
+                    }
+                }
+            }
+        }
+        
+        final int natt = latt.size();
         attributs = new ZoneAttribut[natt];
         attrPane.setLayout(new GridBagLayout());
         final GridBagConstraints c = new GridBagConstraints();
@@ -189,19 +206,19 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
             final ZoneAttribut zatt = new ZoneAttribut();
             attributs[i] = zatt;
             final Element att = latt.get(i);
-            zatt.nom = doc.cfg.nomAttribut(att);
-            zatt.titre = doc.cfg.titreAttribut(refElementCourant, att);
+            zatt.nom = conf.nomAttribut(att);
+            zatt.titre = conf.titreAttribut(refElementCourant, att);
             final JLabel textAtt = new JLabel(zatt.titre + " : ");
-            final ArrayList<String> lval = doc.cfg.listeValeursAttribut(att);
+            final ArrayList<String> lval = conf.listeValeursAttribut(att);
             zatt.valeurs = lval;
             if (lval != null && lval.size() > 0) {
                 zatt.valeurs.add(0, "");
                 final String[] titres = new String[zatt.valeurs.size()];
                 for (int j=0; j<zatt.valeurs.size(); j++)
-                    titres[j] = doc.cfg.titreValeurAttribut(refElementCourant, att, zatt.valeurs.get(j));
+                    titres[j] = conf.titreValeurAttribut(refElementCourant, att, zatt.valeurs.get(j));
                 zatt.champ = new JComboBox(titres);
                 ((JComboBox) zatt.champ).addActionListener(new ActionListener() {
-                     public void actionPerformed(final ActionEvent e) {   
+                     public void actionPerformed(final ActionEvent e) {
                          majExprXpath();
                      }
                  } );
@@ -212,42 +229,42 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
             c.gridx = 0;
             c.gridy = i;
             c.anchor = GridBagConstraints.EAST;
-            attrPane.add(textAtt,c);
+            attrPane.add(textAtt, c);
             c.gridx = 1;
             c.gridy = i;
             c.anchor = GridBagConstraints.WEST;
-            attrPane.add(zatt.champ,c);
+            attrPane.add(zatt.champ, c);
             if (zatt.champ instanceof JTextField) {
                 zatt.typeRech = "contient";
-                groupe = new ButtonGroup();
-                box1 = new JCheckBox(rb.getString("xpath.MotExact"),false);
-                groupe.add(box1);
-                box1.addItemListener(new ItemListener() {
-                    public void itemStateChanged(final ItemEvent e) {   
+                final ButtonGroup groupeAtt = new ButtonGroup();
+                final JCheckBox boxExactAtt = new JCheckBox(rb.getString("xpath.MotExact"), false);
+                groupeAtt.add(boxExactAtt);
+                boxExactAtt.addItemListener(new ItemListener() {
+                    public void itemStateChanged(final ItemEvent e) {
                         zatt.typeRech = "mot exact";
                         majExprXpath();
                     }
                 } );
                 c.gridx = 2;
                 c.gridy = i;
-                attrPane.add(box1,c);  
-                box2 = new JCheckBox(rb.getString("xpath.Contient"),true);
-                groupe.add(box2);
-                box2.addItemListener(new ItemListener() {
-                    public void itemStateChanged(final ItemEvent e) {   
+                attrPane.add(boxExactAtt, c);
+                final JCheckBox boxContientAtt = new JCheckBox(rb.getString("xpath.Contient"), true);
+                groupeAtt.add(boxContientAtt);
+                boxContientAtt.addItemListener(new ItemListener() {
+                    public void itemStateChanged(final ItemEvent e) {
                         zatt.typeRech = "contient";
                         majExprXpath();
                     }
                 } );
                 c.gridx = 3;
                 c.gridy = i;
-                attrPane.add(box2,c);
+                attrPane.add(boxContientAtt, c);
             } else {
                 zatt.typeRech = "mot exact";
             }
         }
         
-        exprPane = new JPanel(new FlowLayout());
+        final JPanel exprPane = new JPanel(new FlowLayout());
         final JLabel textRes = new JLabel(rb.getString("xpath.Expr"));
         textF = new JTextField("", 55);
         textF.setEditable(false);
@@ -255,12 +272,12 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
         exprPane.add(textRes);
         exprPane.add(textF);
         
-        buttonPane = new JPanel();
-        butOk = new JButton(rb.getString("xpath.Ok"));
+        final JPanel buttonPane = new JPanel();
+        final JButton butOk = new JButton(rb.getString("xpath.Ok"));
         butOk.addActionListener(this);
         butOk.setActionCommand("OK");
         getRootPane().setDefaultButton(butOk);
-        butAnnuler = new JButton(rb.getString("xpath.Annuler"));
+        final JButton butAnnuler = new JButton(rb.getString("xpath.Annuler"));
         butAnnuler.addActionListener(this);
         butAnnuler.setActionCommand("Annuler");
         buttonPane.add(butOk);
@@ -277,7 +294,7 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
         pane.add(Box.createVerticalStrut(5));
         pane.add(buttonPane);
     }
-        
+    
     
     public void caretUpdate(final CaretEvent e) {
         majExprXpath();
@@ -285,14 +302,14 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
     
     
     public void actionPerformed(final ActionEvent e) {
-        if (e.getSource() == butOk) {
+        final String cmd = e.getActionCommand();
+        if ("OK".equals(cmd)) {
             dispose();
-            DialRech.tfRechercher.setText(textF.getText());
-            DialRech.setVisible(true);
-        }
-        if (e.getSource() == butAnnuler) {
+            dialRech.setTexteRecherche(textF.getText());
+            dialRech.setVisible(true);
+        } else if ("Annuler".equals(cmd)) {
             dispose();
-            DialRech.setVisible(true);
+            dialRech.setVisible(true);
         }
     }
     
@@ -316,7 +333,7 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
         } else {
             expr.append("//");
             expr.append(nomElementCourant);
-        }    
+        }
         final String text = textInterne.getText();
         if (!text.equals("")) {
             if (existAtt)
@@ -324,12 +341,12 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
             else {
                 expr.append("[");
                 existAtt = true;
-            }            
+            }
             if (typeRechTxt.equals("contient")) {
                 expr.append("contains(.,\"");
                 expr.append(text);
                 expr.append("\")");
-            } else { 
+            } else {
                 expr.append(".=\"");
                 expr.append(text);
                 expr.append("\"");
@@ -344,7 +361,7 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
                 valAtt = zatt.valeurs.get(index);
             }
             final boolean nonvide = (!"".equals(valAtt));
-            if (nonvide){
+            if (nonvide) {
                 if (existAtt) {
                     expr.append(" and ");
                 } else {
@@ -356,14 +373,14 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
                     expr.append(zatt.nom);
                     expr.append(",\"");
                     expr.append(valAtt);
-                    expr.append("\")"); 
+                    expr.append("\")");
                 } else {
                     expr.append("@");
                     expr.append(zatt.nom);
                     expr.append("=\"");
                     expr.append(valAtt);
-                    expr.append("\""); 
-                } 
+                    expr.append("\"");
+                }
             }
         }
         if (existAtt)
@@ -374,8 +391,8 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
     
     public void itemStateChanged(final ItemEvent e) {
         final Object source = e.getSource();
-        if ((source == list) && (e.getStateChange() == 1)) {
-            refElementCourant = listeElements.get(list.getSelectedIndex());
+        if ((source == choix) && (e.getStateChange() == ItemEvent.SELECTED)) {
+            refElementCourant = listeElements.get(choix.getSelectedIndex());
             nomElementCourant = doc.cfg.nomElement(refElementCourant);
             pane.removeAll();
             majAttPanel();
@@ -386,32 +403,9 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
     }
     
     
-    public ArrayList<Element> triListingElt(final ArrayList<Element> list){
-        final ArrayList<Element> listeTriee = new ArrayList<Element>(list.size());
-        final char[][] replaceChar = {{'','O'},{'','I'},{'','I'},{'','A'},{'','A'},{'','E'},{'','E'},{'','E'},{'','E'}};
-        if (list.size() > 0)
-            listeTriee.add(0,list.get(0));
-        for (int i=1; i<list.size(); i++) {
-            boolean ajout = false;
-            String eltTraite = (doc.cfg.titreElement(list.get(i)).toUpperCase());
-            for (final char[] element : replaceChar) {
-                eltTraite = eltTraite.replace(element[0], element[1]);
-            }    
-            for (int j=0; j<i; j++) {
-                String eltTrie = (doc.cfg.titreElement(listeTriee.get(j)).toUpperCase());
-                for (final char[] element : replaceChar) {
-                    eltTrie = eltTrie.replace(element[0], element[1]);
-                } 
-                if (((eltTraite.trim()).compareToIgnoreCase((eltTrie.trim()).toUpperCase())) <= 0) {
-                    listeTriee.add(j, list.get(i));
-                    ajout = true;
-                    j = i;
-                }
-            }
-            if (!ajout)
-                listeTriee.add(i, list.get(i));
-        }
-        
+    private ArrayList<Element> triListingElt(final ArrayList<Element> liste) {
+        final ArrayList<Element> listeTriee = new ArrayList<Element>(liste);
+        Collections.sort(listeTriee, new ComparateurDeTitres());
         final ArrayList<Element> listeSansDoublon = new ArrayList<Element>();
         Element refprecedent = null;
         for (final Element ref : listeTriee) {
@@ -426,7 +420,32 @@ public class DialogueXpath extends JFrame implements ActionListener, ItemListene
             refprecedent = ref;
         }
         return listeSansDoublon;
-    }        
+    }
+    
+    class ComparateurDeTitres implements Comparator<Element> {
+        final String maj1 = "";
+        final String maj2 = "OIIAAEEEE";
+        public int compare(Element e1, Element e2) {
+            final String titremaj1 = convertir(doc.cfg.titreElement(e1).toUpperCase().trim());
+            final String titremaj2 = convertir(doc.cfg.titreElement(e2).toUpperCase().trim());
+            return(titremaj1.compareToIgnoreCase(titremaj2));
+        }
+        private String convertir(final String s) {
+            final StringBuilder sb = new StringBuilder(s);
+            char c;
+            int p;
+            for (int i=0; i<sb.length(); i++) {
+                c = sb.charAt(i);
+                p = maj1.indexOf(c);
+                if (p != -1)
+                    sb.setCharAt(i, maj2.charAt(p));
+            }
+            return(sb.toString());
+        }
+        public boolean equals(Object obj) {
+            return(obj instanceof ComparateurDeTitres);
+        }
+    }
     
     class ZoneAttribut {
         JComponent champ;
