@@ -66,7 +66,6 @@ public abstract class JaxeElement {
     public final static String kBarre = "BARRE";
     
     
-    //static String newline = Jaxe.newline;
     public Position debut = null; // position du premier caractre de l'lment
 
     public Position fin = null; // position du dernier caractre de l'lment
@@ -140,9 +139,47 @@ public abstract class JaxeElement {
         }
         init(pos, noeud);
         majValidite();
+        // en dbut du textpane: pb avec la position 0 qui ne change pas pour le noeud suivant
+        if (pos.getOffset() == 0 && noeud.getNextSibling() != null)
+            majPosSuivant();
     }
-
+    
+    protected void majPosSuivant() {
+        final JaxeElement jesuivant = doc.getElementForNode(noeud.getNextSibling());
+        if (jesuivant != null && jesuivant.debut != null && jesuivant.debut.getOffset() == 0) {
+            try {
+                jesuivant.debut = doc.createPosition(fin.getOffset() + 1);
+                if (jesuivant.fin.getOffset() == 0)
+                    jesuivant.fin = jesuivant.debut;
+            } catch (BadLocationException ex) {
+                LOG.error("majPosSuivant - BadLocationException", ex);
+            }
+            for (int i=0; i<jesuivant.compos.size(); i++)
+                if (jesuivant.compos.get(i).getOffset() == 0)
+                    jesuivant.compos.set(i, jesuivant.debut);
+        }
+    }
+    
     public abstract Node nouvelElement(Element refElement);
+    
+    /**
+     * Renvoie true s'il faut afficher le dialogue d'attributs  la cration de l'lment
+     */
+    public boolean testAffichageDialogue() {
+        final ArrayList<Element> latt = doc.cfg.listeAttributs(refElement);
+        if (latt == null || latt.size() == 0)
+            return(false);
+        final Properties prefs = Preferences.getPref();
+        final boolean toujoursAfficherAttributs = (prefs != null && "true".equals(prefs.getProperty("toujoursAfficherAttributs")));
+        if (toujoursAfficherAttributs)
+            return(true);
+        else {
+            for (Element attdef : latt)
+                if (doc.cfg.attributObligatoire(refElement, attdef))
+                    return(true);
+            return(false);
+        }
+    }
 
     /**
      * Affiche le dialogue correspondant  l'lment
@@ -231,7 +268,7 @@ public abstract class JaxeElement {
      */
     public Position insertComponent(Position pos, final JComponent comp) {
         final int offsetdebut = pos.getOffset();
-        final Style s = doc.textPane.addStyle(null, null);
+        final Style s = doc.addStyle(null, null);
         StyleConstants.setComponent(s, comp);
         try {
             doc.insertString(pos.getOffset(), "*", s, false);
@@ -257,7 +294,7 @@ public abstract class JaxeElement {
      */
     public Position insertIcon(Position pos, final Icon icon) {
         final int offsetdebut = pos.getOffset();
-        final Style s = doc.textPane.addStyle(null, null);
+        final Style s = doc.addStyle(null, null);
         StyleConstants.setIcon(s, icon);
         try {
             doc.insertString(pos.getOffset(), "*", s, false);
@@ -303,7 +340,7 @@ public abstract class JaxeElement {
     }
 
     /**
-     * Renvoit les lments se trouvant dans la zone du texte indique
+     * Renvoit les lments se trouvant dans la zone du texte indique (de dpos  fpos inclu)
      */
     public ArrayList<JaxeElement> elementsDans(final int dpos, final int fpos) {
         final ArrayList<JaxeElement> l = new ArrayList<JaxeElement>();
@@ -312,7 +349,7 @@ public abstract class JaxeElement {
         if (debut.getOffset() > fpos || fin.getOffset() < dpos)
             return l;
         if (debut.getOffset() >= dpos && (fin.getOffset() <= fpos ||
-                this instanceof JESwing && fin.getOffset() == fpos+1))
+                (this instanceof JESwing && fin.getOffset() == fpos+1)))
             l.add(this);
         else
             for (Node n = noeud.getFirstChild(); n != null; n = n.getNextSibling()) {
@@ -355,30 +392,17 @@ public abstract class JaxeElement {
             return(null);
         if (debut.getOffset() > pos || fin.getOffset() < pos)
             return(null);
-        if (noeud.getNodeType() == Node.TEXT_NODE)
-            return("texte");
-        if (noeud.getNodeType() == Node.COMMENT_NODE)
-            return("commentaire");
-        if (noeud.getNodeType() == Node.CDATA_SECTION_NODE)
-            return("cdata");
+        switch(noeud.getNodeType()) {
+            case Node.TEXT_NODE : return("texte");
+            case Node.COMMENT_NODE : return("commentaire");
+            case Node.CDATA_SECTION_NODE : return("cdata");
+            case Node.PROCESSING_INSTRUCTION_NODE : return("instruction");
+        }
         if (!(noeud.getNodeType() == Node.ELEMENT_NODE))
             return(null);
         Element el = (Element) noeud;
         final StringBuilder nomel = new StringBuilder(el.getTagName());
-        if (this instanceof JEStyle) {
-            nomel.setLength(0);
-            final Iterator<Element> it = ((JEStyle)this)._styles.iterator();
-            while (it.hasNext()) {
-                final Element sel = it.next();
-                if (getParent() != null)  {
-                    nomel.append(sel.getNodeName());
-                    nomel.append("[");
-                    nomel.append(nombreXPath());
-                    nomel.append("]");
-                }
-                if (it.hasNext()) nomel.append("/"); else el = sel;
-            }
-        } else if (getParent() != null) {
+        if (getParent() != null) {
             nomel.append("[");
             nomel.append(nombreXPath());
             nomel.append("]");
@@ -461,40 +485,26 @@ public abstract class JaxeElement {
             return;
         try {
             if (this instanceof JECommentaire || this instanceof JECData) {
-                final String texte = doc.getText(debut.getOffset()+1, fin.getOffset()
-                        - debut.getOffset() - 1);
+                final String texte = doc.getText(debut.getOffset()+1, fin.getOffset() - debut.getOffset() - 1);
                 noeud.setNodeValue(texte);
-            } else if (noeud.getNodeType() == Node.TEXT_NODE ||
-                    this instanceof JEStyle) {
+            } else if (this instanceof JETexte) {
                 final Node nsuivant = noeud.getNextSibling();
                 if (nsuivant != null) {
                     final JaxeElement jesuivant = doc.getElementForNode(nsuivant);
-                    if (jesuivant != null
-                            && jesuivant.debut.getOffset() > fin.getOffset() + 1) {
+                    if (jesuivant != null && jesuivant.debut.getOffset() > fin.getOffset() + 1) {
                         // texte rajout  la fin, avant un autre lment
                         fin = doc.createPosition(jesuivant.debut.getOffset() - 1);
                     }
                 }
-                final String texte = doc.getText(debut.getOffset(), fin.getOffset()
-                        - debut.getOffset() + 1);
+                final String texte = doc.getText(debut.getOffset(), fin.getOffset() - debut.getOffset() + 1);
                 if (texte == null || "".equals(texte))
                     getParent().supprimerEnfant(this);
-                else {
-                    if (noeud.getNodeType() == Node.TEXT_NODE)
-                        noeud.setNodeValue(texte);
-                    else {
-                        Node n = noeud;
-                        while (n != null && n.getNodeType() != Node.TEXT_NODE) {
-                            n = n.getFirstChild();
-                        }
-                        if (n != null)
-                            n.setNodeValue(texte);
-                    }
-                }
+                else
+                    noeud.setNodeValue(texte);
             } else {
                 int offdebut = debut.getOffset();
                 int debuttexte;
-                if (this instanceof JESwing)
+                if (this instanceof JESwing || this instanceof JEStyle)
                     debuttexte = offdebut;
                 else
                     debuttexte = offdebut + 1;
@@ -504,9 +514,8 @@ public abstract class JaxeElement {
                         if (debuttexte < je.debut.getOffset()) {
                             JaxeElement jeprev = null;
                             if (n.getPreviousSibling() != null)
-                                    jeprev = doc.getElementForNode(n.getPreviousSibling());
-                            if (jeprev != null
-                                    && (jeprev instanceof JEStyle || jeprev instanceof JETexte)) {
+                                jeprev = doc.getElementForNode(n.getPreviousSibling());
+                            if (jeprev != null && (jeprev instanceof JEStyle || jeprev instanceof JETexte)) {
                                 // texte ajout  la fin du prcdent noeud
                                 jeprev.fin = doc.createPosition(je.debut.getOffset() - 1);
                             } else if (je instanceof JETexte)
@@ -526,14 +535,17 @@ public abstract class JaxeElement {
                         debuttexte = offdebut + 1;
                     }
                 }
-                if (debuttexte < fin.getOffset()) { // texte  la fin, aprs le dernier enfant
+                if (debuttexte < fin.getOffset() || (this instanceof JEStyle && debuttexte == fin.getOffset())) { // texte  la fin, aprs le dernier enfant
                     JaxeElement pje = null;
                     if (noeud.getLastChild() != null)
                         pje = doc.getElementForNode(noeud.getLastChild());
-                    if (pje instanceof JEStyle || pje instanceof JETexte)
+                    if (pje instanceof JEStyle || pje instanceof JETexte) {
                         // texte ajout  la fin du dernier enfant
-                        pje.fin = doc.createPosition(fin.getOffset() - 1);
-                    else {
+                        if (this instanceof JEStyle)
+                            pje.fin = doc.createPosition(fin.getOffset());
+                        else
+                            pje.fin = doc.createPosition(fin.getOffset() - 1);
+                    } else {
                         // nouvelle zone de texte  la fin
                         final String texte = doc.getText(debuttexte, fin.getOffset() - debuttexte);
                         final JETexte newje = JETexte.nouveau(doc, doc.createPosition(debuttexte),
@@ -553,48 +565,23 @@ public abstract class JaxeElement {
     }
 
     /**
-     * nouvel lment DOM. Attention: ambigu quand des espaces de noms sont employs
-     *
-     * @deprecated  Utiliser nouvelElementDOM(JaxeDocument, Element)  la place
-     */
-    @Deprecated
-    public static Node nouvelElementDOM(final JaxeDocument doc, final String type,
-            String nomNoeud) {
-        Node newel;
-        if (type.equals("instruction"))
-            newel = doc.DOMdoc.createProcessingInstruction(nomNoeud, "");
-        else {
-            if (doc.cfg.namespace() == null)
-                newel = doc.DOMdoc.createElementNS(null, nomNoeud);
-            else {
-                Config conf = doc.cfg.getBaliseConf(nomNoeud);
-                if (conf == null)
-                    conf = doc.cfg;
-                if (conf.prefixe() != null)
-                    nomNoeud = conf.prefixe() + ":" + nomNoeud;
-                newel = doc.DOMdoc.createElementNS(conf.namespace(), nomNoeud);
-            }
-        }
-        return (newel);
-    }
-
-    /**
-     * nouvel lment DOM. Espace de noms obtenu  partir de la dfinition de la
-     * configuration.
+     * nouvel lment DOM. Espace de noms obtenu  partir de la dfinition de la configuration.
      * Attention, l'lment est maintenant une rfrence d'lment, ce n'est plus la dfinition du fichier de config.
      */
     public static Element nouvelElementDOM(final JaxeDocument doc, final Element refElement) {
-        if ("BALISE".equals(refElement.getNodeName()) && doc.cfg.getSchema() instanceof SchemaW3C) {
-            LOG.error("nouvelElementDOM utilis avec un lment du fichier de config au lieu d'un lment du schma");
-            return(nouvelElementDOM(doc, doc.cfg.referenceElement(refElement.getAttribute("nom"))));
-        }
         final Element newel;
         String nom = doc.cfg.nomElement(refElement);
         final String espace = doc.cfg.espaceElement(refElement);
-        if (espace == null || "".equals(espace))
+        if (espace == null)
             newel = doc.DOMdoc.createElementNS(null, nom);
         else {
-            final String prefixe = doc.cfg.prefixeElement(refElement);
+            String prefixe;
+            if (doc.DOMdoc.getDocumentElement() != null)
+                prefixe = doc.DOMdoc.lookupPrefix(espace);
+            else
+                prefixe = null;
+            if (prefixe == null)
+                prefixe = doc.cfg.prefixeElement(refElement);
             if (prefixe != null)
                 nom = prefixe + ":" + nom;
             newel = doc.DOMdoc.createElementNS(espace, nom);
@@ -645,21 +632,20 @@ public abstract class JaxeElement {
      * insre newel dans l'arbre DOM
      */
     public void insererDOM(final Position pos, final Node newel) {
-        JaxeElement parent = doc.rootJE.elementA(pos.getOffset());
-        if (parent.debut.getOffset() == pos.getOffset()
-                && !(parent instanceof JESwing)) parent = parent.getParent();
+        insererDOM(doc, pos, newel);
+    }
+    
+    /**
+     * insre newel dans l'arbre DOM
+     */
+    public static void insererDOM(final JaxeDocument doc, final Position pos, final Node newel) {
+        JaxeElement parent = doc.elementA(pos.getOffset());
+        if (parent != null && parent.debut.getOffset() == pos.getOffset() && !(parent instanceof JESwing))
+            parent = parent.getParent();
         if (parent instanceof JETexte || parent instanceof JEStyle) {
-            final int ic = pos.getOffset() - parent.debut.getOffset();
-            if (ic > 0) {
-                // nouvelle zone de texte...  revoir
-                /*
-                 * String s = parent.noeud.getNodeValue(); String s1 =
-                 * s.substring(0, ic); String s2 = s.substring(ic);
-                 * parent.noeud.setNodeValue(s2); Node ns1 =
-                 * doc.DOMdoc.createTextNode(s1); Node parent2 =
-                 * parent.noeud.getParentNode(); parent2.insertBefore(ns1,
-                 * parent.noeud); parent2.insertBefore(newel, parent.noeud);
-                 */
+            while (parent.getParent() instanceof JEStyle)
+                parent = parent.getParent(); // une insertion dans un style se fait toujours au mme niveau que le style de plus haut niveau
+            if (pos.getOffset() > parent.debut.getOffset()) {
                 final JaxeElement je2 = parent.couper(pos);
                 final Node parent2 = parent.noeud.getParentNode();
                 parent2.insertBefore(newel, je2.noeud);
@@ -667,12 +653,35 @@ public abstract class JaxeElement {
                 final Node parent2 = parent.noeud.getParentNode();
                 parent2.insertBefore(newel, parent.noeud);
             }
-        } else {
+        } else if (parent != null) {
             final JaxeElement jelbef = parent.enfantApres(pos.getOffset());
             if (jelbef == null)
                 parent.noeud.appendChild(newel);
             else
                 parent.noeud.insertBefore(newel, jelbef.noeud);
+        } else {
+            // cas des noeuds directement sous le noeud document (comme la racine)
+            JaxeElement jelbef = null;
+            for (Node n = doc.DOMdoc.getFirstChild(); n != null; n = n.getNextSibling()) {
+                final short type = n.getNodeType();
+                if (type == Node.ELEMENT_NODE || type == Node.PROCESSING_INSTRUCTION_NODE || type == Node.COMMENT_NODE) {
+                    final JaxeElement je = doc.getElementForNode(n);
+                    if (je != null) {
+                        if (je.debut.getOffset() == pos.getOffset()) {
+                            jelbef = je;
+                            break;
+                        } else if (je.fin.getOffset() >= pos.getOffset()) {
+                            if (n.getNextSibling() != null)
+                                jelbef = doc.getElementForNode(n.getNextSibling());
+                            break;
+                        }
+                    }
+                }
+            }
+            if (jelbef == null)
+                doc.DOMdoc.appendChild(newel);
+            else
+                doc.DOMdoc.insertBefore(newel, jelbef.noeud);
         }
     }
 
@@ -862,8 +871,8 @@ public abstract class JaxeElement {
         try {
             final int len = je.fin.getOffset() - je.debut.getOffset() + 1;
             /*
-             * String cfin = doc.getText(je.fin.getOffset() + 1, 1); if
-             * (newline.equals(cfin)) len++;
+             * String cfin = doc.getText(je.fin.getOffset() + 1, 1);
+             * if ("\n".equals(cfin)) len++;
              */
             final int idebut = je.debut.getOffset();
             /*
@@ -963,37 +972,16 @@ public abstract class JaxeElement {
         else
             return (p.indentations());
     }
-
+    
     /**
-     * coupe la zone de texte en 2, retourne la nouvelle zone cre aprs
-     * celle-ci
+     * Coupe un lment en 2, retourne le nouvel lment cr aprs celui-ci.
+     * Par dfaut, ne fait rien et renvoie null.
+     * Remplac dans JETexte et JEStyle.
      */
     public JaxeElement couper(final Position pos) {
-        final String t = noeud.getNodeValue();
-        final String t1 = t.substring(0, pos.getOffset() - debut.getOffset());
-        final String t2 = t.substring(pos.getOffset() - debut.getOffset());
-        noeud.setNodeValue(t1);
-        final Node textnode2 = doc.DOMdoc.createTextNode(t2);
-        final Node nextnode = noeud.getNextSibling();
-        final JaxeElement parent = getParent();
-        if (nextnode == null)
-            parent.noeud.appendChild(textnode2);
-        else
-            parent.noeud.insertBefore(textnode2, nextnode);
-        final JETexte je2 = new JETexte(doc);
-        je2.noeud = textnode2;
-        je2.doc = parent.doc;
-        try {
-            je2.debut = doc.createPosition(pos.getOffset());
-            je2.fin = fin;
-            fin = doc.createPosition(pos.getOffset() - 1);
-        } catch (final BadLocationException ex) {
-            LOG.error("couper() - BadLocationException", ex);
-        }
-        doc.dom2JaxeElement.put(je2.noeud, je2);
-        return (je2);
+        return(null);
     }
-
+    
     /**
      * fusionne cet lment avec celui donn, dans le DOM (aucun changement du
      * texte)
@@ -1021,7 +1009,8 @@ public abstract class JaxeElement {
         for (Node n = noeud.getFirstChild(); n != null; n = n.getNextSibling()) {
             final JaxeElement je1 = doc.getElementForNode(n);
             JaxeElement je2 = doc.getElementForNode(n.getNextSibling());
-            while (je2 != null && ((je1 instanceof JETexte && je2 instanceof JETexte) || (je1 instanceof JEStyle && je2 instanceof JEStyle && sameStyle(je1, je2)))) {
+            while (je2 != null && ((je1 instanceof JETexte && je2 instanceof JETexte) ||
+                    (je1 instanceof JEStyle && je2 instanceof JEStyle && ((JEStyle)je1).memeStyle((JEStyle)je2)))) {
                 je1.fusionner(je2);
                 je2 = je1.getNextSibling();
             }
@@ -1061,48 +1050,6 @@ public abstract class JaxeElement {
         }
     }
     
-    public static boolean sameStyle(final JaxeElement source, final JaxeElement target) {
-        if (!(source instanceof JEStyle && target instanceof JEStyle))
-            return false;
-        final List<String> s1 = new ArrayList<String>();
-        final JEStyle sourceJE = (JEStyle)source;
-        final JEStyle targetJE = (JEStyle)target;
-        Iterator<? extends Node> temp = sourceJE._styles.iterator();
-        if (targetJE._styles.size() != sourceJE._styles.size()) {
-            return false;
-        }
-        while (temp.hasNext()) {
-            s1.add(temp.next().getNodeName());
-        }
-        final List<String> s2 = new ArrayList<String>();
-        temp = targetJE._styles.iterator();
-        while (temp.hasNext()) {
-            s2.add(temp.next().getNodeName());
-        }
-
-        boolean result = true;
-        Iterator<String> it = s1.iterator();
-        while (it.hasNext() && result) {
-            if (!s2.contains(it.next()))
-                result = false;
-        }
-        it = s2.iterator();
-        while (it.hasNext() && result) {
-            if (!s1.contains(it.next()))
-                result = false;
-        }
-        if (result) {
-            final String[] sourceStyles = sourceJE.ceStyle.split(";");
-            final String[] targetStyles = targetJE.ceStyle.split(";");
-            Arrays.sort(sourceStyles);
-            Arrays.sort(targetStyles);
-            final List<String> soStList = Arrays.asList(sourceStyles);
-            final List<String> taStList = Arrays.asList(targetStyles);
-            result = soStList.containsAll(taStList) && taStList.containsAll(soStList); 
-        }
-        return result;
-    }
-    
     public boolean aDesAttributs() {
         if (noeud.getNodeType() != Node.ELEMENT_NODE)
             return(false);
@@ -1126,98 +1073,6 @@ public abstract class JaxeElement {
     }
     
     
-    /**
-     * Dfini les ensembles de couleurs  utiliser pour les boutons. Par dfaut,
-     * tableau de tableaux  3 entres: 1) normal 2) slection 3) invalide.
-     * Par exemple
-     * {{jauneLeger, rougeFonce, orange}, {bleuClair, rougeFonce, violet}}
-     * @deprecated utiliser Balise.setCouleurs  la place
-     */
-    @Deprecated
-    public static void setMonBoutonCouleurs(final Color[][] couleurs) {
-        Balise.setCouleurs(couleurs);
-    }
-    
-    /**
-     * Returns the current Colors of the Buttons
-     * 
-     * @return Colors of the Buttons
-     * @deprecated utiliser Balise.getCouleurs  la place
-     */
-    @Deprecated
-    public static Color[][] getMonBoutonCouleurs() {
-        return(Balise.getCouleurs());
-    }
-    
-    /**
-     * Sets the Border of the Button
-     * @deprecated
-     * 
-     * @param border
-     *            New Border of the Button
-     */
-    @Deprecated
-    public static void setMonBoutonBorder(final Border border) {
-        Balise.setBord(border);
-    }
-    
-    /**
-     * Returns the current border of the Buttons
-     * 
-     * @return Border of the Buttons
-     * @deprecated utiliser Balise.getBord  la place
-     */
-    @Deprecated
-    public static Border getMonBoutonBorder() {
-        return(Balise.getBord());
-    }
-    
-    /**
-     * Gets the Font the MonBouton is using
-     * 
-     * @return Current Font of MonBouton
-     * @deprecated utiliser Balise.getPolice  la place
-     */
-    @Deprecated
-    public static Font getMonBoutonFont() {
-        return(Balise.getPolice());
-    }
-
-    /**
-     * Sets the Font the MonBouton should use
-     * 
-     * @param font
-     *            Font to use
-     * @deprecated utiliser Balise.setPolice  la place
-     */
-    @Deprecated
-    public static void setMonBoutonFont(final Font font) {
-        Balise.setPolice(font);
-    }
-
-    /**
-     * Bouton reprsentant le dbut ou la fin d'un lment dans le texte
-     * @deprecated utiliser Balise  la place
-     */
-    @Deprecated
-    public class MonBouton extends Balise {
-        
-        public MonBouton(final JaxeElement je, final boolean division, final int typeBalise) {
-            super(je, division, typeBalise);
-        }
-        
-        @Deprecated
-        public MonBouton(final String texte, final boolean division) {
-            super(JaxeElement.this, texte, division, VIDE);
-        }
-        
-        public MonBouton(final String texte, final boolean division, final int typeBalise) {
-            super(JaxeElement.this, texte, division, typeBalise);
-        }
-        
-    }
-    
-    
     protected static String getString(final String key) {
         return(JaxeResourceBundle.getRB().getString(key));
     }
@@ -1227,7 +1082,7 @@ public abstract class JaxeElement {
 
             final String[] styleSplit = style.split(";");
 
-            final Style s = doc.textPane.addStyle(null, null);
+            final Style s = doc.addStyle(null, null);
 
             for (final String element : styleSplit) {
                 if (element.indexOf(kExposant) > -1)
