@@ -35,6 +35,7 @@ import java.util.Properties;
 import java.util.ResourceBundle;
 
 import javax.swing.Action;
+import javax.swing.JFileChooser;
 import javax.swing.JFrame;
 import javax.swing.JLabel;
 import javax.swing.JMenu;
@@ -88,7 +89,6 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
     private JaxeMenuBar menuBar;
     private String nomFichierCfg;
     
-    private HTMLFrame htmlFrame = null;
     private ValidationFrame validationFrame = null;
     private SourceFrame sourceFrame = null;
     
@@ -348,11 +348,6 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
             arbrexml.newdoc(doc) ;
         setVisible(true);
         
-        if (htmlFrame != null) {
-            htmlFrame.setVisible(false);
-            htmlFrame.dispose();
-            htmlFrame = null;
-        }
         if (validationFrame != null) {
             validationFrame.setVisible(false);
             validationFrame.dispose();
@@ -510,18 +505,6 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
         textPane.setCaretPosition(0);
         majMenus(textPane.getCaretPosition());
         
-        if (doc.cfg != null) {
-            if (htmlFrame == null) {
-                // on n'affiche le HTML que s'il existe dj
-                final File htmlFile = HTMLFrame.fichierHTML(doc.fsave);
-                if (htmlFile.exists())
-                    htmlFrame = new HTMLFrame(doc);
-            } else {
-                htmlFrame.newdoc(doc);
-                htmlFrame.setVisible(true);
-            }
-        }
-        
         menuBar.setActivationMenuValider(doc.cfg != null && doc.cfg.schemaURL != null);
         ArrayList<Element> exportsHTML;
         if (doc.cfg == null)
@@ -546,11 +529,6 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
                 return(false);
         }
         setVisible(false);
-        if (htmlFrame != null) {
-            htmlFrame.setVisible(false);
-            htmlFrame.dispose();
-            htmlFrame = null;
-        }
         if (validationFrame != null) {
             validationFrame.setVisible(false);
             validationFrame.dispose();
@@ -563,11 +541,12 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
         }
         Jaxe.allFrames.remove(this);
         
+        if (!quit && Jaxe.allFrames.size() == 0)
+            Jaxe.dialogueDepart();
+        
         // aide pour le garbage collector
         dispose();
         
-        if (!quit && Jaxe.allFrames.size() == 0)
-            Jaxe.dialogueDepart();
         return(true);
     }
     
@@ -590,26 +569,32 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
     public void enregistrerSous() {
         if (System.getProperty("os.name").startsWith("Mac OS")) // pour viter un bug avec FileDialog et cmd-v
             menuBar.setEnabled(false);
-        FileDialog fd = new FileDialog(this, null, FileDialog.SAVE);
-        if (System.getProperty("os.name").indexOf("Linux") != -1 && JaxeMenuBar.dernierRepertoire != null)
-            fd.setDirectory(JaxeMenuBar.dernierRepertoire);
-        fd.setVisible(true);
-        if (System.getProperty("os.name").startsWith("Mac OS"))
-            menuBar.setEnabled(true);
-        final String sf = fd.getFile();
-        if (sf != null) {
-            JaxeMenuBar.dernierRepertoire = fd.getDirectory();
-            File f = new File(fd.getDirectory(), sf);
+        File f = null;
+        if (System.getProperty("os.name").indexOf("Linux") != -1) {
+            final JFileChooser chooser = new JFileChooser(JaxeMenuBar.dernierRepertoire);
+            final int resultat = chooser.showSaveDialog(this);
+            if (resultat == JFileChooser.APPROVE_OPTION) {
+                JaxeMenuBar.dernierRepertoire = chooser.getCurrentDirectory();
+                f = chooser.getSelectedFile();
+            }
+        } else {
+            final FileDialog fd = new FileDialog(this, null, FileDialog.SAVE);
+            fd.setVisible(true);
+            if (System.getProperty("os.name").startsWith("Mac OS"))
+                menuBar.setEnabled(true);
+            final String sf = fd.getFile();
+            if (sf != null)
+                f = new File(fd.getDirectory(), sf);
+        }
+        if (f != null) {
             if (f.getName().indexOf('.') == -1) {
                 f = new File(f.getPath() + ".xml");
                 if (f.exists()) {
-                    if (JOptionPane.showConfirmDialog(this,
-                        rb.getString("enregistrement.remplacer"), "",
-                        JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
-                            fd = null;
-                            enregistrerSous();
-                            return;
-                        }
+                    if (JOptionPane.showConfirmDialog(this, rb.getString("enregistrement.remplacer"), "",
+                            JOptionPane.YES_NO_OPTION) == JOptionPane.NO_OPTION) {
+                        enregistrerSous();
+                        return;
+                    }
                 }
             }
             try {
@@ -741,8 +726,7 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
                 final int debutSelection = textPane.getSelectionStart();
                 final int finSelection = textPane.getSelectionEnd();
                 for (final Element ref : sousElements) {
-                    if (parentconf == null || parent == null ||
-                            parentconf.insertionPossible(parent, debutSelection, finSelection, ref)) {
+                    if (parent == null || doc.cfg.insertionPossible(parent, debutSelection, finSelection, ref)) {
                         autorisees.add(ref);
                     }
                 }
@@ -791,13 +775,6 @@ public class JaxeFrame extends JFrame implements ComponentListener, EcouteurMAJ
         return(anyenab);
     }
     
-    public void activerHTMLFrame() {
-        if (htmlFrame == null)
-            htmlFrame = new HTMLFrame(doc);
-        else
-            htmlFrame.miseAJour();
-    }
-    
     public void activerValidationFrame() {
         if (validationFrame == null)
             validationFrame = new ValidationFrame(doc, this);
