@@ -18,6 +18,8 @@ import java.awt.FileDialog;
 import java.awt.Toolkit;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
+import java.awt.event.InputEvent;
+import java.awt.event.KeyEvent;
 import java.io.File;
 import java.io.FilenameFilter;
 import java.util.ArrayList;
@@ -25,6 +27,7 @@ import java.util.ResourceBundle;
 
 import javax.swing.AbstractAction;
 import javax.swing.Action;
+import javax.swing.JFileChooser;
 import javax.swing.JFrame;
 import javax.swing.JMenu;
 import javax.swing.JMenuBar;
@@ -34,6 +37,7 @@ import javax.swing.JRadioButtonMenuItem;
 import javax.swing.JTextPane;
 import javax.swing.KeyStroke;
 import javax.swing.UIManager;
+import javax.swing.filechooser.FileFilter;
 import javax.swing.text.JTextComponent;
 import javax.swing.text.TextAction;
 import javax.swing.undo.CannotRedoException;
@@ -105,7 +109,7 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
     JaxeFrame jaxeframe;
     JFrame jframe;
     
-    static String dernierRepertoire = null;
+    static File dernierRepertoire = null;
     
     
     public JaxeMenuBar(final JaxeFrame jaxeframe) {
@@ -147,12 +151,12 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
         fileMenu = new JMenu(rb.getString("menus.Fichier"));
         
         miNew = new JMenuItem (rb.getString("menus.Nouveau"));
-        miNew.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, cmdMenu));
+        miNew.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_N, cmdMenu));
         fileMenu.add(miNew).setEnabled(true);
         miNew.addActionListener(this);
 
         miOpen = new JMenuItem (rb.getString("menus.Ouvrir"));
-        miOpen.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, cmdMenu));
+        miOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O, cmdMenu));
         fileMenu.add(miOpen).setEnabled(true);
         miOpen.addActionListener(this);
         
@@ -161,16 +165,17 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
         miOpenConf.addActionListener(this);
         
         miClose = new JMenuItem (rb.getString("menus.Fermer"));
-        miClose.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_W, cmdMenu));
+        miClose.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_W, cmdMenu));
         fileMenu.add(miClose).setEnabled(true);
         miClose.addActionListener(this);
         
         miSave = new JMenuItem (rb.getString("menus.Enregistrer"));
-        miSave.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, cmdMenu));
+        miSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, cmdMenu));
         fileMenu.add(miSave).setEnabled(true);
         miSave.addActionListener(this);
         
         miSaveAs = new JMenuItem (rb.getString("menus.EnregistrerSous"));
+        miSaveAs.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, InputEvent.SHIFT_MASK | cmdMenu));
         fileMenu.add(miSaveAs).setEnabled(true);
         miSaveAs.addActionListener(this);
         
@@ -186,7 +191,7 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
             miPref.addActionListener(this);
             
             miQuitter = new JMenuItem (rb.getString("menus.Quitter"));
-            miQuitter.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, cmdMenu));
+            miQuitter.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Q, cmdMenu));
             fileMenu.add(miQuitter).setEnabled(true);
             miQuitter.addActionListener(this);
         }
@@ -199,37 +204,37 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
         editMenu = new JMenu(rb.getString("menus.Edition"));
         
         miUndo = editMenu.add(undoAction);
-        miUndo.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Z, cmdMenu));
+        miUndo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_Z, cmdMenu));
 
         miRedo = editMenu.add(redoAction);
-        miRedo.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_R, cmdMenu));
+        miRedo.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_R, cmdMenu));
 
         editMenu.addSeparator();
         
         aCouper = new ActionCouper();
         miCut = editMenu.add(aCouper);
-        miCut.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_X, cmdMenu));
+        miCut.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_X, cmdMenu));
         if (jframe != jaxeframe)
             miCut.setEnabled(false);
         aCopier = new ActionCopier();
         miCopy = editMenu.add(aCopier);
-        miCopy.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_C, cmdMenu));
+        miCopy.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_C, cmdMenu));
         aColler = new ActionColler();
         miPaste = editMenu.add(aColler);
-        miPaste.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_V, cmdMenu));
+        miPaste.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_V, cmdMenu));
         if (jframe != jaxeframe)
             miPaste.setEnabled(false);
         miSelectAll = editMenu.add(new ActionToutSelectionner());
-        miSelectAll.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, cmdMenu));
+        miSelectAll.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_A, cmdMenu));
         
         editMenu.addSeparator();
 
         miFind = editMenu.add(new ActionRechercher());
-        miFind.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F, cmdMenu));
+        miFind.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F, cmdMenu));
         if (jframe instanceof ValidationFrame)
             miFind.setEnabled(false);
         miAgain = editMenu.add(new ActionSuivant());
-        miAgain.setAccelerator(KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_G, cmdMenu));
+        miAgain.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_G, cmdMenu));
         if (jframe instanceof ValidationFrame)
             miAgain.setEnabled(false);
         
@@ -237,6 +242,13 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
         
         miSpelling = editMenu.add(new ActionOrthographe());
         
+        /*
+        // ne marche pas avec un clavier AZERTY alors que a devrait marcher, cf Java bug 4199399
+        // bizarrement a marche si on met VK_PERIOD au lieu de VK_COLON, mais l'affichage est incorrect
+        if (System.getProperty("os.name").startsWith("Mac OS"))
+            miSpelling.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_PERIOD, cmdMenu));
+        */
+        
         add(editMenu);
     }
     
@@ -334,45 +346,67 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
         }
     }
     
+    /**
+     * Filtre pour JFileChooser slectionnant les fichiers avec l'extension .xml.
+     * javax.swing.filechooser.FileNameExtensionFilter pourra remplacer a, mais c'est du Java 1.6
+     */
+    class XMLFileFilter extends FileFilter {
+        public boolean accept(File f) {
+            if (f == null)
+                return(false);
+            if (f.isDirectory())
+                return(true);
+            return(f.getName().toLowerCase().endsWith(".xml"));
+        }
+        public String getDescription() {
+            return(rb.getString("ouverture.FichiersXML"));
+        }
+    }
+    
     public void doOpen() {
-        /* JFileChooser: avantage: choix affichage fichiers XML/tous les fichiers
-           FileDialog: avantage: dialogue standard du systme
-        JFileChooser chooser = new JFileChooser();
-        FileFilter xmlFilter = new ExampleFileFilter("xml", rb.getString("ouverture.FichiersXML"));
-        chooser.addChoosableFileFilter(xmlFilter);
-        int returnVal = chooser.showOpenDialog(this);
-        if(returnVal == JFileChooser.APPROVE_OPTION) {
-            File theFile = chooser.getSelectedFile();
+        /* JFileChooser: avantage: choix affichage fichiers XML/tous les fichiers sur Windows (pas de pb sur Mac)
+           FileDialog: avantage: dialogue standard du systme mais trs moche sur Linux
+           => utilisation de JFileChooser sur Linux, FileDialog sur Mac et Windows
         */
-        final FileDialog fd = new FileDialog(jframe);
-        if (System.getProperty("os.name").indexOf("Linux") != -1 && dernierRepertoire != null)
-            fd.setDirectory(dernierRepertoire);
-        fd.setVisible(true);
-        final String sf = fd.getFile();
-        if (sf != null) {
-            dernierRepertoire = fd.getDirectory();
-            final File theFile = new File(fd.getDirectory(), sf);
-            Jaxe.ouvrir(theFile, jaxeframe);
+        if (System.getProperty("os.name").indexOf("Linux") != -1) {
+            final JFileChooser chooser = new JFileChooser(dernierRepertoire);
+            final int resultat = chooser.showOpenDialog(jframe);
+            if (resultat == JFileChooser.APPROVE_OPTION) {
+                dernierRepertoire = chooser.getCurrentDirectory();
+                final File theFile = chooser.getSelectedFile();
+                Jaxe.ouvrir(theFile, jaxeframe);
+            }
+        } else {
+            final FileDialog fd = new FileDialog(jframe);
+            fd.setVisible(true);
+            final String sf = fd.getFile();
+            if (sf != null) {
+                final File theFile = new File(fd.getDirectory(), sf);
+                Jaxe.ouvrir(theFile, jaxeframe);
+            }
         }
     }
     
     public void doOpenConf() {
-        /*
-        JFileChooser chooser = new JFileChooser(System.getProperty("user.dir") + File.separator + "config");
-        FileFilter xmlFilter = new ExampleFileFilter("xml", rb.getString("ouverture.FichiersXML"));
-        chooser.addChoosableFileFilter(xmlFilter);
-        int returnVal = chooser.showOpenDialog(this);
-        if(returnVal == JFileChooser.APPROVE_OPTION) {
-            File theFile = chooser.getSelectedFile();
-        */
-        final FileDialog fd = new FileDialog(jframe);
-        fd.setFilenameFilter(new ExtFilter("xml"));
-        fd.setDirectory(System.getProperty("user.dir") + File.separator + "config");
-        fd.setVisible(true);
-        final String sf = fd.getFile();
-        if (sf != null) {
-            final File theFile = new File(fd.getDirectory(), sf);
-            Jaxe.ouvrirConf(theFile, jaxeframe);
+        if (System.getProperty("os.name").indexOf("Linux") != -1) {
+            final JFileChooser chooser = new JFileChooser(System.getProperty("user.dir") + File.separator + "config");
+            final FileFilter xmlFilter = new XMLFileFilter();
+            chooser.addChoosableFileFilter(xmlFilter);
+            final int resultat = chooser.showOpenDialog(jframe);
+            if (resultat == JFileChooser.APPROVE_OPTION) {
+                final File theFile = chooser.getSelectedFile();
+                Jaxe.ouvrirConf(theFile, jaxeframe);
+            }
+        } else {
+            final FileDialog fd = new FileDialog(jframe);
+            fd.setFilenameFilter(new ExtFilter("xml"));
+            fd.setDirectory(System.getProperty("user.dir") + File.separator + "config");
+            fd.setVisible(true);
+            final String sf = fd.getFile();
+            if (sf != null) {
+                final File theFile = new File(fd.getDirectory(), sf);
+                Jaxe.ouvrirConf(theFile, jaxeframe);
+            }
         }
     }
     
@@ -435,7 +469,11 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
             menusExport = new JMenuItem[exports.size()];
             for (final Element refExport : exports) {
                 final Action action = new ActionExport(doc, refExport);
-                fileMenu.insert(action, pos);
+                final JMenuItem item = fileMenu.insert(action, pos);
+                final String exportdoc = doc.cfg.documentationExport(refExport);
+                if (exportdoc != null)
+                    item.setToolTipText(exportdoc);
+                // on pourrait utiliser Config.formatageDoc, sauf que le HTML n'est pas gr sur MacOS X pour la barre de menus de l'appli
                 menusExport[pos-7] = fileMenu.getItem(pos);
                 pos++;
             }
@@ -443,6 +481,21 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
         }
     }
     
+    public void premierExportHTML() {
+        if (jaxeframe.doc.fsave == null) {
+            JOptionPane.showMessageDialog(jaxeframe,
+                JaxeResourceBundle.getRB().getString("html.SauverAvant"),
+                JaxeResourceBundle.getRB().getString("erreur.Erreur"),
+                JOptionPane.ERROR_MESSAGE);
+            return;
+        }
+        final ArrayList<Element> listeExportsHTML = jaxeframe.doc.cfg.listeExports("HTML");
+        if (listeExportsHTML == null || listeExportsHTML.size() == 0)
+            return;
+        final Export exp = new Export(jaxeframe.doc, listeExportsHTML.get(0));
+        exp.transformation(Export.fichierHTML(jaxeframe.doc.fsave), true); // ouvre le navigateur  la fin
+    }
+    
     
     protected class ShowWindowAction extends AbstractAction {
         String titre;
@@ -453,7 +506,7 @@ public class JaxeMenuBar extends JMenuBar implements ActionListener, EcouteurMAJ
 
         public void actionPerformed(final ActionEvent e) {
             if (this.titre.equals(rb.getString("menus.FenetreHTML")))
-                jaxeframe.activerHTMLFrame();
+                premierExportHTML();
             else if (this.titre.equals(rb.getString("menus.Validation")))
                 jaxeframe.activerValidationFrame();
             else if (this.titre.equals(rb.getString("menus.Source"))) {
