@@ -19,6 +19,7 @@ import java.awt.Point;
 import java.awt.Rectangle;
 import java.awt.Toolkit;
 import java.awt.event.ActionEvent;
+import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.util.ArrayList;
 import java.util.Stack;
@@ -118,7 +119,7 @@ public class JaxeTextPane extends JTextPane {
         // attention, cette Keymap n'est pas lie  l'objet, et la modifier peut provoquer des fuites de mmoire
         // (elle garderait des rfrence au textpane quand il n'est plus utilis)
         // on utilise donc static pour ActionMenuContextuel
-        final KeyStroke cmdsp = KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_D, cmdMenu);
+        final KeyStroke cmdsp = KeyStroke.getKeyStroke(KeyEvent.VK_D, cmdMenu);
         kmap.addActionForKeyStroke(cmdsp, new ActionMenuContextuel());
 
         doc.addUndoableEditListener(new MyUndoableEditListener());
@@ -354,7 +355,7 @@ public class JaxeTextPane extends JTextPane {
     }
 
     public void selectZone(final int debut, final int fin, final boolean select, final boolean modsel) {
-        ArrayList<JaxeElement> tel = doc.rootJE.elementsDans(debut, fin - 1);
+        ArrayList<JaxeElement> tel = doc.elementsDans(debut, fin - 1);
         if (select) {
             // on change la slection pour ne pas inclure des moiti d'lments
             // (sauf pour le texte)
@@ -363,10 +364,10 @@ public class JaxeTextPane extends JTextPane {
             
             boolean modif;
             JaxeElement jedebut = doc.elementA(debut);
-            if (jedebut instanceof JETexte || jedebut instanceof JEStyle)
+            while (jedebut instanceof JETexte || jedebut instanceof JEStyle)
                 jedebut = jedebut.getParent();
             JaxeElement jefin = doc.elementA(fin);
-            if (jefin instanceof JETexte || jefin instanceof JEStyle)
+            while (jefin instanceof JETexte || jefin instanceof JEStyle)
                 jefin = jefin.getParent();
             do {
                 modif = false;
@@ -375,7 +376,7 @@ public class JaxeTextPane extends JTextPane {
                     ndebut = jedebut.fin.getOffset() + 1;
                     modif = true;
                     jedebut = doc.elementA(ndebut);
-                    if (jedebut instanceof JETexte || jedebut instanceof JEStyle)
+                    while (jedebut instanceof JETexte || jedebut instanceof JEStyle)
                         jedebut = jedebut.getParent();
                 }
                 if (jefin != null && jefin.debut.getOffset() < nfin &&
@@ -384,7 +385,7 @@ public class JaxeTextPane extends JTextPane {
                     nfin = jefin.debut.getOffset();
                     modif = true;
                     jefin = doc.elementA(nfin);
-                    if (jefin instanceof JETexte || jefin instanceof JEStyle)
+                    while (jefin instanceof JETexte || jefin instanceof JEStyle)
                         jefin = jefin.getParent();
                 }
             } while (modif);
@@ -435,7 +436,7 @@ public class JaxeTextPane extends JTextPane {
                 SwingUtilities.invokeLater(new ChangementSelection(ndebut, nfin));
             }
             if (ndebut != debut || nfin != fin)
-                tel = doc.rootJE.elementsDans(ndebut, nfin - 1);
+                tel = doc.elementsDans(ndebut, nfin - 1);
         }
         for (final JaxeElement je : tel)
             je.selection(select);
@@ -474,8 +475,7 @@ public class JaxeTextPane extends JTextPane {
             ligne--;
         else
             ligne = 0;
-        final int pos = doc.getDefaultRootElement().getElement(ligne)
-                .getStartOffset();
+        final int pos = doc.getDefaultRootElement().getElement(ligne).getStartOffset();
         // bidouille pour afficher la position en haut de la fentre
         try {
             scrollRectToVisible(modelToView(doc.getLength()));
@@ -691,10 +691,7 @@ public class JaxeTextPane extends JTextPane {
     public void suivant() {
         if (dlgRecherche != null) {
             texteRecherche = dlgRecherche.getTexteRecherche();
-            if (dlgRecherche.RechXpath)
-                dlgRecherche.suivantXpath(getSelectionStart());
-            else    
-                dlgRecherche.suivant(getSelectionStart());
+            dlgRecherche.suivant(getSelectionStart());
         }
     }
 
