@@ -45,9 +45,8 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
     boolean hasBeenDone;
     public String texte;
     JaxeDocument doc;
-    public boolean texteDansSuivant;
     public boolean texteSansStyle;
-    private final boolean regrouper;
+    private boolean regrouper;
     
     public JaxeUndoableEdit(final int ajsup, final JaxeElement je) {
         this(ajsup, je, true);
@@ -61,7 +60,6 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
         hasBeenDone = true;
         texte = null;
         doc = je.doc;
-        texteDansSuivant = false;
         texteSansStyle = false;
         if (je instanceof JETexte) {
         // les textes ne doivent pas tre utiliss avec je,
@@ -84,7 +82,7 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
         else if (texte != null)
             longueur = texte.length();
         else
-            longueur = -1;
+            longueur = -1; // cas de l'ajout d'un nouvel lment
     }
     
     // constructeur utile pour insrer/supprimer du texte
@@ -101,12 +99,11 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
         hasBeenDone = true;
         this.texte = texte;
         this.doc = doc;
-        texteDansSuivant = false;
         texteSansStyle = false;
         if (ajsup == SUPPRIMER) {
             final JaxeElement jtexte = doc.elementA(offsetdebut);
-            if (jtexte instanceof JETexte && jtexte.debut.getOffset() == offsetdebut &&
-                    jtexte.fin.getOffset() == offsetdebut + texte.length() - 1) {
+            if (jtexte instanceof JETexte && !(jtexte.getParent() instanceof JEStyle) &&
+                    jtexte.debut.getOffset() == offsetdebut && jtexte.fin.getOffset() == offsetdebut + texte.length() - 1) {
                 je = jtexte;
                 //this.texte = null;
                 texteSansStyle = true;
@@ -118,16 +115,13 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
     public boolean addEdit(final UndoableEdit anEdit) {
         if (anEdit instanceof JaxeUndoableEdit) {
             final JaxeUndoableEdit jedit = (JaxeUndoableEdit)anEdit;
-            if (texte != null && jedit.texte != null && ajsup == jedit.ajsup &&
-                !JaxeDocument.newline.equals(jedit.texte)) {
+            if (texte != null && jedit.texte != null && ajsup == jedit.ajsup && !"\n".equals(texte) && !"\n".equals(jedit.texte)) {
                 if ((ajsup == AJOUTER && jedit.offsetdebut == offsetdebut + longueur) ||
-                        (ajsup == SUPPRIMER && jedit.offsetdebut == offsetdebut && !jedit.texteDansSuivant)) {
+                        (ajsup == SUPPRIMER && jedit.offsetdebut == offsetdebut)) {
                     // absorb !
                     texte += jedit.texte;
                     longueur += jedit.longueur;
-                    if (texteDansSuivant && jedit.texteSansStyle) {
-                        texteDansSuivant = false;
-                        texteSansStyle = true;
+                    if (texteSansStyle && je == null && jedit.je != null) {
                         je = jedit.je;
                         je.noeud.setNodeValue(texte);
                     }
@@ -135,14 +129,13 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
                     return true;
                 }
                 if ((ajsup == AJOUTER && jedit.offsetdebut == offsetdebut) || (ajsup == SUPPRIMER &&
-                        jedit.offsetdebut == offsetdebut - jedit.longueur && !texteDansSuivant)) {
+                        jedit.offsetdebut == offsetdebut - jedit.longueur)) {
                     // autre cas, absorb aussi
                     texte = jedit.texte + texte;
                     longueur += jedit.longueur;
                     if (ajsup == SUPPRIMER)
                         offsetdebut -= jedit.longueur;
-                    texteDansSuivant = jedit.texteDansSuivant;
-                    if (jedit.texteSansStyle) {
+                    if (jedit.texteSansStyle && jedit.je != null) {
                         texteSansStyle = true;
                         je = jedit.je;
                         je.noeud.setNodeValue(texte);
@@ -151,6 +144,7 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
                     return true;
                 }
             }
+            // A FAIRE: absorption de styles
         }
         return false;
     }
@@ -223,19 +217,11 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
     }
     
     private void ajouterNoeud() {
-        if (doc.rootJE == null) {
-            doc.rootJE = je;
-            doc.DOMdoc.appendChild(je.noeud);
-            try {
-                je.creer(doc.createPosition(offsetdebut), je.noeud);
-            } catch (final BadLocationException ex) {
-                LOG.error("ajouterNoeud() - BadLocationException", ex);
-            }
-            return;
-        }
         try {
+            if (doc.rootJE == null && je.noeud.getNodeType() == Node.ELEMENT_NODE)
+                doc.rootJE = je;
             final Position newpos = doc.createPosition(offsetdebut);
-            je.insererDOM(newpos, je.noeud);
+            je.insererDOM(newpos, je.noeud); // insererDOM coupe un style en newpos si ncessaire
             doc.dom2JaxeElement.put(je.noeud, je);
             if (je.noeud.getNodeType() == Node.ELEMENT_NODE && je.refElement == null && doc.cfg != null) {
                 final Node nparent = je.noeud.getParentNode();
@@ -246,106 +232,115 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
             }
             je.init(newpos, je.noeud);
             je.debut = doc.createPosition(offsetdebut);
-            je.fin = doc.createPosition(newpos.getOffset()-1);
+            if (je.fin == null)
+                je.fin = doc.createPosition(newpos.getOffset() - 1);
             
             if (longueur == -1)
                 longueur = je.fin.getOffset() - je.debut.getOffset() + 1;
             
-            // JESwing: mise  jour du dbut des parents
+            // JESwing et JEStyle: mise  jour du dbut des parents
             JaxeElement jeparent = je.getParent();
-            while (jeparent instanceof JESwing && jeparent.debut.getOffset() > offsetdebut) {
+            while ((jeparent instanceof JESwing || jeparent instanceof JEStyle) && jeparent.debut.getOffset() > offsetdebut) {
                 jeparent.debut = je.debut;
                 jeparent = jeparent.getParent();
             }
-            
+            // en dbut du textpane: pb avec la position 0 qui ne change pas pour le noeud suivant
+            if (offsetdebut == 0 && je.noeud.getNextSibling() != null)
+                je.majPosSuivant();
             // cas des styles ou des textes  regrouper
+            jeparent = je.getParent();
+            while (jeparent instanceof JEStyle)
+                jeparent = jeparent.getParent();
             if ((je instanceof JEStyle || je instanceof JETexte) && jeparent != null && regrouper)
                 jeparent.regrouperTextes();
+            if (je instanceof JETexte && "\n".equals(texte))
+                doc.majIndentAjout(offsetdebut);
         } catch (final BadLocationException ex) {
             LOG.error("ajouterNoeud() - BadLocationException", ex);
         }
     }
     
     private void ajouter() {
-        if (je != null && !texteSansStyle) {
+        if (je != null) {
             ajouterNoeud();
+            if (!regrouper)
+                regrouper = true;
+            return;
+        }
+        if (offsetdebut <= 0) {
+            LOG.error("JaxeUndoableEdit.ajouter: offsetdebut <= 0");
             return;
         }
         JaxeElement jeparent = doc.elementA(offsetdebut);
         if (jeparent == null)
             return;
-        if (texteSansStyle && jeparent instanceof JETexte)
-            texteDansSuivant = true;
-        if (je == null || (texteSansStyle && texteDansSuivant)) {
-            // pb: comment savoir si un caractre effac  la fin d'un string
-            // avec un style se trouvait dans ce style ou  ct ?
-            // -> utilisation de texteDansSuivant
-            JaxeElement jesuiv = null;
-            if (jeparent.debut.getOffset() == offsetdebut) {
-                jesuiv = jeparent;
-                jeparent = jeparent.getParent();
-                if (jeparent == null)
-                    return;
-            }
-            // le style du texte ajout est celui de l'lment du caractre prcdent
-            // sauf si texteDansSuivant=true ou que l'lment prcdent n'est pas du texte
-            JaxeElement jestyle;
-            if (jesuiv != null)
+        while (jeparent.getParent() instanceof JEStyle)
+            jeparent = jeparent.getParent();
+        JaxeElement jesuiv = null;
+        if (jeparent.debut.getOffset() == offsetdebut) {
+            jesuiv = jeparent;
+            jeparent = jeparent.getParent();
+            if (jeparent == null)
+                return;
+        }
+        // le style du texte ajout est celui de l'lment du caractre prcdent
+        // sauf si l'lment prcdent n'est pas du texte ou si texteSansStyle
+        JaxeElement jeprec = doc.elementA(offsetdebut-1);
+        if (jeprec.getParent() instanceof JEStyle)
+            jeprec = jeprec.getParent();
+        JaxeElement jestyle;
+        if (texteSansStyle) {
+            if (!(jeprec instanceof JEStyle))
+                jestyle = jeprec;
+            else if (jesuiv != null)
                 jestyle = jesuiv;
             else
                 jestyle = jeparent;
-            if (!texteDansSuivant && offsetdebut > 0) {
-                final JaxeElement jeprec = doc.elementA(offsetdebut-1);
-                if (jeprec instanceof JETexte || jeprec instanceof JEStyle)
-                    jestyle = jeprec;
-                else
-                    jestyle = jeparent;
-            }
-            final SimpleAttributeSet att = jestyle.attStyle(null);
-            try {
-                doc.insertString(offsetdebut, texte, att);
-                if (texteDansSuivant && offsetdebut > 0 && jesuiv != null &&
-                        (jesuiv instanceof JETexte || jesuiv instanceof JEStyle))
-                    jesuiv.debut = doc.createPosition(offsetdebut);
-                else if (jesuiv != null && jeparent instanceof JESwing) {
-                    // JESwing: mise  jour du dbut des parents
-                    if (jesuiv instanceof JESwing)
-                        jesuiv.debut = doc.createPosition(offsetdebut);
-                    JaxeElement jesuivparent = jeparent;
-                    while (jesuivparent instanceof JESwing &&
-                            jesuivparent.debut.getOffset() == offsetdebut+longueur) {
-                        jesuivparent.debut = doc.createPosition(offsetdebut);
-                        jesuivparent = jesuivparent.getParent();
-                    }
-                }
-            } catch (final BadLocationException ex) {
-                LOG.error("ajouter() - BadLocationException", ex);
-            }
-            jeparent.mettreAJourDOM();
-            if (regrouper)
-                jeparent.regrouperTextes();
-            if ("\n".equals(texte))
-                doc.majIndentAjout(offsetdebut);
-        } else { // cas texteSansStyle && !texteDansSuivant
-            // si l'lment a t spcifi comme un je, il faut l'ajouter comme je
-            // pour viter qu'il prenne le style de l'lment prcdent
-            // (sauf s'il y a du texte normal aprs car dans ce cas le texte
-            // est ajout l)
-            ajouterNoeud();
+        } else {
+            if (jeprec instanceof JETexte || jeprec instanceof JEStyle)
+                jestyle = jeprec;
+            else
+                jestyle = jeparent;
         }
+        final SimpleAttributeSet att = jestyle.attStyle(null);
+        try {
+            doc.insertString(offsetdebut, texte, att);
+            if (jestyle == jesuiv && jesuiv instanceof JETexte)
+                jesuiv.debut = doc.createPosition(offsetdebut);
+            else if (jesuiv != null && jeparent instanceof JESwing) {
+                // JESwing: mise  jour du dbut des parents
+                if (jesuiv instanceof JESwing)
+                    jesuiv.debut = doc.createPosition(offsetdebut);
+                JaxeElement jesuivparent = jeparent;
+                while (jesuivparent instanceof JESwing && jesuivparent.debut.getOffset() == offsetdebut + longueur) {
+                    jesuivparent.debut = doc.createPosition(offsetdebut);
+                    jesuivparent = jesuivparent.getParent();
+                }
+            }
+        } catch (final BadLocationException ex) {
+            LOG.error("ajouter() - BadLocationException", ex);
+        }
+        jeparent.mettreAJourDOM();
+        if (regrouper)
+            jeparent.regrouperTextes();
+        if ("\n".equals(texte))
+            doc.majIndentAjout(offsetdebut);
+        if (!regrouper)
+            regrouper = true; // les groupements ne posent pas de problme pour les annulations (parce-que offsetdebut et longueur sont dfinis)
     }
     
     private void effacer() {
         JaxeElement jedebut = null;
         if ((je instanceof JETexte || je instanceof JEStyle) && je != doc.rootJE) {
             jedebut = doc.elementA(offsetdebut);
+            while (jedebut.getParent() instanceof JEStyle)
+                jedebut = jedebut.getParent();
             // le je peut avoir t modifi  cause des regroupements de texte : on vrifie
             // et on coupe si ncessaire (le parent est null si le je a t regroup avec un texte  sa gauche)
             if (je.getParent() == null) {
-                if (jedebut.debut.getOffset() < offsetdebut) {
+                if (jedebut.debut.getOffset() < offsetdebut) { // pour corriger un regroupement  gauche
                     try {
                         je = jedebut.couper(doc.textPane.getDocument().createPosition(offsetdebut));
-                        jedebut = je;
                     } catch (final BadLocationException ex) {
                         LOG.error("effacer() - BadLocationException", ex);
                     }
@@ -361,7 +356,7 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
                     LOG.error("effacer: offsetdebut != je.debut.getOffset()");
                 }
             }
-            if (longueur < je.fin.getOffset() - je.debut.getOffset() + 1) {
+            if (longueur < je.fin.getOffset() - je.debut.getOffset() + 1) { // pour corriger un regroupement  droite
                 try {
                     je.couper(doc.textPane.getDocument().createPosition(offsetdebut + longueur));
                 } catch (final BadLocationException ex) {
@@ -371,56 +366,94 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
             if (longueur > je.fin.getOffset() - je.debut.getOffset() + 1)
                 LOG.error("effacer: longueur > je.fin.getOffset() - je.debut.getOffset() + 1");
         }
-        if (texte != null) {
-            if (jedebut == null)
-                jedebut = doc.elementA(offsetdebut);
-            if (jedebut.debut.getOffset() == offsetdebut &&
-                    jedebut.fin.getOffset() > offsetdebut + longueur - 1)
-                texteDansSuivant = true;
-            if (jedebut instanceof JETexte || jedebut instanceof JEStyle) {
-                if (offsetdebut <= jedebut.debut.getOffset() && jedebut.fin.getOffset() <
-                    offsetdebut + longueur)
-                    jedebut.getParent().supprimerEnfantDOM(jedebut);
-                else if (offsetdebut + longueur > jedebut.fin.getOffset()) {
-                    try {
-                        jedebut.fin = doc.createPosition(offsetdebut - 1);
-                    } catch (final BadLocationException ex) {
-                        LOG.error("effacer() - BadLocationException", ex);
-                    }
+        // si un style est au dbut ou  la fin d'un style parent il faut couper  droite ou  gauche pour pouvoir annuler aprs
+        // (sinon on perd l'information que le style tait un enfant du style parent)
+        if (texte != null)
+            jedebut = doc.elementA(offsetdebut);
+        else
+            jedebut = je;
+        JaxeElement jeparent = jedebut.getParent();
+        if (jeparent instanceof JEStyle) {
+            while (jeparent.getParent() instanceof JEStyle)
+                jeparent = jeparent.getParent();
+            if (jeparent.fin.getOffset() >= offsetdebut + longueur) {
+                try {
+                    jeparent.couper(doc.textPane.getDocument().createPosition(offsetdebut + longueur));
+                } catch (final BadLocationException ex) {
+                    LOG.error("effacer() - BadLocationException", ex);
+                }
+                je = jeparent;
+            }
+            if (jeparent.debut.getOffset() < offsetdebut) {
+                try {
+                    je = jeparent.couper(doc.textPane.getDocument().createPosition(offsetdebut));
+                } catch (final BadLocationException ex) {
+                    LOG.error("effacer() - BadLocationException", ex);
                 }
             }
+            // quand le parent est un style, on efface toujours un noeud de style (aprs des coupures ventuelles)
+            if (je == null || je instanceof JETexte)
+                je = jeparent;
+            texte = null;
+            texteSansStyle = false;
+        }
+        if (texte != null) {
+            texteSansStyle = true;
             try {
+                if (jedebut instanceof JETexte) { // autre cas: commentaires XML
+                    if (offsetdebut <= jedebut.debut.getOffset() && jedebut.fin.getOffset() < offsetdebut + longueur) {
+                        jeparent = jedebut.getParent();
+                        jeparent.supprimerEnfantDOM(jedebut);
+                    } else if (offsetdebut + longueur > jedebut.fin.getOffset()) {
+                        // cas o on efface le texte jusqu' la fin de jedebut (offsetdebut + longueur == jedebut.fin.getOffset() + 1)
+                        jedebut.fin = doc.createPosition(offsetdebut - 1);
+                    }
+                }
                 doc.remove(offsetdebut, longueur, false);
             } catch (final BadLocationException ex) {
                 LOG.error("effacer() - BadLocationException", ex);
             }
-            JaxeElement jeparent = doc.elementA(offsetdebut);
+            jeparent = doc.elementA(offsetdebut);
+            while (jeparent.getParent() instanceof JEStyle)
+                jeparent = jeparent.getParent();
             if (jeparent.debut.getOffset() == offsetdebut)
                 jeparent = jeparent.getParent();
             jeparent.mettreAJourDOM();
             if ("\n".equals(texte))
                 doc.majIndentSupp(offsetdebut);
+            if (!regrouper)
+                regrouper = true;
         } else {
             JaxeElement parent = je.getParent();
             if (parent != null) {
                 je.effacer();
+                if (je instanceof JEStyle && parent instanceof JEStyle) {
+                    while (parent instanceof JEStyle && parent.debut.getOffset() == je.debut.getOffset() && parent.fin.getOffset() == je.fin.getOffset()) {
+                        je = parent;
+                        parent = je.getParent();
+                        je.effacer();
+                    }
+                }
                 parent.supprimerEnfant(je);
                 if (regrouper)
                     parent.regrouperTextes();
+                if (!regrouper)
+                    regrouper = true; // les groupements ne posent pas de problme pour les annulations
                 parent.majValidite();
-            } else if (je == doc.rootJE) {
+            } else {
+                // cas de la racine et des commentaires et PI au mme niveau
                 je.effacer();
                 final int len = je.fin.getOffset() - je.debut.getOffset() + 1;
                 final int idebut = je.debut.getOffset();
                 try {
                     doc.remove(idebut, len, false);
-                    doc.rootJE = null;
+                    if (je == doc.rootJE)
+                        doc.rootJE = null;
                     doc.DOMdoc.removeChild(je.noeud);
                 } catch (final BadLocationException ex) {
                     LOG.error("effacer() - BadLocationException", ex);
                 }
-            } else
-                LOG.error("effacer() - je.getParent() == null && je != doc.rootJE");
+            }
         }
         if (je != null)
             doc.dom2JaxeElement.remove(je.noeud);
@@ -492,7 +525,7 @@ public class JaxeUndoableEdit extends Object implements UndoableEdit {
     
     public String toString() { // pour debug
         return("ajsup=" + ajsup + " je=" + je + " offsetdebut=" + offsetdebut + " longueur=" + longueur +
-            " hasBeenDone=" + hasBeenDone + " texte=" + texte + " texteDansSuivant=" + texteDansSuivant +
+            " hasBeenDone=" + hasBeenDone + " texte=" + texte +
             " texteSansStyle=" + texteSansStyle + " regrouper=" + regrouper);
     }
 }
