@@ -14,12 +14,7 @@ package jaxe;
 
 import org.apache.log4j.Logger;
 
-import java.awt.BorderLayout;
-import java.awt.Component;
-import java.awt.Dimension;
-import java.awt.FileDialog;
-import java.awt.FlowLayout;
-import java.awt.Frame;
+import java.awt.*;
 import java.awt.Rectangle;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
@@ -30,15 +25,12 @@ import java.io.FilenameFilter;
 import java.io.IOException;
 import java.security.AccessControlException;
 import java.util.Properties;
+import java.util.prefs.BackingStoreException;
+// java.util.prefs.Preferences
+
+import javax.swing.*;
+import javax.swing.filechooser.FileFilter;
 
-import javax.swing.BorderFactory;
-import javax.swing.BoxLayout;
-import javax.swing.JButton;
-import javax.swing.JCheckBox;
-import javax.swing.JDialog;
-import javax.swing.JFrame;
-import javax.swing.JLabel;
-import javax.swing.JPanel;
 
 /**
  * Prfrences:
@@ -49,7 +41,9 @@ import javax.swing.JPanel;
  *     navigateur
  *     consIndent
  *     iconeValide
+ *     toujoursAfficherAttributs
  *     dictionnaire
+ *     classeXSLT
  */
 public class Preferences extends JDialog implements ActionListener {
     /**
@@ -57,7 +51,10 @@ public class Preferences extends JDialog implements ActionListener {
      */
     private static final Logger LOG = Logger.getLogger(Preferences.class);
 
-    private static final File fpref = trouverFichier();
+    private static final String[] nomsPrefs = {"fenetreArbre", "fenetreInsertion", "fenetreAttributs",
+        "navigateur", "consIndent", "indenter", "iconeValide", "toujoursAfficherAttributs", "dictionnaire", "classeXSLT"};
+    private static final String[] defPrefs = {"true", "true", "true",
+        null, "false", "false", "true", "true", null, "org.apache.xalan.processor.TransformerFactoryImpl"};
     private static Properties prefs = getPreferencesParDefaut();
     
     private final JCheckBox caseArbre;
@@ -65,10 +62,25 @@ public class Preferences extends JDialog implements ActionListener {
     private final JCheckBox caseAttributs;
     private final JLabel labelNav;
     private String prefNav;
-    private final JCheckBox caseIndent;
+    private final JCheckBox caseConsIndent;
+    private final JCheckBox caseIndenter;
+    private final JCheckBox caseIconeValide;
+    private final JCheckBox caseAfficherAttributs;
     private final JLabel labelDico;
     private String prefDico;
-    private final JCheckBox caseIconeValide;
+    private final JComboBox choixXSLT;
+    private final static String[] classesXSLT = {
+        "",
+        "org.apache.xalan.processor.TransformerFactoryImpl",
+        "org.apache.xalan.xsltc.trax.TransformerFactoryImpl",
+        "net.sf.saxon.TransformerFactoryImpl"
+    };
+    private final static String[] nomsXSLT = {
+        JaxeResourceBundle.getRB().getString("pref.defaut"),
+        "Xalan",
+        "XSLTC",
+        "Saxon"
+    };
     
     public static File trouverFichier() {
         try {
@@ -90,29 +102,43 @@ public class Preferences extends JDialog implements ActionListener {
     
     public static Properties getPreferencesParDefaut() {
         final Properties defauts = new Properties();
-        defauts.setProperty("fenetreArbre", "true");
-        defauts.setProperty("fenetreInsertion", "true");
-        defauts.setProperty("fenetreAttributs", "true");
-        //defauts.setProperty("navigateur", null);
+        for (int i=0; i<nomsPrefs.length; i++) {
+            final String nom = nomsPrefs[i];
+            final String valeur = defPrefs[i];
+            if (valeur != null)
+                defauts.setProperty(nom, valeur);
+        }
         if (System.getProperty("os.name").startsWith("Mac"))
             defauts.setProperty("navigateur", "/Applications/Safari.app");
-        defauts.setProperty("consIndent", "false");
-        //defauts.setProperty("dictionnaire", null);
-        defauts.setProperty("iconeValide", "true");
         return(defauts);
     }
     
     public static Properties chargerPref() {
         prefs = getPreferencesParDefaut();
         try {
-            if (fpref != null && fpref.exists()) {
-                final FileInputStream fis = new FileInputStream(fpref);
-                prefs.load(fis);
-                fis.close();
+            final java.util.prefs.Preferences juprefs = java.util.prefs.Preferences.userNodeForPackage(Jaxe.class);
+            if (juprefs.get("fenetreArbre", null) != null) {
+                for (String nomPref : nomsPrefs) {
+                    final String valeur = juprefs.get(nomPref, null);
+                    if (valeur != null)
+                        prefs.setProperty(nomPref, valeur);
+                }
+            } else {
+                try {
+                    final File fpref = trouverFichier();
+                    if (fpref != null && fpref.exists()) {
+                        final FileInputStream fis = new FileInputStream(fpref);
+                        prefs.load(fis);
+                        fis.close();
+                    }
+                } catch (final IOException ex) {
+                    LOG.error("chargerPref()", ex);
+                    return(null);
+                }
+                enregistrerPref(prefs);
             }
-        } catch (final IOException ex) {
+        } catch (SecurityException ex) { // pour les applets
             LOG.error("chargerPref()", ex);
-            return(null);
         }
         return(prefs);
     }
@@ -125,6 +151,20 @@ public class Preferences extends JDialog implements ActionListener {
         if (prefs1 == null)
             return;
         prefs = prefs1;
+        final java.util.prefs.Preferences juprefs = java.util.prefs.Preferences.userNodeForPackage(Jaxe.class);
+        for (String nomPref : nomsPrefs) {
+            final String valeur = prefs.getProperty(nomPref);
+            if (valeur != null)
+                juprefs.put(nomPref, valeur);
+            else if (juprefs.get(nomPref, null) != null)
+                juprefs.remove(nomPref);
+        }
+        try {
+            juprefs.flush();
+        } catch (BackingStoreException ex) {
+            LOG.error("enregistrerPref(Properties)", ex);
+        }
+        /*
         try {
             final FileOutputStream fos = new FileOutputStream(fpref);
             prefs.store(fos, "Prfrences de Jaxe");
@@ -132,6 +172,7 @@ public class Preferences extends JDialog implements ActionListener {
         } catch (final IOException ex) {
             LOG.error("enregistrerPref(Properties)", ex);
         }
+        */
     }
     
     public Preferences(final JFrame jframe) {
@@ -143,9 +184,12 @@ public class Preferences extends JDialog implements ActionListener {
         final String prefInsertion = prefs.getProperty("fenetreInsertion");
         final String prefAttributs = prefs.getProperty("fenetreAttributs");
         prefNav = prefs.getProperty("navigateur");
-        final String prefIndent = prefs.getProperty("consIndent");
-        prefDico = prefs.getProperty("dictionnaire");
+        final String prefConsIndent = prefs.getProperty("consIndent");
+        final String prefIndenter = prefs.getProperty("indenter");
         final String prefIconeValide = prefs.getProperty("iconeValide");
+        final String prefAfficherAttributs = prefs.getProperty("toujoursAfficherAttributs");
+        prefDico = prefs.getProperty("dictionnaire");
+        final String prefClasseXSLT = prefs.getProperty("classeXSLT");
         
         final JPanel prefPanes = new JPanel();
         prefPanes.setLayout(new BoxLayout(prefPanes, BoxLayout.Y_AXIS));
@@ -173,9 +217,11 @@ public class Preferences extends JDialog implements ActionListener {
         final JPanel navPane = new JPanel(new FlowLayout());
         navPane.setBorder(BorderFactory.createTitledBorder(
             JaxeResourceBundle.getRB().getString("pref.Navigateur")));
-        String nomNav = null;
+        String nomNav;
         if (prefNav != null)
             nomNav = (new File(prefNav)).getName();
+        else
+            nomNav = "";
         labelNav = new JLabel(nomNav);
         navPane.add(labelNav);
         final JButton defNav = new JButton(
@@ -189,28 +235,48 @@ public class Preferences extends JDialog implements ActionListener {
         final JPanel enrPane = new JPanel(new FlowLayout());
         enrPane.setLayout(new BoxLayout(enrPane, BoxLayout.Y_AXIS));
         enrPane.setBorder(BorderFactory.createTitledBorder(
-            JaxeResourceBundle.getRB().getString("pref.Affichage")));
-        caseIndent = new JCheckBox(
+            JaxeResourceBundle.getRB().getString("pref.OuvertureEnregistrement")));
+        caseConsIndent = new JCheckBox(
             JaxeResourceBundle.getRB().getString("pref.consIndent"));
-        caseIndent.setSelected(Boolean.parseBoolean(prefIndent));
-        enrPane.add(caseIndent);
+        caseConsIndent.setSelected(Boolean.parseBoolean(prefConsIndent));
+        caseConsIndent.addActionListener(this);
+        caseConsIndent.setActionCommand("consIndent");
+        enrPane.add(caseConsIndent);
+        caseIndenter = new JCheckBox(
+            JaxeResourceBundle.getRB().getString("pref.indenter"));
+        caseIndenter.setSelected(Boolean.parseBoolean(prefIndenter));
+        enrPane.add(caseIndenter);
+        prefPanes.add(enrPane);
+        enrPane.setAlignmentX(Component.LEFT_ALIGNMENT);
+        enrPane.setMaximumSize(new Dimension(Short.MAX_VALUE,Short.MAX_VALUE));
+        
+        final JPanel affPane = new JPanel(new FlowLayout());
+        affPane.setLayout(new BoxLayout(affPane, BoxLayout.Y_AXIS));
+        affPane.setBorder(BorderFactory.createTitledBorder(
+            JaxeResourceBundle.getRB().getString("pref.Affichage")));
         caseIconeValide = new JCheckBox(
             JaxeResourceBundle.getRB().getString("pref.iconeValide"));
         caseIconeValide.setSelected(Boolean.parseBoolean(prefIconeValide));
-        enrPane.add(caseIconeValide);
-        prefPanes.add(enrPane);
-        enrPane.setAlignmentX(Component.LEFT_ALIGNMENT);
+        affPane.add(caseIconeValide);
+        caseAfficherAttributs = new JCheckBox(
+            JaxeResourceBundle.getRB().getString("pref.toujoursAfficherAttributs"));
+        caseAfficherAttributs.setSelected(Boolean.parseBoolean(prefAfficherAttributs));
+        affPane.add(caseAfficherAttributs);
+        prefPanes.add(affPane);
+        affPane.setAlignmentX(Component.LEFT_ALIGNMENT);
+        affPane.setMaximumSize(new Dimension(Short.MAX_VALUE,Short.MAX_VALUE));
         
         final JPanel dicoPane = new JPanel(new FlowLayout());
         dicoPane.setBorder(BorderFactory.createTitledBorder(
             JaxeResourceBundle.getRB().getString("pref.Dictionnaire")));
-        String nomDico = null;
+        String nomDico;
         if (prefDico != null) {
             nomDico = (new File(prefDico)).getName();
             final int pp = nomDico.lastIndexOf('.');
             if (pp != -1)
                 nomDico = nomDico.substring(0, pp);
-        }
+        } else
+            nomDico = "";
         labelDico = new JLabel(nomDico);
         dicoPane.add(labelDico);
         final JButton defDico = new JButton(
@@ -221,6 +287,20 @@ public class Preferences extends JDialog implements ActionListener {
         prefPanes.add(dicoPane);
         dicoPane.setAlignmentX(Component.LEFT_ALIGNMENT);
         
+        final JPanel xsltPane = new JPanel(new FlowLayout());
+        xsltPane.setBorder(BorderFactory.createTitledBorder(
+            JaxeResourceBundle.getRB().getString("pref.XSLT")));
+        choixXSLT = new JComboBox(nomsXSLT);
+        int indexXSLT = 0;
+        for (int i=0; i<classesXSLT.length; i++) {
+            if (classesXSLT[i].equals(prefClasseXSLT))
+                indexXSLT = i;
+        }
+        choixXSLT.setSelectedIndex(indexXSLT);
+        xsltPane.add(choixXSLT);
+        prefPanes.add(xsltPane);
+        xsltPane.setAlignmentX(Component.LEFT_ALIGNMENT);
+        
         this.getContentPane().add(prefPanes, BorderLayout.CENTER);
         
         final JPanel bPane = new JPanel(new FlowLayout(FlowLayout.CENTER, 15, 15));
@@ -259,52 +339,67 @@ public class Preferences extends JDialog implements ActionListener {
             
             prefs.setProperty("fenetreAttributs", Boolean.toString(caseAttributs.isSelected()));
              
-            if (prefNav != null) {
+            if (prefNav != null)
                 prefs.setProperty("navigateur", prefNav);
-            } else {
-                prefs.setProperty("navigateur", "");
-            }
+            else if (prefs.getProperty("navigateur") != null)
+                prefs.remove("navigateur");
             
-            if (prefDico != null) {
+            if (prefDico != null)
                 prefs.setProperty("dictionnaire", prefDico);
-            } else {
-                prefs.setProperty("dictionnaire", "");
-            }
+            else if (prefs.getProperty("dictionnaire") != null)
+                prefs.remove("dictionnaire");
             
-            prefs.setProperty("consIndent", Boolean.toString(caseIndent.isSelected()));
+            prefs.setProperty("consIndent", Boolean.toString(caseConsIndent.isSelected()));
+            prefs.setProperty("indenter", Boolean.toString(caseIndenter.isSelected()));
             final String prefIconeValide = prefs.getProperty("iconeValide");
             prefs.setProperty("iconeValide", Boolean.toString(caseIconeValide.isSelected()));
+            prefs.setProperty("toujoursAfficherAttributs", Boolean.toString(caseAfficherAttributs.isSelected()));
+            
+            prefs.setProperty("classeXSLT", classesXSLT[choixXSLT.getSelectedIndex()]);
             
             Preferences.enregistrerPref(prefs);
             
             if (Boolean.parseBoolean(prefIconeValide) != caseIconeValide.isSelected())
-                for (final JaxeFrame jf : Jaxe.allFrames) {
-                    if (jf.getTextPane() != null)
-                        jf.getTextPane().setIconeValide(caseIconeValide.isSelected());
-                }
+                Jaxe.setIconeValide(caseIconeValide.isSelected());
             
         } else if ("defNav".equals(cmd)) {
             defNavigateur();
         } else if ("defDico".equals(cmd)) {
             defDictionnaire();
+        } else if ("consIndent".equals(cmd)) {
+            if (caseConsIndent.isSelected() && caseIndenter.isSelected())
+                caseIndenter.setSelected(false);
+            caseIndenter.setEnabled(!caseConsIndent.isSelected());
         }
         if ("Enregistrer".equals(cmd) || "Annuler".equals(cmd))
             setVisible(false);
     }
     
     public void defNavigateur() {
-        final FileDialog fdlg = new FileDialog((Frame)getOwner(),
-            JaxeResourceBundle.getRB().getString("pref.DefNavigateur"), FileDialog.LOAD);
-        fdlg.setVisible(true);
         String chemin = null;
-        String dir = fdlg.getDirectory();
-        if (dir != null && dir.endsWith(File.separator))
-            dir = dir.substring(0, dir.length()-1);
-        final String nom = fdlg.getFile();
-        if (dir == null)
-            chemin = nom;
-        else if (nom != null)
-            chemin = dir + File.separator + nom;
+        String nom = null;
+        if (System.getProperty("os.name").indexOf("Linux") != -1) {
+            final JFileChooser chooser = new JFileChooser();
+            chooser.setDialogTitle(JaxeResourceBundle.getRB().getString("pref.DefNavigateur"));
+            final int resultat = chooser.showOpenDialog((Frame)getOwner());
+            if (resultat == JFileChooser.APPROVE_OPTION) {
+                final File f = chooser.getSelectedFile();
+                chemin = f.getAbsolutePath();
+                nom = f.getName();
+            }
+        } else {
+            final FileDialog fdlg = new FileDialog((Frame)getOwner(),
+                JaxeResourceBundle.getRB().getString("pref.DefNavigateur"), FileDialog.LOAD);
+            fdlg.setVisible(true);
+            String dir = fdlg.getDirectory();
+            if (dir != null && dir.endsWith(File.separator))
+                dir = dir.substring(0, dir.length()-1);
+            nom = fdlg.getFile();
+            if (dir == null)
+                chemin = nom;
+            else if (nom != null)
+                chemin = dir + File.separator + nom;
+        }
         if (chemin != null) {
             prefNav = chemin;
             labelNav.setText(nom);
@@ -312,20 +407,46 @@ public class Preferences extends JDialog implements ActionListener {
     }
     
     public void defDictionnaire() {
-        final FileDialog fdlg = new FileDialog((Frame)getOwner(),
-            JaxeResourceBundle.getRB().getString("pref.Dictionnaire"), FileDialog.LOAD);
-        fdlg.setFilenameFilter(new ExtFilter("dico"));
-        fdlg.setDirectory(System.getProperty("user.dir") + File.separator + "dicos");
-        fdlg.setVisible(true);
         String chemin = null;
-        String dir = fdlg.getDirectory();
-        if (dir != null && dir.endsWith(File.separator))
-            dir = dir.substring(0, dir.length()-1);
-        String nom = fdlg.getFile();
-        if (dir == null)
-            chemin = nom;
-        else if (nom != null)
-            chemin = dir + File.separator + nom;
+        String nom = null;
+        final String osName = System.getProperty("os.name");
+        if (osName.indexOf("Linux") != -1 || osName.indexOf("Windows") != -1) {
+            // FileDialog.setFilenameFilter() ne marche pas sur Windows -> on utilise JFileChooser  la place
+            final JFileChooser chooser = new JFileChooser(new File(System.getProperty("user.dir"), "dicos"));
+            chooser.setDialogTitle(JaxeResourceBundle.getRB().getString("pref.Dictionnaire"));
+            chooser.addChoosableFileFilter(new FileFilter() {
+                public boolean accept(File f) {
+                    if (f == null)
+                        return(false);
+                    if (f.isDirectory())
+                        return(true);
+                    return(f.getName().toLowerCase().endsWith(".dico"));
+                }
+                public String getDescription() {
+                    return(".dico");
+                }
+            });
+            final int resultat = chooser.showOpenDialog((Frame)getOwner());
+            if (resultat == JFileChooser.APPROVE_OPTION) {
+                final File f = chooser.getSelectedFile();
+                chemin = f.getAbsolutePath();
+                nom = f.getName();
+            }
+        } else {
+            final FileDialog fdlg = new FileDialog((Frame)getOwner(),
+                JaxeResourceBundle.getRB().getString("pref.Dictionnaire"), FileDialog.LOAD);
+            fdlg.setFilenameFilter(new ExtFilter("dico"));
+            fdlg.setDirectory(System.getProperty("user.dir") + File.separator + "dicos");
+            fdlg.setVisible(true);
+            String dir = fdlg.getDirectory();
+            if (dir != null && dir.endsWith(File.separator))
+                dir = dir.substring(0, dir.length()-1);
+            nom = fdlg.getFile();
+            if (dir == null)
+                chemin = nom;
+            else if (nom != null)
+                chemin = dir + File.separator + nom;
+        }
         if (chemin != null) {
             prefDico = chemin;
             if (nom != null) {
