@@ -37,13 +37,15 @@ import com.thaiopensource.validate.ValidationDriver;
 import com.thaiopensource.validate.rng.RngProperty;
 import com.thaiopensource.xml.sax.DraconianErrorHandler;
 
+import jaxe.wxs.WXSSimpleType;
+
 
 /**
  * Schma Relax NG : validation base sur Jing
  */
 public class SchemaRelaxNG implements InterfaceSchema {
     
-    private static final Logger LOG = Logger.getLogger(SchemaW3C.class);
+    private static final Logger LOG = Logger.getLogger(SchemaRelaxNG.class);
     
     private static final ResourceBundle rb = JaxeResourceBundle.getRB();
     
@@ -256,6 +258,8 @@ public class SchemaRelaxNG implements InterfaceSchema {
      * Bug avec RelaxNG et les modles qui dpendent des valeurs des attributs...
      */
     public Element referenceElement(final Element el, final Element refParent) {
+        // GROS PROBLEME : UN ELEMENT D'UN DOCUMENT PEUT CORRESPONDRE A DEUX DEFINITIONS DANS LE SCHEMA
+        // (la premire est renvoye ici)
         if (refParent == null)
             return(referenceElement(el)); // pour les lments racine
         final ArrayList<Element> liste = listeSousElements(refParent);
@@ -264,7 +268,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         for (final Element ref : liste) {
             if (nomElement(ref).equals(nom)) {
                 final String espaceRef = espaceElement(ref);
-                if ((espace == null && espaceRef.equals("")) || (espace != null && espace.equals(espaceRef)))
+                if ((espace == null && espaceRef == null) || (espace != null && espace.equals(espaceRef)))
                     return(ref);
             }
         }
@@ -280,7 +284,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         Node enfant = refElement.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("name"))
                     return(enfant.getFirstChild().getNodeValue());
             }
@@ -291,7 +295,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
     
     /**
      * Renvoie l'espace de nom de l'lment dont la rfrence est passe en paramtre,
-     * un String vide si targetNamespace est vide, ou null si l'espace de noms n'est pas dfini.
+     * ou null si l'espace de noms n'est pas dfini.
      */
     public String espaceElement(final Element refElement) {
         if (!"".equals(refElement.getAttribute("ns")))
@@ -300,18 +304,18 @@ public class SchemaRelaxNG implements InterfaceSchema {
             return(refElement.lookupNamespaceURI(prefixeString(refElement.getAttribute("name"))));
         else if (refElement.getParentNode() != null)
             return(espaceElement((Element)refElement.getParentNode()));
-        return("");
+        return(null);
     }
     
     /**
      * Renvoie la documentation d'un lment dont on donne la rfrence
-     * (sous forme de texte simple, ou de HTML 3 pour faire des sauts de lignes)
+     * (sous forme de texte simple, avec des \n pour faire des sauts de lignes)
      */
     public String documentationElement(final Element refElement) {
         Node enfant = refElement.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("documentation"))
                     return(enfant.getFirstChild().getNodeValue());
             }
@@ -329,7 +333,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         Node enfant = refElement.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("choice")) {
                     ArrayList<String> liste2 = listeValeursAttribut((Element)enfant);
                     if (liste2 != null)
@@ -344,6 +348,46 @@ public class SchemaRelaxNG implements InterfaceSchema {
         return(new ArrayList<String>(liste));
     }
     
+    /**
+     * Renvoie true si la valeur donne est une valeur valide pour l'lment
+     */
+    public boolean valeurElementValide(final Element refElement, final String valeur) {
+        return(verifType(refElement, valeur));
+    }
+    
+    private boolean verifType(final Element truc, final String valeur) {
+        // implmentation: on ne gre que les types WXS
+        //  amliorer
+        ArrayList<Element> ldata = enfants(truc, "data");
+        if (ldata.size() > 1)
+            return(true);
+        if (ldata.size() == 0) {
+            ArrayList<Element> lref = enfants(truc, "ref");
+            if (lref.size() != 1)
+                return(true);
+            final Element ref = lref.get(0);
+            final String nomRef = ref.getAttribute("name");
+            for (final Element def : ltousdefine) {
+                if (def.getAttribute("name").equals(nomRef))
+                    return(verifType(def, valeur));
+            }
+        }
+        final Element data = ldata.get(0);
+        final String type = data.getAttribute("type");
+        String library = null;
+        Element parent = data;
+        while (parent != null) {
+            if (!"".equals(parent.getAttribute("datatypeLibrary"))) {
+                library = parent.getAttribute("datatypeLibrary");
+                break;
+            }
+            parent = (Element)parent.getParentNode();
+        }
+        if ("http://www.w3.org/2001/XMLSchema-datatypes".equals(library))
+            return(WXSSimpleType.validerValeur(type, valeur));
+        return(true);
+    }
+    
     /**
      * Renvoie le prfixe  utiliser pour crer un lment dont on donne la rfrence,
      * ou null s'il n'y en a pas.
@@ -393,9 +437,15 @@ public class SchemaRelaxNG implements InterfaceSchema {
     }
     
     /**
-     * Renvoie l'espace de noms cible du schma (attribut targetNamespace avec WXS)
+     * Renvoie l'espace de noms cible du schma (attribut targetNamespace avec WXS).
+     * Attention: Le concept d'un espace cible unique pour un schma n'existe pas avec Relax NG.
      */
     public String espaceCible() {
+        // PROBLEME: LE CONCEPT D'UN ESPACE CIBLE UNIQUE N'EXISTE PAS AVEC LES SCHEMAS RELAX NG
+        // un schma donn peut dfinir plusieurs espaces de nom,
+        // et inversement un espace de noms peut tre dfini sur plusieurs schmas.
+        
+        // Ici on renvoie la valeur de l'attribut ns de l'lment racine.
         return(targetNamespace);
     }
     
@@ -406,7 +456,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         final ArrayList<Element> liste = new ArrayList<Element>();
         for (final Element el : elementsNommes) {
             final String tns = espaceElement(el);
-            if (tns != null && !"".equals(tns) && !tns.equals(espace))
+            if (tns != null && !tns.equals(espace))
                 liste.add(el);
         }
         return(liste);
@@ -439,17 +489,12 @@ public class SchemaRelaxNG implements InterfaceSchema {
      * Renvoit true si l'enfant est obligatoire sous le parent.
      */
     public boolean elementObligatoire(final Element refParent, final Element refEnfant) {
-        //  amliorer
-        Node np = refEnfant;
-        while (true) {
-            np = np.getParentNode();
-            if (np == null)
-                return(false);
-            if ("optional".equals(np.getLocalName()))
-                return(false);
-            if (np == refParent)
-                return(true);
+        final Boolean b = trucObligatoire(refParent, refEnfant, null);
+        if (b == null) {
+            LOG.error("enfant non trouv sous un parent: " + nomElement(refEnfant) + " sous " + nomElement(refParent));
+            return(false);
         }
+        return(b.booleanValue());
     }
     
     /**
@@ -469,7 +514,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         Node enfant = refParent.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("element") && !"".equals(((Element)enfant).getAttribute("name")))
                     liste.add((Element)enfant);
                 else if (nom.equals("zeroOrMore") || nom.equals("oneOrMore") || nom.equals("choice") ||
@@ -571,7 +616,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         if (refElement.getParentNode() == null)
             return(liste);
         final Element parent = (Element)refElement.getParentNode();
-        final String nomParent = parent.getNodeName();
+        final String nomParent = parent.getLocalName();
         if ("element".equals(nomParent))
             liste.add(parent);
         else if (nomParent.equals("zeroOrMore") || nomParent.equals("oneOrMore") ||
@@ -593,14 +638,16 @@ public class SchemaRelaxNG implements InterfaceSchema {
      * on donne la rfrence en paramtre
      */
     public ArrayList<Element> listeAttributs(final Element refElement) {
+        // PROBLEME : COMME AVEC LES ELEMENTS, UN ATTRIBUT PEUT CORRESPONDRE A DEUX DEFINITIONS DANS LE SCHEMA
         final LinkedHashSet<Element> liste = new LinkedHashSet<Element>();
         Node enfant = refElement.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("attribute"))
                     liste.add((Element)enfant);
-                else if (nom.equals("optional") || nom.equals("choice"))
+                else if (nom.equals("optional") || nom.equals("choice") || nom.equals("interleave"))
+                    // pb avec group (les attributs risquent d'tre lists plusieurs fois)
                     liste.addAll(listeAttributs((Element)enfant));
                 else if (nom.equals("ref")) {
                     final String nomRef = ((Element)enfant).getAttribute("name");
@@ -619,15 +666,17 @@ public class SchemaRelaxNG implements InterfaceSchema {
      * Renvoie le nom d'un attribut  partir de sa rfrence
      */
     public String nomAttribut(final Element refAttribut) {
-        return(refAttribut.getAttribute("name"));
+        return(localValue(refAttribut.getAttribute("name")));
     }
     
     /**
-     * Renvoie l'espace de noms d'un attribut  partir de sa rfrence
+     * Renvoie l'espace de noms d'un attribut  partir de sa rfrence, ou null si aucun n'est dfini
      */
     public String espaceAttribut(final Element refAttribut) {
         if (!"".equals(refAttribut.getAttribute("ns")))
             return(refAttribut.getAttribute("ns"));
+        else if (prefixeString(refAttribut.getAttribute("name")) != null)
+            return(refAttribut.lookupNamespaceURI(prefixeString(refAttribut.getAttribute("name"))));
         return(null);
     }
     
@@ -654,13 +703,73 @@ public class SchemaRelaxNG implements InterfaceSchema {
     }
     
     /**
-     * Renvoie true si un attribut est obligatoire,  partir de sa dfinition
+     * Renvoie true si un attribut est obligatoire,  partir de sa dfinition.
+     * Attention: ce n'est pas possible  dterminer avec seulement la rfrence d'attribut avec Relax NG.
+     * Il vaut mieux utiliser attributObligatoire.
      */
+    @Deprecated
     public boolean estObligatoire(final Element refAttribut) {
+        // PROBLEME: LA REFERENCE DE L'ELEMENT EST NECESSAIRE EN PLUS POUR SAVOIR SI L'ATTRIBUT EST OBLIGATOIRE
+        // (par exemple si une rfrence optionnelle vers une liste d'attributs est utilise dans la dfinition d'un lment)
         final Element parent = (Element)refAttribut.getParentNode();
         return(!"optional".equals(parent.getLocalName()));
     }
     
+    /**
+     * Renvoit true si l'attribut est obligatoire sous le parent.
+     */
+    public boolean attributObligatoire(final Element refParent, final Element refAttribut) {
+        final Boolean b = trucObligatoire(refParent, refAttribut, null);
+        if (b == null) {
+            LOG.error("attribut non trouv sous un parent: " + nomAttribut(refAttribut) + " sous " + nomElement(refParent));
+            return(false);
+        }
+        return(b.booleanValue());
+    }
+    
+    /**
+     * Cherche si un lment ou un attribut est obligatoire de faon rcursive avec les rfrences/dfinitions.
+     * Le parent est un lment du schma.
+     * truc est un lment element ou attribute.
+     * Renvoie null si le truc n'est pas trouv, Boolean.FALSE s'il n'est pas obligatoire et Boolean.TRUE s'il est obligatoire.
+     */
+    private Boolean trucObligatoire(final Element parent, final Element truc, ArrayList<Element> dejavu) {
+        if (dejavu == null)
+            dejavu = new ArrayList<Element>();
+        if (dejavu.contains(parent))
+            return(null);
+        dejavu.add(parent);
+        Node enfant = parent.getFirstChild();
+        while (enfant != null) {
+            if (enfant == truc)
+                return(Boolean.TRUE);
+            if (enfant.getNodeType() == Node.ELEMENT_NODE) {
+                final String nom = enfant.getLocalName();
+                if ("optional".equals(nom) || "zeroOrMore".equals(nom) || "choice".equals(nom)) { //  amliorer pour choice
+                    final Boolean b = trucObligatoire(((Element)enfant), truc, dejavu);
+                    if (b != null)
+                        return(Boolean.FALSE);
+                } else if ("ref".equals(nom)) {
+                    final String nomRef = ((Element)enfant).getAttribute("name");
+                    for (final Element def : ltousdefine) {
+                        if (def.getAttribute("name").equals(nomRef)) {
+                            final Boolean b = trucObligatoire(def, truc, dejavu);
+                            if (b != null)
+                                return(b);
+                            break;
+                        }
+                    }
+                } else {
+                    final Boolean b = trucObligatoire((Element)enfant, truc, dejavu);
+                    if (b != null)
+                        return(b);
+                }
+            }
+            enfant = enfant.getNextSibling();
+        }
+        return(null);
+    }
+    
     /**
      * Renvoie la liste des valeurs possibles pour un attribut,  partir de sa rfrence.
      * Renvoie null s'il y a un nombre infini de valeurs possibles.
@@ -670,13 +779,23 @@ public class SchemaRelaxNG implements InterfaceSchema {
         Node enfant = refAttribut.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("choice")) {
                     ArrayList<String> liste2 = listeValeursAttribut((Element)enfant);
                     if (liste2 != null)
                         liste.addAll(liste2);
                 } else if (nom.equals("value"))
                     liste.add(enfant.getFirstChild().getNodeValue());
+                else if (nom.equals("ref")) {
+                    final String nomRef = ((Element)enfant).getAttribute("name");
+                    for (final Element def : ltousdefine) {
+                        if (def.getAttribute("name").equals(nomRef)) {
+                            ArrayList<String> liste2 = listeValeursAttribut(def);
+                            if (liste2 != null)
+                                liste.addAll(liste2);
+                        }
+                    }
+                }
             }
             enfant = enfant.getNextSibling();
         }
@@ -698,7 +817,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
     public boolean attributValide(final Element refAttribut, final String valeur) {
         if ((valeur == null || "".equals(valeur)) && estObligatoire(refAttribut))
             return(false);
-        return(true);
+        return(verifType(refAttribut, valeur));
     }
     
     /**
@@ -715,7 +834,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
         Node enfant = refElement.getFirstChild();
         while (enfant != null) {
             if (enfant.getNodeType() == Node.ELEMENT_NODE) {
-                final String nom = enfant.getNodeName();
+                final String nom = enfant.getLocalName();
                 if (nom.equals("text") || nom.equals("data") || nom.equals("mixed"))
                     return(true);
                 else if (nom.equals("ref")) {
@@ -747,7 +866,7 @@ public class SchemaRelaxNG implements InterfaceSchema {
                 for (final Element ref : elementsNommes) {
                     if (nom.equals(nomElement(ref))) {
                         final String espaceRef = espaceElement(ref);
-                        if ((espace == null && espaceRef.equals("")) || (espace != null && espace.equals(espaceRef)))
+                        if ((espace == null && espaceRef == null) || (espace != null && espace.equals(espaceRef)))
                             return(ref);
                     }
                 }
