@@ -31,6 +31,7 @@ import org.w3c.dom.*;
 /**
  * Classe permettant de vrifier la validit d'un type simple (= simpleType dans les schmas XML)
  */
+@Deprecated
 public class VerifTypeSimple {
     /**
      * Logger for this class
@@ -46,7 +47,9 @@ public class VerifTypeSimple {
     
     /**
      * Constructeur prenant en paramtre la config et un lment "element" ou "attribute" d'un schma W3C
+     * @deprecated remplac par jaxe.Config.valeurElementValide() et jaxe.Config.attributValide()
      */
+    @Deprecated
     public VerifTypeSimple(final Config cfg, final Element snodedef) {
 //        compiler = new AwkCompiler();
 //        matcher = new AwkMatcher();
