@@ -20,6 +20,7 @@ import java.awt.event.ItemEvent;
 import java.awt.event.ItemListener;
 import java.awt.BorderLayout;
 import java.awt.Color;
+import java.awt.Dimension;
 import java.awt.FlowLayout;
 import java.awt.GridBagLayout;
 import java.awt.GridBagConstraints;
@@ -27,6 +28,7 @@ import java.awt.Insets;
 import java.awt.Point;
 
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Set;
@@ -36,6 +38,10 @@ import javax.swing.border.EtchedBorder;
 import javax.swing.event.DocumentEvent;
 import javax.swing.event.DocumentListener;
 import javax.swing.text.BadLocationException;
+import javax.swing.undo.AbstractUndoableEdit;
+import javax.swing.undo.CannotUndoException;
+import javax.swing.undo.CompoundEdit;
+import javax.swing.undo.UndoableEdit;
 
 import jaxe.Balise;
 import jaxe.Config;
@@ -44,7 +50,6 @@ import jaxe.DialogueAttributs;
 import jaxe.ImageKeeper;
 import jaxe.JaxeDocument;
 import jaxe.JaxeElement;
-import jaxe.VerifTypeSimple;
 
 import org.w3c.dom.Attr;
 import org.w3c.dom.Element;
@@ -77,7 +82,6 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
     final private Config cfg;
     private ArrayList<Node> enfants;
     private JComponent comp = null;
-    private final VerifTypeSimple verif;
     private final ArrayList<Element> refEnfantsPossibles;
     private final ArrayList<Element> refAttributsPossibles;
     private JLabel labelTitre = null;
@@ -101,7 +105,10 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
         this.noeud = noeud;
         this.affParent = affParent;
         this.doc = doc;
-        this.cfg = doc.cfg;
+        if (!attribut)
+            cfg = doc.cfg.getRefConf(refNoeud);
+        else
+            cfg = affParent.cfg;
         this.attribut = attribut;
         if (affParent == null)
             profondeur = 0;
@@ -116,10 +123,6 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
         }
         
         lireEnfants();
-        if (avecEnfants())
-            verif = null;
-        else
-            verif = cfg.getVerifTypeSimple(refNoeud); // sera null si ce n'est pas un schma W3C
     }
     
     /**
@@ -195,7 +198,17 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
         majPanel(null);
         if (affParent == null) {
             panelEnfants.setFocusCycleRoot(true);
-            panelElement = new JPanel(new BorderLayout());
+            panelElement = new JPanel(new BorderLayout()) {
+                @Override
+                public Dimension getMaximumSize() {
+                    if (System.getProperty("os.name").startsWith("Mac OS")) { // curseur pas visible  droite sur la JVM d'Apple avec getMaximumSize()
+                        final Dimension tps = doc.textPane.getSize();
+                        final JaxeElement je = doc.getElementForNode(noeud);
+                        return(new Dimension(tps.width - 20*(je.indentations()+1) - 2, tps.height));
+                    }
+                    return(super.getMaximumSize());
+                }
+            };
             panelElement.setOpaque(false);
             panelElement.setBorder(BorderFactory.createEmptyBorder(0, 5, 0, 5));
             panelEnfants.setBorder(BorderFactory.createCompoundBorder(BorderFactory.createTitledBorder(getTitre()),
@@ -401,13 +414,13 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
             }
         }
         if (attribut || cfg.contientDuTexte(refNoeud)) {
-            String baseType = null;
-            ArrayList<String> enumeration = null;
-            if (verif != null) {
-                baseType = verif.getBaseType();
-                enumeration = verif.getEnumeration();
-            }
-            if ("boolean".equals(baseType)) {
+            ArrayList<String> enumeration;
+            if (attribut)
+                enumeration = cfg.listeValeursAttribut(refNoeud);
+            else
+                enumeration = cfg.listeValeursElement(refNoeud);
+            final String[] tvalbool = {"true", "false", "1", "0"};
+            if (enumeration != null && Arrays.equals(enumeration.toArray(), tvalbool)) {
                 final String titre = getTitre();
                 final JCheckBox cb = new JCheckBox(titre);
                 if ("true".equals(valeur.trim()) || "1".equals(valeur.trim()))
@@ -479,8 +492,12 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
                 }
             }
             panelDroite.add(comp, BorderLayout.CENTER);
-            if (verif != null && !"".equals(valeur))
-                setValidite(verif.estValide(valeur));
+            if (!"".equals(valeur)) {
+                if (attribut)
+                    setValidite(cfg.attributValide(refNoeud, valeur));
+                else
+                    setValidite(cfg.valeurElementValide(refNoeud, valeur));
+            }
         }
         if (!attribut && affParent != null && affParent.enfantsMultiples(refNoeud)) {
             final JPanel panelBoutons = new JPanel(new FlowLayout(FlowLayout.CENTER, 0, 0));
@@ -526,30 +543,57 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
         if (valeur == null)
             valeur = "";
         doc.setModif(true);
-        if (noeud == null && !"".equals(valeur))
-            creerNoeud();
-        else if (noeud != null && "".equals(valeur) && affParent != null) {
-            effacerNoeud();
+        MyCompoundEdit cedit = null;
+        if (noeud == null && !"".equals(valeur)) {
+            cedit = new MyCompoundEdit();
+            creerNoeud(cedit);
+        } else if (noeud != null && "".equals(valeur) && affParent != null) {
+            effacerNoeud(false, null);
             return;
         }
         if (!attribut) {
             final Element el = (Element)noeud;
             final Node firstChild = el.getFirstChild();
             if (firstChild == null) {
+                final FormUndoableEdit fedit = new FormUndoableEdit(this, "", valeur);
+                if (cedit != null)
+                    cedit.addEdit(fedit);
+                else
+                    doc.textPane.addEdit(fedit);
                 final Node textnode = el.getOwnerDocument().createTextNode(valeur);
                 el.appendChild(textnode);
-            } else if (firstChild.getNodeType() == Node.TEXT_NODE)
+            } else if (firstChild.getNodeType() == Node.TEXT_NODE) {
+                final FormUndoableEdit fedit = new FormUndoableEdit(this, firstChild.getNodeValue(), valeur);
+                if (cedit != null)
+                    cedit.addEdit(fedit);
+                else
+                    doc.textPane.addEdit(fedit);
                 firstChild.setNodeValue(valeur);
-            else
+            } else
                 LOG.error("AffichageFormulaire.enregistrerChangement : pas de noeud texte pour enregistrer le champ");
         } else {
+            final FormUndoableEdit fedit = new FormUndoableEdit(this, noeud.getNodeValue(), valeur);
+            if (cedit != null)
+                cedit.addEdit(fedit);
+            else
+                doc.textPane.addEdit(fedit);
             final Element elparent = (Element)affParent.getNoeud();
             final String nom = cfg.nomAttribut(refNoeud);
             final String espace = cfg.espaceAttribut(refNoeud);
             elparent.setAttributeNS(espace, nom, valeur);
         }
-        if (verif != null)
-            setValidite("".equals(valeur) || verif.estValide(valeur));
+        if (cedit != null) {
+            cedit.end();
+            doc.textPane.addEdit(cedit);
+        }
+        if ("".equals(valeur))
+            setValidite(true);
+        else {
+            if (attribut)
+                setValidite(cfg.attributValide(refNoeud, valeur));
+            else
+                setValidite(cfg.valeurElementValide(refNoeud, valeur));
+        }
     }
     
     /**
@@ -635,22 +679,23 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
     /**
      * Cre l'lment DOM correspondant  l'affichage, si ncessaire en crant l'lment DOM parent.
      */
-    private void creerNoeud() {
+    private void creerNoeud(final MyCompoundEdit cedit) {
         if (attribut) {
             Element elparent = (Element)affParent.getNoeud();
             if (elparent == null) {
-                affParent.creerNoeud();
+                affParent.creerNoeud(cedit);
                 elparent = (Element)affParent.getNoeud();
             }
             final String nom = cfg.nomAttribut(refNoeud);
             final String espace = cfg.espaceAttribut(refNoeud);
             elparent.setAttributeNS(espace, nom, "");
             noeud = elparent.getAttributeNodeNS(espace, nom);
+            cedit.addEdit(new FormUndoableEdit(FormUndoableEdit.TypeEdition.AJOUTER, this));
         } else {
             noeud = JaxeElement.nouvelElementDOM(doc, refNoeud);
             Element elparent = (Element)affParent.getNoeud();
             if (elparent == null) {
-                affParent.creerNoeud();
+                affParent.creerNoeud(cedit);
                 elparent = (Element)affParent.getNoeud();
                 elparent.appendChild( noeud.getOwnerDocument().createTextNode("\n") );
             }
@@ -663,6 +708,7 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
                 elparent.insertBefore(noeud, suivant);
                 elparent.insertBefore(textnode, suivant);
             }
+            cedit.addEdit(new FormUndoableEdit(FormUndoableEdit.TypeEdition.AJOUTER, this));
         }
         affParent.lireEnfants();
         doc.textPane.miseAJourArbre();
@@ -692,7 +738,17 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
     /**
      * Efface le noeud DOM, et son parent s'il devient vide.
      */
-    private void effacerNoeud() {
+    private void effacerNoeud(final boolean annulation, final CompoundEdit cedit1) {
+        CompoundEdit cedit = null;
+        if (!annulation) {
+            final FormUndoableEdit fedit = new FormUndoableEdit(FormUndoableEdit.TypeEdition.SUPPRIMER, this);
+            if (cedit1 != null)
+                cedit1.addEdit(fedit);
+            else {
+                cedit = new CompoundEdit();
+                cedit.addEdit(fedit);
+            }
+        }
         final Element elparent = (Element)affParent.getNoeud();
         if (attribut) {
             final String nom = cfg.nomAttribut(refNoeud);
@@ -705,17 +761,24 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
                 elparent.removeChild(textNode);
         }
         noeud = null;
-        affParent.testEffacementParent();
+        if (!annulation)
+            affParent.testEffacementParent(cedit1 != null ? cedit1 : cedit);
+        else
+            lireEnfants();
         doc.textPane.miseAJourArbre();
+        if (cedit1 == null && cedit != null) {
+            cedit.end();
+            doc.textPane.addEdit(cedit);
+        }
     }
     
     /**
      * Efface l'lment s'il est vide.
      */
-    private void testEffacementParent() {
+    private void testEffacementParent(final CompoundEdit cedit) {
         lireEnfants();
         if (enfants.size() == 0 && noeud != null && affParent != null)
-            effacerNoeud();
+            effacerNoeud(false, cedit);
     }
     
     /**
@@ -734,7 +797,7 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
      */
     private boolean obligatoire() {
         if (attribut)
-            return(cfg.estObligatoire(refNoeud));
+            return(affParent.attributObligatoire(refNoeud));
         return(affParent.elementObligatoire(refNoeud));
     }
     
@@ -747,6 +810,13 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
         return(cfg.getSchema().elementObligatoire(refNoeud, ref));
     }
     
+    /**
+     * Renvoit true si l'attribut de rfrence ref est obligatoire.
+     */
+    private boolean attributObligatoire(final Element ref) {
+        return(cfg.attributObligatoire(refNoeud, ref));
+    }
+    
     /**
      * Mise  jour de l'affichage cause par le changement dans un sous-affichage.
      */
@@ -759,7 +829,7 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
      */
     private void retirerAffichageEnfant(final AffichageFormulaire aff) {
         if (aff.getNoeud() != null)
-            aff.effacerNoeud();
+            aff.effacerNoeud(false, null);
         majPanel(null);
     }
     
@@ -795,6 +865,36 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
             return(null);
     }
     
+    private AffichageFormulaire chercherAffichage(final Node n) {
+        if (n == noeud)
+            return(this);
+        if (affEnfants == null)
+            return(null);
+        for (AffichageFormulaire aff : affEnfants) {
+            final AffichageFormulaire a = aff.chercherAffichage(n);
+            if (a != null)
+                return(a);
+        }
+        return(null);
+    }
+    
+    private AffichageFormulaire chercherAffichageParRef(final Element ref) {
+        if (ref == refNoeud)
+            return(this);
+        if (affEnfants == null)
+            return(null);
+        for (AffichageFormulaire aff : affEnfants) {
+            final AffichageFormulaire a = aff.chercherAffichageParRef(ref);
+            if (a != null)
+                return(a);
+        }
+        return(null);
+    }
+    
+    private JaxeDocument getJaxeDocument() {
+        return(doc);
+    }
+    
     public void selection(final boolean select) {
         if (select)
             panelEnfants.setBackground(Balise.getCouleurs()[0][1]);
@@ -802,10 +902,26 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
             panelEnfants.setBackground(null);
     }
     
+    public void setFocus() {
+        if (affEnfants == null || affEnfants.size() == 0)
+            return;
+        AffichageFormulaire aff = affEnfants.get(0);
+        while (aff != null) {
+            if (aff.comp != null) {
+                aff.comp.requestFocusInWindow();
+                return;
+            }
+            if (aff.affEnfants == null || aff.affEnfants.size() == 0)
+                return;
+            aff = aff.affEnfants.get(0);
+        }
+    }
+    
     public void majAffichage() {
         //  faire
     }
     
+    
     class ActionAide extends AbstractAction {
         Element refNoeud;
         ActionAide(final Element refNoeud) {
@@ -825,4 +941,168 @@ public class AffichageFormulaire implements DocumentListener, ItemListener {
             dlg.setVisible(true);
         }
     }
+    
+    static class MyCompoundEdit extends CompoundEdit {
+        public MyCompoundEdit() {
+            super();
+        }
+        public boolean addEdit(UndoableEdit anEdit) {
+            final boolean absorbed = super.addEdit(anEdit);
+            if (!absorbed && !isInProgress() && anEdit instanceof FormUndoableEdit) {
+                final UndoableEdit ledit = lastEdit();
+                if (ledit != null && ledit instanceof FormUndoableEdit)
+                    return(ledit.addEdit(anEdit));
+            }
+            return(absorbed);
+        }
+    }
+    
+    static class FormUndoableEdit extends AbstractUndoableEdit {
+        public enum TypeEdition { AJOUTER, SUPPRIMER, MODIFIER };
+        private TypeEdition ajsup;
+        private String ancienTexte, nouveauTexte;
+        private Node noeud, parent;
+        private AffichageFormulaire premierAffichage;
+        private Node suivant;
+        
+        public FormUndoableEdit(final TypeEdition ajsup, final AffichageFormulaire aff) {
+            this.ajsup = ajsup;
+            noeud = aff.getNoeud();
+            if (noeud instanceof Attr)
+                parent = ((Attr)noeud).getOwnerElement();
+            else
+                parent = noeud.getParentNode();
+            premierAffichage = aff;
+            while (premierAffichage.affParent != null)
+                premierAffichage = premierAffichage.affParent;
+            suivant = noeud.getNextSibling();
+            if (suivant != null)
+                suivant = suivant.getNextSibling();
+        }
+        public FormUndoableEdit(final AffichageFormulaire aff, final String ancienTexte, final String nouveauTexte) {
+            ajsup = TypeEdition.MODIFIER;
+            noeud = aff.getNoeud();
+            if (noeud instanceof Attr)
+                parent = ((Attr)noeud).getOwnerElement();
+            else
+                parent = noeud.getParentNode();
+            premierAffichage = aff;
+            while (premierAffichage.affParent != null)
+                premierAffichage = premierAffichage.affParent;
+            this.ancienTexte = ancienTexte;
+            this.nouveauTexte = nouveauTexte;
+            suivant = null;
+            if (noeud instanceof Element) {
+                suivant = noeud.getNextSibling();
+                if (suivant != null)
+                    suivant = suivant.getNextSibling();
+            }
+        }
+        
+        private void recreerElement() {
+            final Node textnode = noeud.getOwnerDocument().createTextNode("\n");
+            final Element elparent = (Element)parent;
+            if (suivant == null) {
+                elparent.appendChild(noeud);
+                elparent.appendChild(textnode);
+            } else {
+                elparent.insertBefore(noeud, suivant);
+                elparent.insertBefore(textnode, suivant);
+            }
+        }
+        private void ajouter() {
+            final AffichageFormulaire affP = premierAffichage.chercherAffichage(parent);
+            if (noeud instanceof Element)
+                recreerElement();
+            else {
+                final Element elparent = (Element)parent;
+                elparent.setAttributeNodeNS((Attr)noeud);
+            }
+            affP.lireEnfants();
+            affP.getJaxeDocument().textPane.miseAJourArbre();
+            affP.majPanel(null);
+            final AffichageFormulaire affAj = affP.chercherAffichage(noeud);
+            if (affAj != null && affAj.comp != null)
+                affAj.comp.requestFocusInWindow();
+        }
+        private void supprimer() {
+            final AffichageFormulaire aff = premierAffichage.chercherAffichage(noeud);
+            final Element ref = aff.refNoeud;
+            final AffichageFormulaire affP = aff.affParent;
+            suivant = noeud.getNextSibling();
+            if (suivant != null)
+                suivant = suivant.getNextSibling();
+            aff.effacerNoeud(true, null);
+            affP.majPanel(null);
+            SwingUtilities.invokeLater(new Runnable() {
+                // pour viter un pb si le parent est effac juste aprs
+                public void run() {
+                    final AffichageFormulaire affref = premierAffichage.chercherAffichageParRef(ref);
+                    if (affref != null && affref.comp != null)
+                        affref.comp.requestFocusInWindow();
+                }
+            });
+        }
+        private void changerTexte(final String s) {
+            final AffichageFormulaire affP = premierAffichage.chercherAffichage(parent);
+            if (noeud instanceof Element) {
+                final AffichageFormulaire aff = affP.chercherAffichage(noeud);
+                Node noeudTexte = noeud.getFirstChild();
+                if (aff == null && !"".equals(s))
+                    recreerElement();
+                else if (noeudTexte != null)
+                    noeudTexte.setNodeValue(s);
+                else {
+                    noeudTexte = noeud.getOwnerDocument().createTextNode(s);
+                    noeud.appendChild(noeudTexte);
+                }
+                affP.majPanel(null);
+            } else { // attribut
+                final Element elparent = (Element)parent;
+                if (!elparent.hasAttributeNS(noeud.getNamespaceURI(), noeud.getLocalName()))
+                    elparent.setAttributeNodeNS((Attr)noeud);
+                noeud.setNodeValue(s);
+                affP.majPanel(null);
+            }
+            final AffichageFormulaire affAj = affP.chercherAffichage(noeud);
+            if (affAj != null && affAj.comp != null)
+                affAj.comp.requestFocusInWindow();
+        }
+        private void refaireModifTexte() {
+            changerTexte(nouveauTexte);
+        }
+        private void defaireModifTexte() {
+            changerTexte(ancienTexte);
+        }
+        public boolean addEdit(UndoableEdit anEdit) {
+            if (anEdit instanceof FormUndoableEdit) {
+                final FormUndoableEdit fedit = (FormUndoableEdit)anEdit;
+                if (ajsup == TypeEdition.MODIFIER && noeud == fedit.noeud && fedit.ajsup == TypeEdition.MODIFIER) {
+                    nouveauTexte = fedit.nouveauTexte;
+                    fedit.die();
+                    return(true);
+                }
+            }
+            return(false);
+        }
+        public void undo() throws CannotUndoException {
+            super.undo();
+            if (ajsup == TypeEdition.AJOUTER)
+                supprimer();
+            else if (ajsup == TypeEdition.SUPPRIMER)
+                ajouter();
+            else
+                defaireModifTexte();
+        }
+        public void redo() throws CannotUndoException {
+            super.redo();
+            if (ajsup == TypeEdition.AJOUTER)
+                ajouter();
+            else if (ajsup == TypeEdition.SUPPRIMER)
+                supprimer();
+            else
+                refaireModifTexte();
+        }
+    }
+    
 }
