@@ -29,6 +29,7 @@ import javax.swing.*;
 import javax.swing.text.DefaultEditorKit;
 import javax.swing.text.JTextComponent;
 
+import jaxe.Config;
 import jaxe.JaxeDocument;
 import jaxe.JaxeResourceBundle;
 
@@ -53,6 +54,7 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
     JaxeDocument doc;
     String srcAttr;
     ArrayList<Element> latt;
+    Config conf;
     
     public DialogueFichier(final JFrame jframe, final JaxeDocument doc, final String titre, final Element refElement, final Element el, final String srcAttr) {
         super(jframe, titre, true);
@@ -61,7 +63,8 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
         this.refElement = refElement;
         this.el = el;
         this.srcAttr = srcAttr;
-        latt = doc.cfg.listeAttributs(refElement);
+        conf = doc.cfg.getRefConf(refElement);
+        latt = conf.listeAttributs(refElement);
         final int natt = latt.size();
         noms = new String[natt];
         final String[] titres = new String[natt];
@@ -70,14 +73,19 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
         defauts = new String[natt];
         for (int i=0; i<natt; i++) {
             final Element att = latt.get(i);
-            noms[i] = doc.cfg.nomAttribut(att);
-            titres[i] = doc.cfg.titreAttribut(refElement, att);
-            espaces[i] = doc.cfg.espaceAttribut(att);
+            noms[i] = conf.nomAttribut(att);
+            titres[i] = conf.titreAttribut(refElement, att);
+            espaces[i] = conf.espaceAttribut(att);
+            if (espaces[i] != null) {
+                final String prefixe = conf.prefixeAttribut(el, att);
+                if (prefixe != null)
+                    noms[i] = prefixe + ":" + noms[i];
+            }
             String elval = el.getAttribute(noms[i]);
-            defauts[i] = doc.cfg.valeurParDefaut(att);
+            defauts[i] = conf.valeurParDefaut(att);
             if ("".equals(elval) && defauts[i] != null && el.getAttributeNode(noms[i]) == null)
                 elval = defauts[i];
-            final ArrayList<String> lval = doc.cfg.listeValeursAttribut(att);
+            final ArrayList<String> lval = conf.listeValeursAttribut(att);
             if (lval != null && lval.size() > 0) {
                 final JComboBox popup = new JComboBox();
                 champs[i] = popup;
@@ -85,7 +93,7 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
                     popup.addItem("");
                 for (int j=0; j<lval.size(); j++) {
                     final String sval = lval.get(j);
-                    popup.addItem(doc.cfg.titreValeurAttribut(refElement, att, sval));
+                    popup.addItem(conf.titreValeurAttribut(refElement, att, sval));
                     if (sval.equals(elval)) {
                         if (defauts[i] == null)
                             popup.setSelectedIndex(j+1);
@@ -94,7 +102,7 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
                     }
                 }
             } else {
-                final ArrayList<String> lvs = doc.cfg.listeValeursSuggereesAttribut(refElement, att);
+                final ArrayList<String> lvs = conf.listeValeursSuggereesAttribut(refElement, att);
                 if (lvs != null && lvs.size() > 0) {
                     final JComboBox popup = new JComboBox();
                     popup.setEditable(true);
@@ -104,7 +112,7 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
                     int indexsel = -1;
                     for (int j=0; j<lvs.size(); j++) {
                         final String sval = lvs.get(j);
-                        popup.addItem(doc.cfg.titreValeurAttribut(refElement, att, sval));
+                        popup.addItem(conf.titreValeurAttribut(refElement, att, sval));
                         if (sval.equals(elval)) {
                             if (defauts[i] == null)
                                 indexsel = j+1;
@@ -128,7 +136,7 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
         for (int i=0; i<titres.length; i++) {
             final JLabel label = new JLabel(titres[i]);
             final Element att = latt.get(i);
-            if (doc.cfg.estObligatoire(att))
+            if (conf.attributObligatoire(refElement, att))
                 label.setForeground(new Color(150, 0, 0)); // rouge fonc
             else
                 label.setForeground(new Color(0, 100, 0)); // vert fonc
@@ -267,9 +275,9 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
                 final String valeur;
                 if (indexval >= 0) {
                     final Element att = latt.get(i);
-                    ArrayList<String> lval = doc.cfg.listeValeursAttribut(att);
+                    ArrayList<String> lval = conf.listeValeursAttribut(att);
                     if (lval == null || lval.size() == 0)
-                        lval = doc.cfg.listeValeursSuggereesAttribut(refElement, att);
+                        lval = conf.listeValeursSuggereesAttribut(refElement, att);
                     valeur = lval.get(indexval);
                 } else
                     valeur = (String)combo.getSelectedItem();
@@ -304,7 +312,7 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
         final String[] rep = lireReponses();
         for (int i=0; i<latt.size(); i++) {
             final Element att = latt.get(i);
-            if (doc.cfg.estObligatoire(att) && (rep[i] == null || "".equals(rep[i]))) {
+            if (conf.attributObligatoire(refElement, att) && (rep[i] == null || "".equals(rep[i]))) {
                 getToolkit().beep();
                 if (champs[i] instanceof JTextComponent)
                     ((JTextComponent)champs[i]).selectAll();
@@ -351,45 +359,51 @@ public class DialogueFichier extends JDialog implements ActionListener, KeyListe
     }
     
     public void selectionnerFichier() {
-        /*
-        JFileChooser chooser;
-        if (doc.fsave != null)
-            chooser = new JFileChooser(doc.fsave.getParentFile());
-        else
-            chooser = new JFileChooser();
-        int returnVal = chooser.showOpenDialog(this);
-        if(returnVal == JFileChooser.APPROVE_OPTION) {
-            File theFile = chooser.getSelectedFile();
-        */
-        final FileDialog fd = new FileDialog(jframe);
-        if (doc.fsave != null)
-            fd.setDirectory(doc.fsave.getParent());
-        fd.setVisible(true);
-        final String sf = fd.getFile();
-        if (sf != null) {
-            final File theFile = new File(fd.getDirectory(), sf);
-            setNomFichier(cheminURI(theFile));
+        if (System.getProperty("os.name").indexOf("Linux") != -1) {
+            JFileChooser chooser;
+            if (doc.fsave != null)
+                chooser = new JFileChooser(doc.fsave.getParentFile());
+            else
+                chooser = new JFileChooser();
+            final int resultat = chooser.showOpenDialog(jframe);
+            if (resultat == JFileChooser.APPROVE_OPTION) {
+                final File theFile = chooser.getSelectedFile();
+                setNomFichier(cheminURI(theFile));
+            }
+        } else {
+            final FileDialog fd = new FileDialog(jframe);
+            if (doc.fsave != null)
+                fd.setDirectory(doc.fsave.getParent());
+            fd.setVisible(true);
+            final String sf = fd.getFile();
+            if (sf != null) {
+                final File theFile = new File(fd.getDirectory(), sf);
+                setNomFichier(cheminURI(theFile));
+            }
         }
     }
     
     public void copierFichier() {
-        /*
-        JFileChooser chooser;
-        if (doc.fsave != null)
-            chooser = new JFileChooser(doc.fsave.getParentFile());
-        else
-            chooser = new JFileChooser();
-        int returnVal = chooser.showOpenDialog(this);
-        if(returnVal == JFileChooser.APPROVE_OPTION) {
-            File theFile = chooser.getSelectedFile();
-        */
-        final FileDialog fd = new FileDialog(jframe);
-        fd.setVisible(true);
-        final String sf = fd.getFile();
-        if (sf != null) {
-            final File theFile = new File(fd.getDirectory(), sf);
-            copierFichier(theFile, doc.fsave.getParent() + File.separator + theFile.getName());
-            setNomFichier(theFile.getName());
+        File f = null;
+        if (System.getProperty("os.name").indexOf("Linux") != -1) {
+            JFileChooser chooser;
+            if (doc.fsave != null)
+                chooser = new JFileChooser(doc.fsave.getParentFile());
+            else
+                chooser = new JFileChooser();
+            int resultat = chooser.showOpenDialog(this);
+            if (resultat == JFileChooser.APPROVE_OPTION)
+                f = chooser.getSelectedFile();
+        } else {
+            final FileDialog fd = new FileDialog(jframe);
+            fd.setVisible(true);
+            final String sf = fd.getFile();
+            if (sf != null)
+                f = new File(fd.getDirectory(), sf);
+        }
+        if (f != null) {
+            copierFichier(f, doc.fsave.getParent() + File.separator + f.getName());
+            setNomFichier(f.getName());
         }
     }
     
