@@ -63,7 +63,6 @@ public class JETable extends JaxeElement implements ActionListener {
      */
     private static final Logger LOG = Logger.getLogger(JETable.class);
 
-    static String newline = "\n";
     String TRtag = "tr";
     String TDtag = "td";
     String THtag = null;
@@ -310,7 +309,7 @@ public class JETable extends JaxeElement implements ActionListener {
                 final Element cellel = nouvelElementDOM(doc, TDtag, ligneel);
                 ligneel.appendChild(cellel);
             }
-            final Node textnode = doc.DOMdoc.createTextNode(newline);
+            final Node textnode = doc.DOMdoc.createTextNode("\n");
             newel.appendChild(textnode);
         }
 
@@ -424,7 +423,7 @@ public class JETable extends JaxeElement implements ActionListener {
             el.appendChild(tr);
         } else {
             final Element trsel = findligne(lsel+1);
-            final Node textnode = doc.DOMdoc.createTextNode(newline);
+            final Node textnode = doc.DOMdoc.createTextNode("\n");
             if (trsel == null) {
                 el.appendChild(tr);
                 el.appendChild(textnode);
