@@ -44,6 +44,7 @@ import javax.swing.text.Style;
 import javax.swing.text.StyleConstants;
 
 import jaxe.Balise;
+import jaxe.Config;
 import jaxe.DialogueAttributs;
 import jaxe.DialogueChamps;
 import jaxe.ImageKeeper;
@@ -72,7 +73,6 @@ public class JETableTexte extends JaxeElement implements ActionListener {
      */
     private static final Logger LOG = Logger.getLogger(JETableTexte.class);
 
-    static String newline = "\n";
     String tableTag = null;
     String TRtag = "tr";
     String TDtag = "td";
@@ -87,11 +87,13 @@ public class JETableTexte extends JaxeElement implements ActionListener {
     Element[][] grille; // utile pour grer colspan et rowspan
     boolean inTable = true;
     private JPanel pboutons;
-    
     TableHelper helper;
+    Config conf;
+    
 
     public JETableTexte(final JaxeDocument doc) {
         this.doc = doc;
+        conf = doc.cfg;
     }
     
     protected void obtenirTags(final Element refElement) {
@@ -109,8 +111,10 @@ public class JETableTexte extends JaxeElement implements ActionListener {
     @Override
     public void init(Position pos, final Node noeud) {
         final Element el = (Element) noeud;
-        if (refElement != null)
+        if (refElement != null) {
+            conf = doc.cfg.getRefConf(refElement);
             obtenirTags(refElement);
+        }
         
         helper = new TableHelper(noeud, TRtag, TDtag, THtag, colspanAttr, rowspanAttr);
 
@@ -186,15 +190,15 @@ public class JETableTexte extends JaxeElement implements ActionListener {
             } else
                 avecEntete = false;
 
-            final Element reftd = doc.cfg.referenceElement(TDtag);
+            final Element reftd = conf.referenceElement(TDtag);
             ArrayList<Element> lattributs = null;
             if (reftd != null)
-                lattributs = doc.cfg.listeAttributs(reftd);
+                lattributs = conf.listeAttributs(reftd);
             boolean avecRowspan = false;
             boolean avecColspan = false;
             if (lattributs != null)
                 for (final Element el2 : lattributs) {
-                    final String nomAtt = doc.cfg.nomAttribut(el2);
+                    final String nomAtt = conf.nomAttribut(el2);
                     if (rowspanAttr != null && rowspanAttr.equals(nomAtt))
                         avecRowspan = true;
                     else if (colspanAttr != null && colspanAttr.equals(nomAtt))
@@ -343,7 +347,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
 
                 // Add new rowspan-Value
                 num += addnum;
-                ((Element) jesel.noeud).setAttributeNS(doc.cfg.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(num));
+                ((Element) jesel.noeud).setAttributeNS(conf.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(num));
 
                     // Remove the Cell in the next Row
                 final Element parent = (Element) el.getParentNode();
@@ -374,7 +378,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
 
                                 if (rownums + y > p.y && rownums > 1) {
                                     rownums--;
-                                    ele.setAttributeNS(doc.cfg.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(rownums));
+                                    ele.setAttributeNS(conf.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(rownums));
                                 }
                             }
                         }
@@ -432,7 +436,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
 
                 // Decrease Number
                 num--;
-                ((Element) jesel.noeud).setAttributeNS(doc.cfg.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(num));
+                ((Element) jesel.noeud).setAttributeNS(conf.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(num));
 
                 // find position in tr to insert it
                 Node child = node.getFirstChild();
@@ -515,7 +519,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                 num += addnum;
 
                 // Set new Colspan
-                ((Element) jesel.noeud).setAttributeNS(doc.cfg.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(num));
+                ((Element) jesel.noeud).setAttributeNS(conf.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(num));
 
                 doc.setModif(true);
 
@@ -553,7 +557,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
             doc.textPane.miseAJourAnnulation();
             // Decrease Colspans
             num--;
-            ((Element) jesel.noeud).setAttributeNS(doc.cfg.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(num));
+            ((Element) jesel.noeud).setAttributeNS(conf.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(num));
 
             // Create new Element
             final Element td = nouvelElementDOM(doc, TDtag, (Element) jesel.noeud);
@@ -782,15 +786,14 @@ public class JETableTexte extends JaxeElement implements ActionListener {
         final Element newel = nouvelElementDOM(doc, refElement);
         if (newel == null)
             return null;
-        final ArrayList<Element> latt = doc.cfg.listeAttributs(refElement);
-        if (latt != null && latt.size() > 0) {
+        if (testAffichageDialogue()) {
             final DialogueAttributs dlg = new DialogueAttributs(doc.jframe, doc,
-                JaxeResourceBundle.getRB().getString("zone.NouvelleBalise") + " " + doc.cfg.titreElement(refElement), refElement, newel);
+                JaxeResourceBundle.getRB().getString("zone.NouvelleBalise") + " " + conf.titreElement(refElement), refElement, newel);
             if (!dlg.afficher())
                 return null;
             dlg.enregistrerReponses();
         }
-        Node textnode = doc.DOMdoc.createTextNode(newline);
+        Node textnode = doc.DOMdoc.createTextNode("\n");
         newel.appendChild(textnode);
         for (int i = 0; i < nlignes; i++) {
             final Element ligneel = nouvelElementDOM(doc, TRtag, newel);
@@ -799,7 +802,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                 final Element cellel = nouvelElementDOM(doc, TDtag, ligneel);
                 ligneel.appendChild(cellel);
             }
-            textnode = doc.DOMdoc.createTextNode(newline);
+            textnode = doc.DOMdoc.createTextNode("\n");
             newel.appendChild(textnode);
         }
         this.doc.setModif(true);
@@ -816,7 +819,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
     public void afficherDialogue(final JFrame jframe) {
         final Element el = (Element) noeud;
 
-        final ArrayList<Element> latt = doc.cfg.listeAttributs(refElement);
+        final ArrayList<Element> latt = conf.listeAttributs(refElement);
         if (latt != null && latt.size() > 0) {
             final DialogueAttributs dlg = new DialogueAttributs(doc.jframe, doc, el.getTagName(), refElement, el);
             if (dlg.afficher()) {
@@ -1008,7 +1011,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
             trnext = helper.ligneSuivante(jetrsel.noeud);
         }
         final Element tr = nouvelElementDOM(doc, TRtag, el);
-        final Node textnode = doc.DOMdoc.createTextNode(newline);
+        final Node textnode = doc.DOMdoc.createTextNode("\n");
         for (int ic = 0; ic < nbcolonnes; ic++) {
             if (rsel != -1 && rsel + 1 < nblignes && grille[rsel][ic] == grille[rsel + 1][ic]) {
                 final Element td = grille[rsel][ic];
@@ -1021,7 +1024,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                         } catch (final NumberFormatException e) {
                         }
                     }
-                    td.setAttributeNS(doc.cfg.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(irowspan + 1));
+                    td.setAttributeNS(conf.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(irowspan + 1));
                 }
                 while (ic + 1 < nbcolonnes && grille[rsel][ic + 1] == td)
                     ic++;
@@ -1088,7 +1091,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                             } catch (final NumberFormatException e) {
                             }
                         }
-                        td.setAttributeNS(doc.cfg.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(irowspan - 1));
+                        td.setAttributeNS(conf.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(irowspan - 1));
                     }
                     while (ic + 1 < nbcolonnes && grille[rsel][ic + 1] == td)
                         ic++;
@@ -1122,7 +1125,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                             } catch (final NumberFormatException e) {
                             }
                         }
-                        td.setAttributeNS(doc.cfg.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(irowspan - 1));
+                        td.setAttributeNS(conf.espaceAttribut(rowspanAttr), rowspanAttr, Integer.toString(irowspan - 1));
                     }
                 }
             }
@@ -1167,7 +1170,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                         } catch (final NumberFormatException e) {
                         }
                     }
-                    td.setAttributeNS(doc.cfg.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(icolspan + 1));
+                    td.setAttributeNS(conf.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(icolspan + 1));
                 }
                 while (il + 1 < nblignes && grille[il + 1][csel] == td) {
                     il++;
@@ -1238,7 +1241,7 @@ public class JETableTexte extends JaxeElement implements ActionListener {
                             } catch (final NumberFormatException e) {
                             }
                         }
-                        td.setAttributeNS(doc.cfg.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(icolspan - 1));
+                        td.setAttributeNS(conf.espaceAttribut(colspanAttr), colspanAttr, Integer.toString(icolspan - 1));
                     }
                 } else
                     ((Element) td.getParentNode()).removeChild(td);
