@@ -23,6 +23,7 @@ import jaxe.equations.element.MathIdentifier;
 import jaxe.equations.element.MathNumber;
 import jaxe.equations.element.MathOperator;
 import jaxe.equations.element.MathOver;
+import jaxe.equations.element.MathRoot;
 import jaxe.equations.element.MathRootElement;
 import jaxe.equations.element.MathRow;
 import jaxe.equations.element.MathSqrt;
@@ -51,57 +52,102 @@ public class StringMathBuilder {
     private final MathRootElement rootElement;
     
     // oprateurs remplacs en premier, avant l'analyse de la syntaxe
-    private static final String[][] special = {{"<==", "\u21D0"}, {"==>", "\u21D2"}, {"<=>", "\u21D4"},
-                                        {"!=", "\u2260"}, {"~=", "\u2248"}, {"<=", "\u2264"},
-                                        {">=", "\u2265"}, {"<<", "\u226A"}, {">>", "\u226B"},
-                                        //  "->" kepts for backward-compatibility
-                                        {"-->", "\u2192"}, {"<->", "\u2194"}, {"->", "\u2192"},
-                                        {"<--", "\u2190"},
-                                        {"forall", "\u2200"}, {"quelquesoit", "\u2200"},
-                                        {"exists", "\u2203"}, {"ilexiste", "\u2203"},
-                                        {"part", "\u2202"}, {"drond", "\u2202"},
-                                        {"nabla", "\u2207"}, {"prop", "\u221D"},
-                                        {"cross", ""}, {"croix", ""},
-                                        {"plusmn", ""}, {"plusoumoins", ""}, {"plusminus", ""},
-                                        {"...", "\u2026"}};
+    private static final String[][] special = {
+        {"<==", "\u21D0"}, {"==>", "\u21D2"}, {"<=>", "\u21D4"},
+        {"!=", "\u2260"}, {"~=", "\u2248"}, {"~", "\u223C"},
+        {"<=", "\u2264"}, {">=", "\u2265"}, {"<<", "\u226A"}, {">>", "\u226B"},
+        //  "->" kepts for backward-compatibility
+        {"-->", "\u2192"}, {"<->", "\u2194"}, {"->", "\u2192"}, {"<--", "\u2190"},
+        {"equiv", "\u2261"},
+        {"forall", "\u2200"}, {"quelquesoit", "\u2200"},
+        {"exists", "\u2203"}, {"ilexiste", "\u2203"},
+        {"part", "\u2202"}, {"drond", "\u2202"},
+        {"nabla", "\u2207"},
+        {"prop", "\u221D"},
+        {"times", ""}, {"cross", ""}, {"croix", ""},
+        {"wedge", "\u2227"}, {"pvec", "\u2227"},
+        {"plusmn", ""}, {"plusoumoins", ""}, {"plusminus", ""},
+        {"cap", "\u2229"}, {"cup", "\u222A"},
+        {"...", "\u2026"}
+    };
     
     // oprateurs
     private static final String sops = 
-            "_^*/\u2207\u2213-+\u2200\u2203=\u2260\u2248<>\u2264\u2265\u226A\u226B\u221D" +
-            "\u2190\u2192\u2194\u21D0\u21D2\u21D4";
+        "_^*/\u2207\u2213\u2227-+\u2200\u2203\u2202=\u2260\u2248\u223C\u2261<>\u2264\u2265\u226A\u226B\u221D" +
+        "|\u2229\u222A\u2190\u2192\u2194\u21D0\u21D2\u21D4";
     
-    private static final String[][] symbols = {
-                    // grec-minuscule
-                    {"alpha", "\u03B1"}, {"beta", "\u03B2"}, {"gamma", "\u03B3"},
-                    {"delta", "\u03B4"}, {"epsilon", "\u03B5"}, {"zeta", "\u03B6"},
-                    {"eta", "\u03B7"}, {"theta", "\u03B8"}, {"iota", "\u03B9"},
-                    {"kappa", "\u03BA"}, {"lambda", "\u03BB"}, {"mu", "\u03BC"},
-                    {"nu", "\u03BD"}, {"xi", "\u03BE"}, {"omicron", "\u03BF"},
-                    {"pi", "\u03C0"}, {"rho", "\u03C1"}, {"sigma", "\u03C3"},
-                    {"tau", "\u03C4"}, {"upsilon", "\u03C5"}, {"phi", "\u03C6"},
-                    {"chi", "\u03C7"}, {"psi", "\u03C8"}, {"omega", "\u03C9"},
-                    // grec-majuscule
-                    {"Alpha", "\u0391"}, {"Beta", "\u0392"}, {"Gamma", "\u0393"},
-                    {"Delta", "\u0394"}, {"Epsilon", "\u0395"}, {"Zeta", "\u0396"},
-                    {"Eta", "\u0397"}, {"Theta", "\u0398"}, {"Iota", "\u0399"},
-                    {"Kappa", "\u039A"}, {"Lambda", "\u039B"}, {"Mu", "\u039C"},
-                    {"Nu", "\u039D"}, {"Xi", "\u039E"}, {"Omicron", "\u039F"},
-                    {"Pi", "\u03A0"}, {"Rho", "\u03A1"}, {"Sigma", "\u03A3"},
-                    {"Tau", "\u03A4"}, {"Upsilon", "\u03A5"}, {"Phi", "\u03A6"},
-                    {"Chi", "\u03A7"}, {"Psi", "\u03A8"}, {"Omega", "\u03A9"},
-                    // autre grec
-                    {"thetasym", "\u03D1"}, {"upsih", "\u03D2"}, {"piv", "\u03D6"},
-                    // autres caractres
-                    {"infin", "\u221E"}, {"infty", "\u221E"}, {"infini", "\u221E"},
-                    {"sun", "\u2609"}, {"soleil", "\u2609"},
-                    {"star", "\u2605"}, {"toile", "\u2605"},
-                    {"earth", "\u2641"}, {"terre", "\u2641"},
-                    {"planck", "\u210F"},
-                    {"angstrom", "\u212B"}, {"angstrm", "\u212B"},
-                    {"petitl", "\u2113"},
-                    {"asterisk", "*"}, {"astrisque", "*"}, // \uFF0A ?
-                    /*{"grandg", "\u1D4A2"},*/
-                };
+    // symboles qui peuvent tre en italique s'ils servent d'identifiant
+    private static final String[][] symboles_id = {
+        // grec-minuscule
+        {"alpha", "\u03B1"}, {"beta", "\u03B2"}, {"gamma", "\u03B3"},
+        {"delta", "\u03B4"}, {"epsilon", "\u03B5"}, {"zeta", "\u03B6"},
+        {"eta", "\u03B7"}, {"theta", "\u03B8"}, {"iota", "\u03B9"},
+        {"kappa", "\u03BA"}, {"lambda", "\u03BB"}, {"mu", "\u03BC"},
+        {"nu", "\u03BD"}, {"xi", "\u03BE"}, {"omicron", "\u03BF"},
+        {"rho", "\u03C1"}, {"sigma", "\u03C3"},
+        {"tau", "\u03C4"}, {"upsilon", "\u03C5"}, {"phi", "\u03C6"},
+        {"chi", "\u03C7"}, {"psi", "\u03C8"}, {"omega", "\u03C9"},
+        // grec-majuscule
+        {"Alpha", "\u0391"}, {"Beta", "\u0392"}, {"Gamma", "\u0393"},
+        {"Delta", "\u0394"}, {"Epsilon", "\u0395"}, {"Zeta", "\u0396"},
+        {"Eta", "\u0397"}, {"Theta", "\u0398"}, {"Iota", "\u0399"},
+        {"Kappa", "\u039A"}, {"Lambda", "\u039B"}, {"Mu", "\u039C"},
+        {"Nu", "\u039D"}, {"Xi", "\u039E"}, {"Omicron", "\u039F"},
+        {"Pi", "\u03A0"}, {"Rho", "\u03A1"}, {"Sigma", "\u03A3"},
+        {"Tau", "\u03A4"}, {"Upsilon", "\u03A5"}, {"Phi", "\u03A6"},
+        {"Chi", "\u03A7"}, {"Psi", "\u03A8"}, {"Omega", "\u03A9"},
+        // autre grec
+        {"thetasym", "\u03D1"}, {"upsih", "\u03D2"}, {"piv", "\u03D6"},
+        {"phiv", "\u03D5"}, {"phi1", "\u03D5"}
+    };
+    // symboles qu'il ne faut pas mettre en italique
+    private static final String[][] symboles_droits = {
+        // grec-minuscule
+        {"pi", "\u03C0"}, 
+        // autres caractres
+        {"infin", "\u221E"}, {"infty", "\u221E"}, {"infini", "\u221E"},
+        {"parallel", "\u2225"}, {"parallle", "\u2225"},
+        {"sun", "\u2609"}, {"soleil", "\u2609"},
+        {"star", "\u2605"}, {"toile", "\u2605"},
+        {"mercury", "\u263F"}, {"mercure", "\u263F"},
+        {"venus", "\u2640"}, {"vnus", "\u2640"},
+        {"earth", "\u2295"}, {"terre", "\u2295"}, // 2641 est officiel d'aprs UNICODE mais 2295 est mieux avec STIX...
+        {"mars", "\u2642"}, {"jupiter", "\u2643"},
+        {"saturn", "\u2644"}, {"saturne", "\u2644"},
+        {"uranus", "\u26E2"}, // UNICODE 6.0 draft !
+        {"neptun", "\u2646"}, {"neptune", "\u2646"},
+        {"planck", "\u210F"},
+        {"angstrom", "\u212B"}, {"angstrm", "\u212B"},
+        {"asterisk", "*"}, {"astrisque", "*"}, // \uFF0A ?
+        {"ell", "\u2113"}, {"smalll", "\u2113"}, {"petitl", "\u2113"},
+        // les noms en Xscr viennent de http://www.w3.org/TR/xml-entity-names/
+        {"Ascr", "\uD835\uDC9C"}, {"biga", "\uD835\uDC9C"}, {"granda", "\uD835\uDC9C"}, // 1D49C
+        {"Bscr", "\u212C"}, {"bigb", "\u212C"}, {"grandb", "\u212C"},
+        {"Cscr", "\uD835\uDC9E"}, {"bigc", "\uD835\uDC9E"}, {"grandc", "\uD835\uDC9E"}, // 1D49E
+        {"Dscr", "\uD835\uDC9F"}, {"bigd", "\uD835\uDC9F"}, {"grandd", "\uD835\uDC9F"}, // 1D49F
+        {"Escr", "\u2130"}, {"bige", "\u2130"}, {"grande", "\u2130"},
+        {"Fscr", "\u2131"}, {"bigf", "\u2131"}, {"grandf", "\u2131"},
+        {"Gscr", "\uD835\uDCA2"}, {"bigg", "\uD835\uDCA2"}, {"grandg", "\uD835\uDCA2"}, // 1D4A2
+        {"Hscr", "\u210B"}, {"bigh", "\u210B"}, {"grandh", "\u210B"},
+        {"Iscr", "\u2110"}, {"bigi", "\u2110"}, {"grandi", "\u2110"},
+        {"Jscr", "\uD835\uDCA5"}, {"bigj", "\uD835\uDCA5"}, {"grandj", "\uD835\uDCA5"}, // 1D4A5
+        {"Kscr", "\uD835\uDCA6"}, {"bigk", "\uD835\uDCA6"}, {"grandk", "\uD835\uDCA6"}, // 1D4A6
+        {"Lscr", "\u2112"}, {"bigl", "\u2112"}, {"grandl", "\u2112"},
+        {"Mscr", "\u2133"}, {"bigm", "\u2133"}, {"grandm", "\u2133"},
+        {"Nscr", "\uD835\uDCA9"}, {"bign", "\uD835\uDCA9"}, {"grandn", "\uD835\uDCA9"}, // 1D4A9
+        {"Oscr", "\uD835\uDCAA"}, {"bigo", "\uD835\uDCAA"}, {"grando", "\uD835\uDCAA"}, // 1D4AA
+        {"Pscr", "\uD835\uDCAB"}, {"bigp", "\uD835\uDCAB"}, {"grandp", "\uD835\uDCAB"}, // 1D4AB
+        {"Qscr", "\uD835\uDCAC"}, {"bigq", "\uD835\uDCAC"}, {"grandq", "\uD835\uDCAC"}, // 1D4AC
+        {"Rscr", "\u211B"}, {"bigr", "\u211B"}, {"grandr", "\u211B"},
+        {"Sscr", "\uD835\uDCAE"}, {"bigs", "\uD835\uDCAE"}, {"grands", "\uD835\uDCAE"}, // 1D4AE
+        {"Tscr", "\uD835\uDCAF"}, {"bigt", "\uD835\uDCAF"}, {"grandt", "\uD835\uDCAF"}, // 1D4AF
+        {"Uscr", "\uD835\uDCB0"}, {"bigu", "\uD835\uDCB0"}, {"grandu", "\uD835\uDCB0"}, // 1D4B0
+        {"Vscr", "\uD835\uDCB1"}, {"bigv", "\uD835\uDCB1"}, {"grandv", "\uD835\uDCB1"}, // 1D4B1
+        {"Wscr", "\uD835\uDCB2"}, {"bigw", "\uD835\uDCB2"}, {"grandw", "\uD835\uDCB2"}, // 1D4B2
+        {"Xscr", "\uD835\uDCB3"}, {"bigx", "\uD835\uDCB3"}, {"grandx", "\uD835\uDCB3"}, // 1D4B3
+        {"Yscr", "\uD835\uDCB4"}, {"bigy", "\uD835\uDCB4"}, {"grandy", "\uD835\uDCB4"}, // 1D4B4
+        {"Zscr", "\uD835\uDCB5"}, {"bigz", "\uD835\uDCB5"}, {"grandz", "\uD835\uDCB5"} // 1D4B5
+    };
     
     // fonctions qui peuvent se passer de parenthses quand il n'y a qu'un argument simple
     private static final String[] fctnopar = {"sin", "cos", "tan", "acos", "asin", "atan"};
@@ -386,6 +432,8 @@ public class StringMathBuilder {
         boolean unites;
         public JEQFonction(final JEQ nom, final Vector<JEQ> arguments) {
             this.nom = nom;
+            if (nom instanceof JEQVariable && ((JEQVariable)nom).nom.matches("[0-9]+"))
+                ((JEQVariable)nom).nom = ((JEQVariable)nom).nom + "?"; // un nom de fonction avec que des chiffres ?!?
             vp = arguments;
             unites = ("unit".equals(getNomFct()) || "unit".equals(getNomFct()));
             if (unites)
@@ -423,24 +471,37 @@ public class StringMathBuilder {
                 p4 = null;
             String nomfct = getNomFct();
             MathElement me;
-            if ("sqrt".equals(nomfct) || "racine".equals(nomfct)) {
+            if ("sqrt".equals(nomfct) || ("racine".equals(nomfct) && (p1 == null || p2 == null))) {
                 me = new MathSqrt();
-                me.addMathElement(p1.versMathML());
+                me.addMathElement(elemOrQuestion(p1));
+            } else if ("racine".equals(nomfct) || "root".equals(nomfct)) {
+                me = new MathRoot();
+                me.addMathElement(elemOrQuestion(p1));
+                me.addMathElement(elemOrQuestion(p2));
             } else if ("exp".equals(nomfct)) {
                 me = new MathSup();
                 final MathIdentifier mi = new MathIdentifier();
                 mi.addText("e");
                 me.addMathElement(mi);
-                me.addMathElement(p1.versMathML());
+                me.addMathElement(elemOrQuestion(p1));
             } else if ("abs".equals(nomfct)) {
                 me = new MathRow();
                 MathOperator mo = new MathOperator();
                 mo.addText("|");
                 me.addMathElement(mo);
-                me.addMathElement(p1.versMathML());
+                me.addMathElement(elemOrQuestion(p1));
                 mo = new MathOperator();
                 mo.addText("|");
                 me.addMathElement(mo);
+            } else if ("norm".equals(nomfct) || "norme".equals(nomfct)) {
+                me = new MathRow();
+                MathOperator mo = new MathOperator();
+                mo.addText("\u2016");
+                me.addMathElement(mo);
+                me.addMathElement(elemOrQuestion(p1));
+                mo = new MathOperator();
+                mo.addText("\u2016");
+                me.addMathElement(mo);
             } else if ("fact".equals(nomfct) || "factorielle".equals(nomfct)
                         || "factorial".equals(nomfct)) {
                 me = new MathRow();
@@ -524,7 +585,13 @@ public class StringMathBuilder {
                 final MathOver mover = new MathOver();
                 mover.setAccent(true);
                 mover.addMathElement(elemOrQuestion(p1));
-                mover.addMathElement(elemOrQuestion(p2));
+                if (p2 instanceof JEQOperation && ((JEQOperation)p2).op == '\u223C') {
+                    final MathOperator mo = new MathOperator();
+                    mo.setStretchy(true);
+                    mo.addText("\u223C");
+                    mover.addMathElement(mo);
+                } else
+                    mover.addMathElement(elemOrQuestion(p2));
                 me = mover;
             } else if ("matrix".equals(nomfct) || "matrice".equals(nomfct)) {
                 me = new MathRow();
@@ -584,20 +651,20 @@ public class StringMathBuilder {
                 final MathOver mover = new MathOver();
                 mover.setAccent(true);
                 mover.addMathElement(elemOrQuestion(p1));
-                MathText mt = new MathText();
+                MathOperator mop = new MathOperator();
                 if (p2 instanceof JEQNombre) {
                     try {
                         final int n = Integer.parseInt(((JEQNombre)p2).valeur);
                         String spts = "";
                         for (int i=0; i<n; i++)
                             spts = spts + '.';
-                        mt.addText(spts);
+                        mop.addText(spts);
                     } catch (final NumberFormatException ex) {
-                        mt.addText("?");
+                        mop.addText("?");
                     }
                 } else
-                    mt.addText("?");
-                mover.addMathElement(mt);
+                    mop.addText("?");
+                mover.addMathElement(mop);
                 me = mover;
             } else if ("unit".equals(nomfct) || "unit".equals(nomfct)) {
                 final MathRow mrow = new MathRow();
@@ -607,14 +674,47 @@ public class StringMathBuilder {
                 mrow.addMathElement(mo);
                 mrow.addMathElement(elemOrQuestion(p2));
                 return mrow;
+            } else if ("moyenne".equals(nomfct) || "mean".equals(nomfct)) {
+                me = new MathRow();
+                MathOperator mo = new MathOperator();
+                mo.addText("\u2329");
+                me.addMathElement(mo);
+                me.addMathElement(elemOrQuestion(p1));
+                mo = new MathOperator();
+                mo.addText("\u232A");
+                me.addMathElement(mo);
+            } else if ("vecteur".equals(nomfct) || "vector".equals(nomfct)) {
+                // les vecteurs sont affichs en gras quand c'est possible
+                final MathElement mp1 = elemOrQuestion(p1);
+                if (mp1 instanceof MathIdentifier) {
+                    ((MathIdentifier)mp1).setMathvariant("bold");
+                    me = mp1;
+                } else if (mp1 instanceof MathSub && mp1.getMathElement(0) instanceof MathIdentifier) {
+                    ((MathIdentifier)(mp1.getMathElement(0))).setMathvariant("bold");
+                    me = mp1;
+                } else {
+                    final MathOver mover = new MathOver();
+                    mover.setAccent(true);
+                    mover.addMathElement(elemOrQuestion(p1));
+                    final MathOperator mo = new MathOperator();
+                    mo.setStretchy(true);
+                    mo.addText("\u2192");
+                    mover.addMathElement(mo);
+                    me = mover;
+                }
             } else {
                 me = new MathRow();
                 boolean par = true;
                 if (nom instanceof JEQVariable) {
                     MathText mt = new MathText();
-                    for (final String[] element : symbols)
-                        if (element[0].equals(nomfct)) {
-                            nomfct = element[1];
+                    for (final String[] tsymbole : symboles_id)
+                        if (tsymbole[0].equals(nomfct)) {
+                            nomfct = tsymbole[1];
+                            break;
+                        }
+                    for (final String[] tsymbole : symboles_droits)
+                        if (tsymbole[0].equals(nomfct)) {
+                            nomfct = tsymbole[1];
                             break;
                         }
                     mt.addText(nomfct);
@@ -632,7 +732,7 @@ public class StringMathBuilder {
                     mo.addText("("); // \u2061 = &ApplyFunction; serait mieux mais pas reconnu
                     me.addMathElement(mo);
                 }
-                me.addMathElement(p1.versMathML());
+                me.addMathElement(elemOrQuestion(p1));
                 for (int i=1; i<vp.size(); i++) { // ATTENTION, LA BOUCLE COMMENCE A 1
                     final MathOperator mo = new MathOperator();
                     mo.addText(";");
@@ -776,20 +876,47 @@ public class StringMathBuilder {
                     mrow.addMathElement(me2);
                 }
                 return mrow;
-            } else {
+            } else if (op == '+') {
                 final MathRow mrow = new MathRow();
                 if (p1 != null)
                     mrow.addMathElement(p1.versMathML());
                 final MathOperator mo = new MathOperator();
+                mo.addText("+");
+                mrow.addMathElement(mo);
+                if (p2 != null) {
+                    MathElement me2 = p2.versMathML();
+                    if (me2 instanceof MathRow && me2.getMathElementCount() > 0) {
+                        MathElement me2b = me2;
+                        while (me2b instanceof MathRow && me2b.getMathElementCount() > 0)
+                            me2b = me2b.getMathElement(0);
+                        if ("-".equals(me2b.getText()))
+                            me2 = ajPar(me2);
+                    }
+                    mrow.addMathElement(me2);
+                }
+                return mrow;
+            } else {
+                final MathRow mrow = new MathRow();
+                if (p1 != null) {
+                    MathElement me1 = p1.versMathML();
+                    if (op == '\u2227' && p1 instanceof JEQOperation && "+-".indexOf(((JEQOperation)p1).op) != -1)
+                        me1 = ajPar(me1);
+                    mrow.addMathElement(me1);
+                }
+                final MathOperator mo = new MathOperator();
                 mo.addText(Character.toString(op));
-                if ("=\u2260\u2248\u2264\u2265\u226A\u226B".indexOf(op) != -1) {
+                if ("=\u2260\u2248\u223C\u2261\u2264\u2265\u226A\u226B\u221D".indexOf(op) != -1) {
                     // espace autour des oprateurs d'galit
                     mo.setLspace(0.5);
                     mo.setRspace(0.5);
                 }
                 mrow.addMathElement(mo);
-                if (p2 != null)
-                    mrow.addMathElement(p2.versMathML());
+                if (p2 != null) {
+                    MathElement me2 = p2.versMathML();
+                    if (op == '\u2227' && p2 instanceof JEQOperation && "+-".indexOf(((JEQOperation)p2).op) != -1)
+                        me2 = ajPar(me2);
+                    mrow.addMathElement(me2);
+                }
                 return mrow;
             }
         }
@@ -824,7 +951,7 @@ public class StringMathBuilder {
         String nom;
         boolean unites;
         public JEQVariable(final String nom) {
-            this.nom = nom;
+            this.nom = nom.trim();
             unites = false;
         }
         public void setUnites() {
@@ -843,16 +970,29 @@ public class StringMathBuilder {
                 return mo;
             } else {
                 String s = nom;
-                for (final String[] element : symbols)
-                    if (element[0].equals(s)) {
-                        s = element[1];
+                boolean droit = unites;  // les units ne doivent pas tre en italique
+                for (final String[] tsymbole : symboles_id)
+                    if (tsymbole[0].equals(s)) {
+                        s = tsymbole[1];
                         break;
                     }
-                if (s.indexOf(',') != -1 || s.indexOf('.') != -1)
-                    s = "?"; // les ',' et les '.' sont interdits dans les noms de variables
+                for (final String[] tsymbole : symboles_droits)
+                    if (tsymbole[0].equals(s)) {
+                        s = tsymbole[1];
+                        droit = true;
+                        break;
+                    }
+                if (s.indexOf(',') != -1 || s.indexOf('.') != -1 || s.indexOf('(') != -1 || s.indexOf(')') != -1)
+                    s = "?"; // les ',' les '.' et les parenthses sont interdits dans les noms de variables
+                if (s.indexOf(' ') != -1)
+                    s = s.replace(" ", "?"); // les espaces aussi !
+                if (s.indexOf('\u00A0') != -1) // pour les petits malins
+                    s = s.replace("\u00A0", "?");
+                if (s.matches("[0-9]+[a-zA-Z]+"))
+                    s = "?"; // il manque l'oprateur *
                 MathElement me;
-                if (unites)
-                    me = new MathText();  // les units ne doivent pas tre en italique
+                if (droit)
+                    me = new MathText();
                 else
                     me = new MathIdentifier();
                 me.addText(s);
