@@ -68,61 +68,42 @@ public class MathOperator extends MathElement
      * Paints a delimitier
      *
      * @param g The graphics context to use for painting 
-   * @param posX The first left position for painting 
-   * @param posY The position of the baseline
+     * @param posX The first left position for painting 
+     * @param posY The position of the baseline
      * @param upperSymbol The symbol for upper edge
      * @param middleSymbol The symbol for middle part
      * @param lowerSymbol The symbol for lower edge
      */
     private void paintDelimiter(final Graphics g, final int posX, final int posY,
-                                                            final char upperSymbol, final char middleSymbol,
-                                                            final char lowerSymbol)
+        final char upperSymbol, final char middleSymbol, final char lowerSymbol)
     {
-        final int height = getParent().getHeight(false);
-        final int middle = posY - getMiddleShift();
-        final GlyphVector gv;
+        final int height = getExprHeight();
+        final int aheight = getExprAscentHeight();
         final Graphics2D g2d = (Graphics2D) g;
-
-        // RenderingHints hints = new RenderingHints(
-        // RenderingHints.KEY_TEXT_ANTIALIASING,
-        // RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
-        // g2d.setRenderingHints(hints);
-        g2d.setColor(Color.black);
-
-        final int fontascent = getSymbolFontMetrics().getAscent();
-        final int countparts = height / fontascent + 1;
-        final int halfcount = countparts / 2;
-
-        if (countparts % 2 == 0)
-        {
-            for (int i = 1; i < halfcount; i++)
-            {
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX,
-                                                        middle - (fontascent - 1) * (i - 1));
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX,
-                                                        middle + (fontascent - 1) * i);
-            }
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, upperSymbol), posX,
-                                                    middle - (fontascent - 1) * (halfcount - 1));
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, lowerSymbol), posX,
-                                                    middle + (fontascent - 1) * halfcount);
-        }
-        else
-        {
-            for (int i = 1; i < halfcount; i++)
-            {
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX,
-                                                        posY - (fontascent - 1) * i);
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX,
-                                                        posY + (fontascent - 1) * i);
-            }
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX,
-                                                    posY);
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, upperSymbol), posX,
-                                                    posY - (fontascent - 1) * halfcount);
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, lowerSymbol), posX,
-                                                    posY + (fontascent - 1) * halfcount);
+        final GlyphVector gvtop = getFont().createGlyphVector(g2d.getFontRenderContext(), new char[]{ upperSymbol });
+        final GlyphVector gvmid = getFont().createGlyphVector(g2d.getFontRenderContext(), new char[]{ middleSymbol });
+        final GlyphVector gvbot = getFont().createGlyphVector(g2d.getFontRenderContext(), new char[]{ lowerSymbol });
+        
+        final double gvtopHeight = gvtop.getVisualBounds().getHeight() - 1;
+        final double gvmidHeight = gvmid.getVisualBounds().getHeight() - 1;
+        final double gvbotHeight = gvbot.getVisualBounds().getHeight() - 1;
+        
+        if (height - gvtopHeight - gvbotHeight > 0) {
+            final double goal = height - gvtopHeight - gvbotHeight + 3;
+            double sy = goal / gvmidHeight;
+            AffineTransform transform = AffineTransform.getScaleInstance(1, sy);
+            gvmid.setGlyphTransform(0, transform);
+            for (int i=0; i<2; i++)
+                if (gvmid.getVisualBounds().getHeight() != goal) {
+                    sy = sy * goal / gvmid.getVisualBounds().getHeight();
+                    transform = AffineTransform.getScaleInstance(1, sy);
+                    gvmid.setGlyphTransform(0, transform);
+                } else
+                    break;
+            g2d.drawGlyphVector(gvmid, posX, posY - aheight + (int)Math.round(gvtopHeight) - (int)Math.round(gvmid.getVisualBounds().getY()) - 1);
         }
+        g2d.drawGlyphVector(gvtop, posX, posY - aheight + (int)Math.round(gvtopHeight) - (int)Math.round(gvtopHeight + gvtop.getVisualBounds().getY()));
+        g2d.drawGlyphVector(gvbot, posX, posY + height - aheight - (int)Math.round(gvbotHeight + gvbot.getVisualBounds().getY()));
     }
 
   /**
@@ -141,50 +122,26 @@ public class MathOperator extends MathElement
                                                                      final char upperSymbol, final char middleSymbol,
                                                                      final char connectSymbol, final char lowerSymbol)
     {
-        final int height = getParent().getHeight(false);
+        final int height = getExprHeight();
         final int middle = posY - getMiddleShift();
-        final GlyphVector gv;
-        final Graphics2D g2d = (Graphics2D) g;
 
-        // RenderingHints hints = new RenderingHints(
-        // RenderingHints.KEY_TEXT_ANTIALIASING,
-        // RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
-        // g2d.setRenderingHints(hints);
-        g2d.setColor(Color.black);
+        g.setColor(Color.black);
+        g.setFont(getFont());
 
-        final int fontascent = getSymbolFontMetrics().getAscent();
+        final int fontascent = getFontMetrics().getAscent();
         int countparts = height / fontascent + 1;
         final int halfcount = countparts / 2;
 
         if (countparts % 2 == 0)
             countparts++;
 
-        for (int i = 1; i < halfcount; i++)
-        {
-            if (getMathBase().windaube &&    //  MS-Symbol bug ?!?
-                            (getFontSize() == 8 || getFontSize() == 9
-                            || getFontSize() == 10 || getFontSize() == 13
-                            || getFontSize() == 14 || getFontSize() == 18
-                            || getFontSize() == 22 || getFontSize() == 32))
-            {
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, connectSymbol),
-                                                        posX + 1, posY - (fontascent - 1) * i);
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, connectSymbol),
-                                                        posX + 1, posY + (fontascent - 1) * i);
-            }
-            else
-            {
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, connectSymbol), posX,
-                                                        posY - (fontascent - 1) * i);
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, connectSymbol), posX,
-                                                        posY + (fontascent - 1) * i);
-            }
+        for (int i = 1; i < halfcount; i++) {
+            g.drawString(""+connectSymbol, posX, posY - (fontascent - 1) * i);
+            g.drawString(""+connectSymbol, posX, posY + (fontascent - 1) * i);
         }
-        g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX, posY);
-        g2d.drawGlyphVector(getSymbolGlyphVector(g2d, upperSymbol), posX,
-                                                posY - (fontascent - 1) * halfcount);
-        g2d.drawGlyphVector(getSymbolGlyphVector(g2d, lowerSymbol), posX,
-                                                posY + (fontascent - 1) * halfcount);
+        g.drawString(""+middleSymbol, posX, posY);
+        g.drawString(""+upperSymbol, posX, posY - (fontascent - 1) * halfcount);
+        g.drawString(""+lowerSymbol, posX, posY + (fontascent - 1) * halfcount);
     }
 
   /**
@@ -207,14 +164,10 @@ public class MathOperator extends MathElement
     {
         final int height = getParent().getWidth(true);
         final int middle = posY - getMiddleShift();
-        final GlyphVector gv;
         final Graphics2D g2d = (Graphics2D) g;
 
-        // RenderingHints hints = new RenderingHints(
-        // RenderingHints.KEY_TEXT_ANTIALIASING,
-        // RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
-        // g2d.setRenderingHints(hints);
         g2d.setColor(Color.black);
+        g2d.setFont(getFont());
         final AffineTransform at = new AffineTransform();
 
         at.translate(posX, posY);
@@ -222,7 +175,7 @@ public class MathOperator extends MathElement
         at.translate(-posX, -middle);
         g2d.setTransform(at);
 
-        final int fontascent = getSymbolFontMetrics().getAscent();
+        final int fontascent = getFontMetrics().getAscent();
         int countparts = height / fontascent + 1;
         final int halfcount = countparts / 2;
 
@@ -231,16 +184,12 @@ public class MathOperator extends MathElement
 
         for (int i = 1; i < halfcount; i++)
         {
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, connectSymbol), posX,
-                                                    posY - (fontascent - 1) * i);
-            g2d.drawGlyphVector(getSymbolGlyphVector(g2d, connectSymbol), posX,
-                                                    posY + (fontascent - 1) * i);
+            g2d.drawString(""+connectSymbol, posX, posY - (fontascent - 1) * i);
+            g2d.drawString(""+connectSymbol, posX, posY + (fontascent - 1) * i);
         }
-        g2d.drawGlyphVector(getSymbolGlyphVector(g2d, middleSymbol), posX, posY);
-        g2d.drawGlyphVector(getSymbolGlyphVector(g2d, upperSymbol), posX,
-                                                posY - (fontascent - 1) * halfcount);
-        g2d.drawGlyphVector(getSymbolGlyphVector(g2d, lowerSymbol), posX,
-                                                posY + (fontascent - 1) * halfcount);
+        g2d.drawString(""+middleSymbol, posX, posY);
+        g2d.drawString(""+upperSymbol, posX, posY - (fontascent - 1) * halfcount);
+        g2d.drawString(""+lowerSymbol, posX, posY + (fontascent - 1) * halfcount);
 
         g2d.setTransform(new AffineTransform());
     }
@@ -266,33 +215,20 @@ public class MathOperator extends MathElement
             final int ascent = getParent().getAscentHeight(false);
             final int descent = getParent().getDescentHeight(false);
             final int height = ascent + descent;
-            final int width = (int) (height * 0.2);
-            final int halfwidth = width / 2;
-            final int middle = posY - getMiddleShift();
-            // int middle = posY+descent-(ascent+descent)/2;
-            final int fontascent = getSymbolFontMetrics().getAscent();
-            final int countparts = height / fontascent + 1;
-
-            // double dcountparts = (double)(((double)height/(double)fontascent)+1);
-            // System.out.println("Height = " + height + ", fontmetrics =  " +
-            // base.getFontMetrics(fontsize).getHeight() + ", Fontascent = " + fontascent);
-            // g.setColor(Color.black);
+            //final int width = (int) (height * 0.2);
+            //final int halfwidth = width / 2;
+            //final int middle = posY - getMiddleShift();
+            //final int fontascent = getFontMetrics().getAscent();
+            //final int countparts = height / fontascent + 1;
 
             if (getText().equals("("))
             {
-                // System.out.println("countparts = " + countparts);
-                // g.setColor(Color.green);
-                // g.drawLine(posX,middle,posX+10,middle);
-                // g.setColor(Color.black);
-
-                // if( countparts == 1 )
                 if (height <= getFontMetrics().getHeight())
                 {
                     g.setFont(getFont());
                     g.drawString(getText(), posX, posY);
                 }
                 else
-                    //paintDelimiter(g, posX, posY, 167, 168, 169); 
                     paintDelimiter(g, posX, posY, '\u239B', '\u239C', '\u239D');
             }
             else if (getText().equals(")"))
@@ -303,7 +239,6 @@ public class MathOperator extends MathElement
                     g.drawString(getText(), posX, posY);
                 }
                 else
-                    //paintDelimiter(g, posX, posY, 183, 184, 185); 
                     paintDelimiter(g, posX, posY, '\u239E', '\u239F', '\u23A0');
             }
             else if (getText().equals("["))
@@ -314,7 +249,6 @@ public class MathOperator extends MathElement
                     g.drawString(getText(), posX, posY);
                 }
                 else
-                    //paintDelimiter(g, posX, posY, 170, 171, 172); 
                     paintDelimiter(g, posX, posY, '\u23A1', '\u23A2', '\u23A3');
             }
             else if (getText().equals("]"))
@@ -325,20 +259,17 @@ public class MathOperator extends MathElement
                     g.drawString(getText(), posX, posY);
                 }
                 else
-                    //paintDelimiter(g, posX, posY, 186, 187, 188); 
                     paintDelimiter(g, posX, posY, '\u23A4', '\u23A5', '\u23A6');
 
             }
             else if (getText().equals("{"))
             {
-                // System.out.println("countparts = " + countparts);
                 if (height <= getFontMetrics().getHeight())
                 {
                     g.setFont(getFont());
                     g.drawString(getText(), posX, posY);
                 }
                 else
-                    //paintCurlyDelimiter(g, posX, posY, 173, 174, 176, 175); 
                     paintCurlyDelimiter(g, posX, posY, '\u23A7', '\u23A8', '\u23AA', '\u23A9');
             }
             else if (getText().equals("}"))
@@ -349,7 +280,6 @@ public class MathOperator extends MathElement
                     g.drawString(getText(), posX, posY);
                 }
                 else
-                    //paintCurlyDelimiter(g, posX, posY, 189, 190, 176, 191); 
                     paintCurlyDelimiter(g, posX, posY, '\u23AB', '\u23AC', '\u23AA', '\u23AD');
             }
             else if (getText().equals("|"))
@@ -358,67 +288,37 @@ public class MathOperator extends MathElement
             }
             else if (getText().equals("\u222B"))
             {
-                // if( height*2 <= base.getFontMetrics(fontsize).getHeight() )
-                // {
-                // System.out.println("Fritz");
-                /*Graphics2D g2d = (Graphics2D) g;
-
-                g2d.setColor(Color.black);
-
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, 180), posX, middle);
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, 182), posX,
-                                                        middle + fontascent - 1);*/
-                // g2d.drawGlyphVector(base.getSymbolGlyphVector(
-                // g2d,fontsize*2,179), posX, posY);
-                // }
-                // else
-                // paintDelimiter(g, posX, posY, 180, 181, 182);
-                
-                /*if (height <= getFontMetrics().getHeight())
-                {
-                    g.setFont(getMathBase().getSymbolFont(getFontSize()*2));
-                    g.drawString(getText(), posX, posY);
-                }
-                else
-                    paintDelimiter(g, posX, posY, '\u2320', '\u23AE', '\u2321');*/
-                final Graphics2D g2d = (Graphics2D) g;
-                g2d.setColor(Color.black);
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, '\u2320'), posX, middle);
-                g2d.drawGlyphVector(getSymbolGlyphVector(g2d, '\u2321'), posX, middle + fontascent - 1);
+                paintDelimiter(g, posX, posY, '\u2320', '\u23AE', '\u2321');
             }
         }
         else if (getText().length() == 1
                          && "\uFE37\uFE38".indexOf(getText().charAt(0)) >= 0)
         {
-            final int width = getParent().getWidth(true);
-            final int halfwidth = width / 2;
-            final int height = Math.max((int) (width * 0.2), 2);
-            final int halfheight = height / 2;
-            final int middle = posX;
+            //final int width = getParent().getWidth(true);
+            //final int halfwidth = width / 2;
+            //final int height = Math.max((int) (width * 0.2), 2);
+            //final int halfheight = height / 2;
 
             if (getText().equals("\uFE37"))
-                //paintCurlyDelimiterHorizontal(g, posX, posY, 173, 174, 176, 175);
                 paintCurlyDelimiterHorizontal(g, posX, posY, '\u23A7', '\u23A8', '\u23AA', '\u23A9');
             else if (getText().equals("\uFE38"))
-                //paintCurlyDelimiterHorizontal(g, posX, posY, 189, 190, 176, 191);
                 paintCurlyDelimiterHorizontal(g, posX, posY, '\u23AB', '\u23AC', '\u23AA', '\u23AD');
         }
         else if (getText().length() == 1
-                         && "\u2211\u220f".indexOf(getText().charAt(0)) >= 0
+                         && "\u2211\u220F".indexOf(getText().charAt(0)) >= 0
                          && stretchy)
         {
-            final Graphics2D g2d = (Graphics2D) g;
-
-            g2d.setColor(Color.black);
-
+            g.setColor(Color.black);
+            if (getExprHeight() > getFontMetrics().getHeight()) {
+                final Font bigfont = getMathBase().getFont(getFontSize() * 2);
+                g.setFont(bigfont);
+            } else
+                g.setFont(getFont());
             if ("\u2211".indexOf(getText().charAt(0)) >= 0)
-                g2d.drawGlyphVector(getMathBase().getSymbolGlyphVector(g2d, getFontSize()
-                                //* 2, 166), posX, posY);
-                                * 2, '\u2211'), posX, posY);
+                g.drawString("\u2211", posX, posY);
             else
-                g2d.drawGlyphVector(getMathBase().getSymbolGlyphVector(g2d, getFontSize()
-                                //* 2, 150), posX, posY);
-                                * 2, '\u220F'), posX, posY);
+                g.drawString("\u220F", posX, posY);
+            g.setFont(getFont());
         }
         else if (getText().equals("\u00AF") && stretchy) // over bar
         {
@@ -427,21 +327,63 @@ public class MathOperator extends MathElement
         }
         else if (getText().equals("^") && stretchy) // hat
         {
-            final int width = getParent().getWidth(false) - 2;
+            final int width = getParent().getWidth(false) - 3;
             g.drawLine(posX, posY, posX + width/2, posY - 3);
             g.drawLine(posX + width/2, posY - 3, posX + width, posY);
         }
+        else if (".".equals(getText()) || "..".equals(getText()) || "...".equals(getText()))
+        {
+            g.setFont(getFont());
+            g.drawString(getText(), posX + 1, posY);
+        }
         else
         {
-            final String s = getText();
-            Font font = getFont();
-            final int upto = font.canDisplayUpTo(s);
-            if (upto != -1 && upto != s.length())
-                font = new Font(getMathBase().findFont(s, font), Font.PLAIN, getFontSize());
-            g.setFont(font);
-            g.drawString(s, posX, posY);
+            g.setFont(getFont());
+            g.drawString(getText(), posX, posY);
         }
     }
+    
+    private MathElement getExpr() {
+        if (getText().equals("\u222B") || "\u2211\u220F".contains(getText())) {
+            MathRow row;
+            if (getParent() instanceof MathRow)
+                row = (MathRow)getParent();
+            else
+                row = (MathRow)(getParent().getParent());
+            // test intgrale/sommation/produit double
+            final MathElement contenu = row.getMathElement(1);
+            if (!(contenu instanceof MathRow && contenu.getMathElementCount() > 0))
+                return(contenu);
+            final MathElement interieur1 = contenu.getMathElement(0);
+            if (!(interieur1 instanceof MathRow && interieur1.getMathElementCount() > 0))
+                return(contenu);
+            final MathElement interieur2 = interieur1.getMathElement(0);
+            if (!(interieur2 instanceof MathUnderOver || interieur2 instanceof MathUnder ||
+                    interieur2 instanceof MathOver || interieur2 instanceof MathOperator))
+                return(contenu);
+            MathElement op;
+            if (interieur2 instanceof MathUnderOver || interieur2 instanceof MathUnder || interieur2 instanceof MathOver)
+                op = interieur2.getMathElement(0);
+            else
+                op = interieur2;
+            if (!("\u222B".equals(op.getText()) || "\u2211\u220F".contains(op.getText())))
+                return(contenu);
+            return(interieur1.getMathElement(1));
+        } else
+            return(getParent());
+    }
+    
+    private int getExprHeight() {
+        return(getExpr().getHeight(false));
+    }
+
+    private int getExprAscentHeight() {
+        return(getExpr().getAscentHeight(false));
+    }
+
+    private int getExprDescentHeight() {
+        return(getExpr().getDescentHeight(false));
+    }
 
   /**
    * Return the current width of this element 
@@ -470,9 +412,14 @@ public class MathOperator extends MathElement
                 return 5 + totalspace;
             else if ("\uFE37\uFE38".indexOf(firstchar) >= 0)
                 return 1 + totalspace;
-            else if ("\u2211\u222B\u220f".indexOf(firstchar) >= 0)
+            else if ("\u222B".indexOf(firstchar) >= 0)
                 return getMathBase().getFontMetrics(getFontSize()*2).stringWidth(getText()) + totalspace;
-            else if ("^\u00AF".indexOf(firstchar) >= 0 && stretchy && dynamicParts)
+            else if ("\u2211\u220F".indexOf(firstchar) >= 0) {
+                if (getExprHeight() > getFontMetrics().getHeight())
+                    return getMathBase().getFontMetrics(getFontSize()*2).stringWidth(getText()) + totalspace;
+                else
+                    return getFontMetrics().stringWidth(getText()) + totalspace;
+            } else if ("^\u00AF".indexOf(firstchar) >= 0 && stretchy && dynamicParts)
                 return getParent().getWidth(false) - 2;
         }
         return getFontMetrics().stringWidth(getText()) + totalspace;
@@ -504,13 +451,15 @@ public class MathOperator extends MathElement
     @Override
     public int getAscentHeight(final boolean dynamicParts)
     {
-        if (getText().length() == 1
-                        && "[{()}]".indexOf(getText().charAt(0)) >= 0)
-        {
+        if (getText().length() == 1 && "[()]\u222B".indexOf(getText().charAt(0)) >= 0) {
             if (!dynamicParts || !stretchy)
                 return getFontMetrics().getAscent();
-            final int ascent = getSymbolFontMetrics().getAscent();
-            final int countparts = getParent().getHeight(false) / ascent + 1;
+            return(getExprAscentHeight() + 1);
+        } else if (getText().length() == 1 && "{}".indexOf(getText().charAt(0)) >= 0) {
+            if (!dynamicParts || !stretchy)
+                return getFontMetrics().getAscent();
+            final int ascent = getFontMetrics().getAscent();
+            final int countparts = getExprHeight() / ascent + 1;
 
             if ("{}".indexOf(getText().charAt(0)) >= 0)
                 if (countparts % 2 == 0)
@@ -518,19 +467,21 @@ public class MathOperator extends MathElement
             return (int) (countparts * ascent * 0.5 + getMiddleShift());
         }
         else if (getText().length() == 1
-                         && "\u222B".indexOf(getText().charAt(0)) >= 0)
-        {
-            final int ascent = getSymbolFontMetrics().getAscent();
-
-            return ascent + getMiddleShift();
-        }
-        else if (getText().length() == 1
-                         && "\u2211\u220f".indexOf(getText().charAt(0)) >= 0
-                         && stretchy)
-            return getMathBase().getFontMetrics(getFontSize() * 2).getAscent();
-        else if (getText().length() == 1
+                         && "\u2211\u220F".indexOf(getText().charAt(0)) >= 0
+                         && stretchy) {
+            if (getExprHeight() > getFontMetrics().getHeight())
+                return getMathBase().getFontMetrics(getFontSize() * 2).getAscent();
+            else
+                return getFontMetrics().getAscent();
+        } else if (getText().length() == 1
                          && "\uFE37\uFE38".indexOf(getText().charAt(0)) >= 0)
             return 0;
+        else if ("\u00AF".equals(getText()) && stretchy)
+            return(3);
+        else if ("\u223C".equals(getText()) && stretchy)
+            return(getFontMetrics().getAscent()/2);
+        else if (".".equals(getText()) || "..".equals(getText()) || "...".equals(getText()))
+            return(getFontMetrics().getAscent()/2);
         else
             return getFontMetrics().getAscent();
     }
@@ -547,13 +498,15 @@ public class MathOperator extends MathElement
     @Override
     public int getDescentHeight(final boolean dynamicParts)
     {
-        if (getText().length() == 1
-                        && "[{()}]".indexOf(getText().charAt(0)) >= 0)
-        {
+        if (getText().length() == 1 && "[()]\u222B".indexOf(getText().charAt(0)) >= 0) {
             if (!dynamicParts || !stretchy)
                 return getFontMetrics().getDescent();
-            final int ascent = getSymbolFontMetrics().getAscent();
-            final int countparts = getParent().getHeight(false) / ascent + 1;
+            return(getExprDescentHeight() + 1);
+        } else if (getText().length() == 1 && "{}".indexOf(getText().charAt(0)) >= 0) {
+            if (!dynamicParts || !stretchy)
+                return getFontMetrics().getDescent();
+            final int ascent = getFontMetrics().getAscent();
+            final int countparts = getExprHeight() / ascent + 1;
 
             if ("{}".indexOf(getText().charAt(0)) >= 0)
                 if (countparts % 2 == 0)
@@ -561,19 +514,15 @@ public class MathOperator extends MathElement
             return (int) (countparts * ascent * 0.5 - getMiddleShift());
         }
         else if (getText().length() == 1
-                         && "\u222B".indexOf(getText().charAt(0)) >= 0)
-        {
-            final int ascent = getSymbolFontMetrics().getAscent();
-
-            return ascent - getMiddleShift();
-        }
-        else if (getText().length() == 1
-                         && "\u2211\u220f".indexOf(getText().charAt(0)) >= 0
-                         && stretchy)
-            return getMathBase().getFontMetrics(getFontSize() * 2).getDescent();
-        else if (getText().length() == 1
+                         && "\u2211\u220F".indexOf(getText().charAt(0)) >= 0
+                         && stretchy) {
+            if (getExprHeight() > getFontMetrics().getHeight())
+                return getMathBase().getFontMetrics(getFontSize() * 2).getDescent();
+            else
+                return getFontMetrics().getDescent();
+        } else if (getText().length() == 1
                          && "\uFE37\uFE38".indexOf(getText().charAt(0)) >= 0)
-            return getSymbolFontMetrics().stringWidth("}");
+            return getFontMetrics().stringWidth("}");
         else
             return getFontMetrics().getDescent();
     }
